unit AsmStatus;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, PascalDefs, AsmDefs, System.UITypes;


procedure RunAssembler(f: string);
Procedure MakeError(T: word);
Procedure ProcessLinePass1;
procedure Status(s: string);
function ParseItem(Terminator: TSpaceTerminator) : boolean;
procedure Link;
procedure LinkFile (Filename: string);
function Reconstruct: string;
function TryExpression(var Value: longint): boolean;


var
  CurrentFile:  string;
  RSLFIlename: string;

implementation


uses Compiler, StringConversion, MainForm, ShellApi, HexFile, RpnUtils;

const
  CurrentRSLVersion = 1;

type
  TExpressionTypes=(ExUnknown,ExRegister8,ExExpression,ExIndexedZ,ExIndexedY);
  TMemoryAreas=(CodeActive,DataActive,ABSActive,ConstActive,E2Active);

  TID = record
     Value: longint;
     Extern: boolean;
     Referenced,
     DynamicVar: boolean;
     end;

  TOBJRec = record
             Address1,Address2,Address3: byte;
             Code1,Code2: byte;
             SourceFileNumber: byte;
             LineNumber1,LineNumber2: byte;
             Macro: byte;
           end;
  TOBJHeader = record
                FileNames: longint;
                Records1,Records2,Records3,Records4: byte;
                Bytes: byte;
                NumberNames: byte;
                AVR: array[0..15] of byte;
              end;
{$A-}
  TRelocationInfo = record
                      Address,Index: longint;
                      Flags: word;
                    end;

  TSection = record
               name: array[0..7] of char;
               PhysicalAddress,VirtualAddress,Size: Longint;
               RawDataPointer,RelocationPointer,LineNumberPointer: longint;
               RelocationEntries,LineNumberEntries: word;
               Flags: longint;
             end;

  TCoffHeader = record
                  Magic,Sections: word;
                  TimeDate: longint;
                  SymbolTablePointer,SymbolTableCount: Longint;
                  NumberBytesOptional: word;
                  Flags: word;
                  Section1,Section2,Section3: TSection;
                end;
  TCoffLineInfo = record
                    address: longint;
                    Line: word;
                  end;
  TCoffSymbol = record
                  Name: array[0..7] of char;
                  Value: longint;
                  Section: word;
                  Spec: word;
                  SClass: byte;
                  Aux: byte;
                end;
{$A+}
const
  SizeOfIDBuffer= 30000;
  AVRStr : array[0..15] of char = ('A','V','R',' ','O','b','j','e','c','t',
                                   ' ','F','i','l','e',#0);
var
{$A-}
  OBJRec: TOBJRec;
  OBJHeader: TOBJHeader;
{$A+}
  MnemonicID: integer;
  NumberOfLinkFiles,
  NumberOfMacroArgs,
  MacroExpansionNumber,
  AddLinkFiles: word;

  LLChar: char;
  LocalLabel: string;

  Numeric,
  NumberOfIDs,
  CodeAssemblyPointer,
  DataAssemblyPointer,
  ABSAssemblyPointer,
  BranchDisplacement,
  Marker,
  PreviousValue,
  NumLabelsChanged,
  CodeLoadAt,
  E2AssemblyPointer,
  ConstAssemblyPointer,
  LineOffset,
  HighestConstAssemblyPointer,
  HighestConst,
  CurrentTrackingVariable,
  E2Size,
  ProcedureStartLine,
  CoffSymbolIndex,
  LineNumberEntries,
  StaticCount,
  TRIMDown,
  SourceLineNumber: longint;

  ExpressionType: TExpressionTypes;

  RegisterCode,
  BranchType,
  LastPass,
  HighLowFlag,
  SourceFileNumber,
  OldSourceFileNumber,
  StorageByte: byte;

  ExpressionItem:   string;
  Os: string;

  ExpressionOperator: string;

  ID: array[0..SizeOfIDBuffer] of TID;

  RegisterSubs: Array[0..31] of string;

  InstructionError,
  LabelsChanged,
  FinalPass,
  DelayedError,
  LineHasLabel,
  TempLabelsChanged,
  If_Skip,
  SyntaxCheck,
  ParserToOFile,
  CreateObject,
  ByteInStorage,
  InsideIfRef,
  FirstCodeSpecial,
  ExpandMacro,
  FromSource,
  RedefinedLabel,
  ExtendedLPM: boolean;

  ActiveArea: TMemoryAreas;
  DebugFile: Textfile;
  OFile,ObjectFile: file;
  Labels: array[1..100] of TStringList;
  TFile: TextFile;

  SymLineWaiting:  string;
  ProcedureName:   string;

  MacroArg: array[0..9] of string;
  MacroName: string;
  E2Buffer: array[0..$FFFF] of byte;
{$A-}
  CoffHeader: TCoffHeader;
  RelocationInfo: TRelocationInfo;
  CoffLineInfo: TCoffLineInfo;
  CoffSymbol: TCoffSymbol;
  CoffLineFile,CoffSymFile,CoffStringFile,CoffStaticFile: File;
{$A+}

const
  Regs: array[0..31] of string =('R0','R1','R2','R3','R4','R5','R6','R7',
                                   'R8','R9','R10','R11','R12','R13','R14',
                                   'R15','R16','R17','R18','R19','R20','R21',
                                   'R22','R23','R24','R25','R26','R27','R28',
                                   'R29','R30','R31');

{==============================================================================}

procedure Status(s: string);

   var  R: string;
        I,C: word;

   begin
   I := 1;
   C := Length(S);
   R := '';
   while I <= C do
      begin
      if S[I] = char (vkTab) then
         begin
         R := R + ' ';
         while length(R) mod 8 <> 0 do
            R := R + ' ';
         end
      else
         R := R + S[I];
      inc(I);
      end;

   fmMainForm.AddMessage (R);
   end;

{==============================================================================}

procedure MakeError(T: word);

   var  S: string;
        I: word;

   begin
   if Error then
      exit;
   case T of
      0: S := 'Syntax error';
      1: S := 'Not a Mnemonic or Directive';
      2: S := 'Identifier not defined: '+ExpressionItem;
      3: S := 'Multiple definition of this label: '+ExpressionItem;
      4: S := 'Too many labels in project';
      5: S := 'Byte allocated ouside of memory range';
      6: S := 'Unsupported argument for ORG directive';
      7: S := 'Bit expression must evaluate to 0-7';
      8: S := 'Relative Branch destination not reachable';
      9: S := 'Labels not permitted inside Macros';
      10: S := 'Label not found during multiple pass';
      11: S := 'parameter out of range';
      12: S := 'DDIR directives not permitted during Autoscale';
      13: S := 'Scale directive only allowed in longword mode';
      14: S := 'Unterminated string';
      15: S := 'External identifier not defined global: '+ExpressionItem;
      16: S := 'Equate has no identifier';
      17: S := '"Endif" without matching "if"';
      18: S := '"Ifref" may not be nested inside another "If"';
      19: S := 'Load address greater than 16MB';
      20: S := 'IncludeFile not found';
      21: S := 'Nested include files not supported';
      22: S := 'Syntax error during macro expansion';
      23: S := 'Macro argument number not defined';
      24: S := 'Multiple use of CODESP not valid';
      25: S := 'Register tracking variable not defined';
      26: S := 'Constant expression out of range';
      27: S := 'No current tracking variable active (##)';
      28: S := 'Register name too short (3 chars minimum)';
      30: S := 'Unbalanced "if" statement in source';
      31: S := 'Code size too large for Demo';
      32: S := 'Value not in range for register (0-31)';
      33: S := 'Global label never defined';
      34: S := 'Error in RSL source file: ' + CurrentLine;
      100: S := ErrorString;
      101: S := 'Error in RSL on line ' + IntToStr (LineNumber) + ': ' + ErrorString;
      end;

   if CurrentFileIsInclude then
      S := IncludeFileName + ': ' + S;
   Status(s);
   if Pass = 1 then
      Status(CurrentLine)
   else
      begin
      Status ('File: ' + ExtractFilename (CurrentFile) + '   Line #' + IntToStr (LineNumber+1));
      Status (CurrentLine);
      I := 1;
//     repeat
//        if GetXmlData (CurrentLine, I, S) = 'O' then
//           Status (S);
//     until I > length (CurrentLine);
      end;
//    Status(Reconstruct);
   Error := true;
   end;

{==============================================================================}

procedure WriteOpLn(S: string);                                                  //ss short

   begin
   S := S + '<\N><\O' + CurrentLine + '>';
   OpLines.Add (S);
   end;

{==============================================================================}

procedure InsertByteInBuffer(B: byte);

   begin
   if If_Skip then
      begin
      InstructionError := false;
      exit;
      end;

   if AssemblyPointer < BufferSize then
      begin
      case ActiveArea of
         CodeActive:
            begin
      //inc(CodeAssemblyPointer);
            if FinalPass then
               begin
               if AssemblyPointer + CodeLoadAt < BufferSize then
                  Buffer^[AssemblyPointer + CodeLoadAt] := B;
               end;

            if ExeSize < AssemblyPointer + CodeLoadAt then
               ExeSize := AssemblyPointer + CodeLoadAt;
            CodeAssemblyPointer := AssemblyPointer + CodeLoadAt + 1;
            end;

         DataActive:
            DataAssemblyPointer := AssemblyPointer + 1;

         ABSActive:
            begin
            if FinalPass then
               if AssemblyPointer < BufferSize then
                  Buffer^[AssemblyPointer] := B;

            if ExeSize < AssemblyPointer + 1 then
               ExeSize := AssemblyPointer + 1;

            ABSAssemblyPointer := AssemblyPointer + 1;
            end;

         ConstActive:
            begin
      //inc(ConstAssemblyPointer);
            if FinalPass then
               begin
               if AssemblyPointer + CodeLoadAt < BufferSize then
                  Buffer^[AssemblyPointer + CodeLoadAt] := B;
               end;

            if ExeSize < AssemblyPointer + CodeLoadAt then
               ExeSize := AssemblyPointer + CodeLoadAt;

            ConstAssemblyPointer := AssemblyPointer + CodeLoadAt + 1;
            if ConstAssemblyPointer > HighestConstAssemblyPointer then
               HighestConstAssemblyPointer := ConstAssemblyPointer;
            end;

         E2Active:
            begin
            if FinalPass then
               if AssemblyPointer < $FFFF then
                  E2Buffer[AssemblyPointer] := B;

            if AssemblyPointer < $FFFF then
               E2AssemblyPointer := AssemblyPointer + 1;

            if E2Size <= AssemblyPointer + 1 then
               E2Size := AssemblyPointer + 1;
            end;
         end;

      inc(AssemblyPointer);
      end
   else
      MakeError(5);

   InstructionError := false;
   end;

{==============================================================================}

procedure InsertByte(B: byte); forward;

{==============================================================================}

procedure Code2(W: word);

   var  L: word;
        LW: longint;

   begin
   if If_Skip then
      begin
      InstructionError := false;
      exit;
      end;

   if ByteInStorage then
      InsertByte(0);

   if odd(AssemblyPointer) then
      InsertByteInBuffer(0);

   if CreateObject and (ActiveArea in [CodeActive, ConstActive, ABSActive]) then
      begin
      LW := SourceLineNumber + 1;
      OBJRec.LineNumber1 := hi(LW);
      OBJRec.LineNumber2 := lo(LW);
      OBJRec.Code1 := hi(W);
      OBJRec.Code2 := lo(W);
      L := AssemblyPointer shr 1;
      OBJRec.Address1 := L shr 16;
      OBJRec.Address2 := L shr 8;
      OBJRec.Address3 := L;
      OBJRec.SourceFileNumber := SourceFileNumber - 1;
      OBJRec.Macro := 0;
      BlockWrite(ObjectFile, OBJRec, 9);
      end;

   InsertByteInBuffer(lo(W));
   InsertByteInBuffer(hi(W));
   end;

{==============================================================================}

procedure InsertByte(B: byte);

   begin
   if ActiveArea in [CodeActive, CONSTActive, ABSActive] then
      begin
      if not ByteInStorage then
         begin
         ByteInStorage := true;
         StorageByte := B;
         InstructionError := false;
         end
      else
         begin
         ByteInStorage := false;
         Code2(StorageByte + B * 256);
         end;
      end
   else
      InsertByteInBuffer(B);
   end;

{==============================================================================}

function RelativeRange(FL, Target: longint): boolean; {used to size 7 bit relative branching}

   begin
   if If_Skip then
      begin
      result:=true;
      exit;
      end;

   if Pass = 1 then
   begin
      result := true;        //for pass 1 assume it will fit in 8 bit displacement
      exit;
      end;

   result := true; {try for 7 bit offset}
   if Target >= FL then
      begin
      if Target <= FL + 126 then
         begin
         BranchDisplacement := (Target - FL) shr 1;
         exit;
         end;
      end
   else
      begin
      if Target >= FL - 128 then
         begin
         BranchDisplacement := ((FL - target) shr 1) * -1;
         BranchDisplacement := BranchDisplacement and $7F;
         exit;
         end;
      end;

   result := false; {no, cannot do..}
   end;

{==============================================================================}

function RelativeRange2K(FL, Target: longint): boolean; {used to size 12 bit relative branching}

   begin
   if If_Skip then
      begin
      result := true;
      exit;
      end;

   if Pass = 1 then
      begin
      result := true;
      exit; //for pass 1 assume it will fit in 12 bit displacement
      end;

   result := true; {try for 12 bit offset}
   if Target >= FL then
      begin
      if Target <= FL + 4094 then
         begin
         BranchDisplacement := (Target - FL) shr 1;
         exit;
         end;
      end
   else
      begin
      if Target >= FL - 4096 then
         begin
         BranchDisplacement := ((FL - Target) shr 1) * -1;
         BranchDisplacement := BranchDisplacement and $FFF;
         exit;
         end;
      end;

   result := false; {no, cannot do..}
   if not result and ROM4K then
      begin
      if Target >= FL then
         begin
         BranchDisplacement := ((FL - target) shr 1) * -1;
         BranchDisplacement := BranchDisplacement and $FFF;
         result := true;
         exit;
         end
      else
         begin
         BranchDisplacement := ((Target - FL) shr 1);
         BranchDisplacement := BranchDisplacement and $FFF;
         result := true;
         exit;
         end;
      end;
   end;

{==============================================================================}

procedure InsertWord(N: longint);

   begin
   Code2(N);
   end;

{==============================================================================}

procedure AddLabel(S: string; V: LongInt; LabelType: TLabelType);                      //ss  short

   var  A,W: longint;
        C: char;

   begin                                                                 //  {$H-}
   if S[length (S)] = ':' then
      DecStringLength (S);                       // dec(S[0])

   if InsideMacro and (Pass = 1) then
      begin
      S := MacroName + '&' + IntToStr (MacroExpansionNumber) + '&' + S;
      LastLabel := S;
      end
   else if LLChar <> ' ' then
      begin
      if S [1] = LLChar then
         S := LocalLabel + S;
      end;

   if S = '' then
      begin
      MakeError(0);
      exit;
      end;

   if InsideMacro then
      begin
      MakeError(9);
      exit;
      end;

   if S = '$' then
      begin
      ErrorString := 'Label may not be $ (reserved)';
      MakeError(100); exit;
      end;

   if If_Skip then
      exit;

   if ByteInStorage and (V = Dollar) then
      inc(V);

   RedefinedLabel := false;
   with Labels[LinkFileNumber] do
      begin
      W := IndexOf(S);
      if Pass = 1 then
         begin
         if W <> -1 then
            begin
        //MakeError(3);
            RedefinedLabel := true;             // Label already in list
            W := longint(Objects[W]);           // Get location of label in ID
            A := ID[W].Value;                   // Get current value in ID
            ID[W].Value := V;                   // Put new value in ID
            PreviousValue := A;
            if A <> V then                      // if new value <> previous value
               begin
               inc(NumLabelsChanged);
               LabelsChanged := true;
               end;
            end
         else
            begin
            LabelsChanged := true;
            AddObject(S, Pointer(NumberOfIDs));      // Add label with location of prarameters in ID array
            ID[NumberOfIDs].Value := V;
            PreviousValue := V;

            if LabelType = ltExternal then
               ID[NumberOfIDs].Extern := true
            else
               ID[NumberOfIDs].Extern := false;

            ID[NumberOfIDs].Referenced := false;
            ID[NumberOfIDs].DynamicVar := false;
            inc(NumberOfIDs);
            inc(NumLabelsChanged);
            if NumberOfIDs > SizeOfIDBuffer then
               MakeError(4);
            end;
         end
      else   // Pass > 1
         begin
         if W = -1 then
            MakeError(10)
         else
            begin
            W := longint(Objects[W]);
            PreviousValue := ID[W].Value;
            if V <> PreviousValue then
               begin
               LabelsChanged:=true;
               ID[W].Value := V;
               inc(NumLabelsChanged);
               end;

            if FinalPass and WriteSymbolFile then
               begin
               if ID[W].Referenced then
                  begin
                  W := ID[W].value;
            {if ID[w].DynamicVar then
            begin
              c:='V';
            end else}
//            begin
                  if ActiveArea=CodeActive then
                     C := 'C'
                  else
                     C := 'D';
                  end;
               SymlineWaiting := C + ':' + IntToHex(W, 8) + ' ' + S;
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure AddGlobal(S: String);

   var  W: longint;

   begin
   if S[S.length] = ':' then
     DecStringLength (S);

   if S = '' then
      begin
      MakeError(0);
      exit;
      end;

   if InsideMacro then
      begin
      MakeError(9);
      exit;
      end;

   if S = '$' then
      begin
      ErrorString:='Label may not be $ (reserved)';
      MakeError(100); exit;
      end;

   if If_Skip then
      exit;

    with Global do
       begin
       W := IndexOf(S);
       if Pass = 1 then
          begin
          ErrorString := 'Global label multiple definition - ' + S;
          if W <> -1 then
             MakeError(100)                 // Global already in list on Pass 1
          else
             begin
             W:=Linkfilenumber;
             AddObject(S, pointer(W));      // Set which file defined global
             end;
          end;
       end;
    end;

{==============================================================================}

function ParseItem (Terminator: TSpaceTerminator): boolean;

   label  CarryOn, Restart;

   var  C: char;   //ss   ansichar

   begin
   if (Pass = 1) and FromSource then
      begin
      CItem := '';
      Quotes := false;
      result := false;
      Comma := false;
      if LinePointer > length(CurrentLine) then
         exit;

      while (LinePointer < length(CurrentLine)) do                               // Move LinePointer to first character not space, comma or #9
         begin
         C := CurrentLine [LinePointer];
         if (C <= ' ') or (C = ',') or (C = #9) then      // #9 = tab
            inc(LinePointer)
         else
            break; {get rid of initial white space and commas}
         end;

      LastLinePointer := LinePointer;
Restart:
      if CurrentLine[LinePointer] = '''' then
         begin
         Quotes := true;
         inc(LinePointer);
         CItem := CItem + '''';
         end;

      if not Quotes then
         begin
         while (LinePointer <= length(CurrentLine)) do
            begin
            C := CurrentLine[LinePointer];
            if Terminator = spSpaceIsTerminator then
               begin
               if not (CharInSet (C, [',', ' ', ';', #9])) then
//               if (C <> ',') and (C <> ' ') and (C <> ';') and (C <> #9) then
                  begin
                  CItem := CItem + C;
                  inc(LinePointer);
                  end
               else
                  break;
               end
            else
               begin
               if C = #9 then
                  C := ' ';
               if (C <> ',') and (C <> ';') then
                  begin
                  if C <> ' ' then
                     CItem := CItem + C;
                  inc(LinePointer);
                  end
               else
                  break;
               end;
            end;
         end
      else
         begin
CarryOn:
         while (LinePointer <= length(CurrentLine)) and (CurrentLine[LinePointer] <> '''') do
            begin
            CItem := CItem + OriginalLine[LinePointer];
            inc(LinePointer);
            end;

         if LinePointer < length(CurrentLine) then
            begin
            if CurrentLine[LinePointer] = '''' then
               inc(LinePointer);
            if CurrentLine[LinePointer] = '''' then
               begin
               CItem := CItem + '''';
               inc(LinePointer);
               goto carryOn;
               end
            else
               begin
               CItem := CItem + '''';
               Quotes := false;
               goto restart;
               end;
            end
         else
            begin
            CItem := CItem + '''';
            inc(LinePointer);
            end;
         end;
      if (LinePointer <= length(CurrentLine)) and (CurrentLine[LinePointer] = ',') then
         Comma := true;

      if CItem <> '' then
         Result := true
      else
         Result:=false;

      if CItem <> '' then
         if CItem [1] = ';' then
            Result:=false;

      if Result and ParserToOFile then
         begin
         if Comma then
            Os := Os + '<\D3>' + '<\T' + CItem + '>'
         else
            Os := Os + '<\D4>' + '<\T' + CItem + '>';
         end;
      end
   else    // Pass > 1
      begin
      if XMLField [LinePointer].DataType = 'N' then   // end of line
         begin
         Result := false;
         CItem := '';
         exit;
         end;

      if XMLField [LinePointer].Data = 'D3' then
         Comma := true
      else if XMLField [LinePointer].Data = 'D4' then
         Comma := false;

      inc(LinePointer);
      if XMLField [LinePointer].DataType = 'T' then
         begin
         CItem := XMLField [LinePointer].Data;
         inc(LinePointer);
         end
      else
         begin
         ErrorString := 'Data field not found';
         MakeError (101);
         exit;
         end;
      result := true;
      end;
   end;

{==============================================================================}

{procedure SquashSpaces;   // Remove spaces from CItem

   var  S: string;
        N: word;

   begin
   S := '';
   for N := 1 to length(CItem) do
      begin
      if CItem[N] <> ' ' then
         S := S + CItem[N];
      end;
   CItem:=s;
   end;

{==============================================================================}

function GetExpressionItem(var I: word; var NumericValid: boolean): boolean;

   var  done: boolean;
        c: char;                                                                 //  ansichar
        l,x: word;
        w,lw: longint;
        Err:  integer;

{====================}

   function DecimalWord: boolean; // Test for decimal number

      var  S:  string;

      begin
//ss      dec (ExpressionItem [0]);
      S := ExpressionItem;
      DecStringLength (ExpressionItem);
      result:=true;
         try
         Numeric:=StrToInt(ExpressionItem);
         except on EConvertError do
            begin
            result:=false;
//ss            inc (ExpressionItem [0]);
            ExpressionItem := S;
//            IncStringLength (ExpressionItem);
            end;
         end;
      end;

{====================}

   function HexWord: boolean;  // Test for hex number

      var  S:  string;

      begin
//ss      dec (ExpressionItem [0]);
      S := ExpressionItem;
      DecStringLength (ExpressionItem);
      l:=length(ExpressionItem);
      x:=1;
      Numeric:=0;
      result:=true;
      while (x<=l) and result do
         begin
         Numeric:=Numeric*16;
         if CharInSet (ExpressionItem[x], ['0'..'9']) then
            Numeric:=Numeric+ord(ExpressionItem[x])-$30
         else if CharInSet (ExpressionItem[x], ['A'..'F']) then
            Numeric:=Numeric+ord(ExpressionItem[x])-$41+10
         else
            result:=false;
         inc(x);
         end;
      if not result then
//ss         inc (ExpressionItem [0]);
         ExpressionItem := S;
//         IncStringLength (ExpressionItem);
      end;

{===================}

   function HexWordSpecial: boolean;
      begin
      ExpressionItem:=copy(ExpressionItem,2,255);
//      delete (ExpressionItem, 1, 1);
      l:=length(ExpressionItem);
      x:=1;
      Numeric:=0;
      result:=true;
      while (x<=l) and result do
         begin
         Numeric:=Numeric*16;
         if CharInSet (ExpressionItem[x], ['0'..'9']) then
            Numeric:=Numeric+ord(ExpressionItem[x])-$30
         else if CharInSet (ExpressionItem[x], ['A'..'F']) then
            Numeric:=Numeric+ord(ExpressionItem[x])-$41+10
         else
            result:=false;
         inc(x);
         end;
      if not result then
         ExpressionItem:='$'+ExpressionItem;
      end;

{====================}

   function BinaryWord: boolean;

      var  S:  string;

      begin
//ss      dec(ExpressionItem[0]);
      S := ExpressionItem;
      DecStringLength (ExpressionItem);
      l:=length(ExpressionItem);
      x:=1;
      Numeric:=0;
      result:=true;
      while (x<=l) and result do
         begin
         Numeric:=Numeric*2;
         if CharInSet (ExpressionItem[x], ['0','1']) then
            Numeric:=Numeric+ord(ExpressionItem[x])-$30
         else
            result:=false;
         inc(x);
         end;
      if not result then
//ss         inc(ExpressionItem[0]);
         ExpressionItem := S;
//         IncStringLength (ExpressionItem);
      end;

{====================}

   begin  // GetExpressionItem \\
   ExpressionItem:='';
   ExpressionOperator:='';
   done:=false;
   Result:=false;
   if length(CItem) >= I + 2 then
      if (CItem[I]='''') and (CItem[I+2]='''') then
         begin
         ExpressionItem:=copy(CItem,I,3); I:=I+3;
         end;

   while (I <= length(CItem)) and not done do
      begin
      C := CItem[I];
      inc(I);
      if CharInSet (C, ['.','(',')','+','-','*','/']) then
         begin
         Done := true;
         ExpressionOperator := C;
         if C = '.' then
            begin
            if Copy(CItem,I,3)='B0(' then
               begin
               ExpressionOperator := '.LO';
               I := I + 2;
               end
            else if Copy(CItem, I, 3) = 'B1(' then
               begin
               ExpressionOperator:='.HI';
               I := I+2;
               end
            else if Copy(CItem,I,3)='B2(' then
               begin
               ExpressionOperator := '.B2';
               I := I+2;
               end
            else if Copy(CItem,I,3)='B3(' then
               begin
               ExpressionOperator:='.B3';
               I:=I+2;
               end
            else if Copy(CItem,I,3)='W0(' then
               begin
               ExpressionOperator:='.W0';
               I:=I+2;
               end
            else if Copy(CItem,I,3)='W1(' then
               begin
               ExpressionOperator:='.W1';
               I:=I+2;
               end
            else if Copy(CItem,I,3)='LO(' then
               begin
               ExpressionOperator:='.LO';
               I:=I+2;
               end
            else if Copy(CItem,I,3)='HI(' then
               begin
               ExpressionOperator:='.HI';
               I:=I+2;
               end
            else if Copy(CItem,I,4)='AND.' then
               begin
               ExpressionOperator:='.AND.';
               I:=I+4;
               end
            else if Copy(CItem,I,3)='OR.' then
               begin
               ExpressionOperator:='.OR.';
               I:=I+3;
               end
            else if Copy(CItem,I,4)='XOR.' then
               begin
               ExpressionOperator:='.XOR.';
               I:=I+4;
               end
            else if Copy(CItem,I,4)='SHR.' then
               begin
               ExpressionOperator:='.SHR.';
               I:=I+4;
               end
            else if Copy(CItem,I,4)='SHL.' then
               begin
               ExpressionOperator:='.SHL.';
               I:=I+4;
               end
            else if Copy(CItem,I,4)='MOD.' then
               begin
               ExpressionOperator:='.MOD.';
               I:=I+4;
               end;
            end;  // if C = '.'
         end   // if CharInSet
      else
         begin
         ExpressionItem := ExpressionItem + C;
         end;
      end;

   if Done and (ExpressionItem = '') then
      begin
      result := true;
      Numeric := 0;
      NumericValid := false;
      exit;
      end;

   Done:=false;
   if ExpressionItem <> '' then
      begin
      NumericValid := true;
      C := ExpressionItem[length(ExpressionItem)];
      if (ExpressionItem[1] = '$') and (length(ExpressionItem) > 1) then
         Done:=HexWordSpecial
      else if C = 'D' then
         Done := DecimalWord
      else if C = 'H' then
         Done := HexWord
      else if C = 'B' then
         Done := BinaryWord
      else if C = '''' then
         begin
         if (ExpressionItem[1] = '''') and (length(ExpressionItem) = 3) then
            begin
            Numeric := ord(ExpressionItem[2]);
            Done:=true;
            result := Done;
            exit;
            end
         end
      else
         begin
         done:=true;
//            try
//            Numeric:=StrToInt(ExpressionItem);
            val (ExpressionItem, Numeric, Err);
            if Err > 0 then
               Done := false;
//            except on EConvertError do
//               begin
//               done:=false;
//               end;
//            end;
         end;

      if not Done then
         begin
         if If_Skip then
            begin
            Numeric := 0;
            Done := true;
            result := done;
            exit;
            end;

         if ExpressionItem = '$' then
            begin
            Numeric := AssemblyPointer;
            Done := true;
            result := Done;
            exit;
            end;

{     if ExpressionItem[1]=LLChar then ExpressionItem:=LocalLabel+ExpressionItem;
      w:=Labels[LinkfileNumber].IndexOf(ExpressionItem);
      if InsideMacro and (w=-1) then
      begin
        w:=Labels[LinkfileNumber].IndexOf(MacroName+'&'+IntToStr(MacroExpansionNumber)+'&'+ExpressionItem);
      end;
      if w=-1 then
      begin
        if pass>1 then
        begin
          ErrorString:='Identifier not defined: '+ExpressionItem;
          delayederror:=true;
          done:=true;
        end else
        begin
          Numeric:=0;
          done:=true;
        end;
      end else
      begin
        //w:=longint(ALabels.Objects[w]);
        w:=longint(Labels[LinkFileNumber].Objects[w]);
        if pass>1 then
        begin
          if ID[w].Extern then
          begin
            w:=Global.IndexOf(ExpressionItem);
            if w=-1 then
            begin
              if SyntaxCheck then
              begin
                w:=0;
              end else MakeError(15);
            end else
            begin
              lw:=longint(Global.Objects[w]);
              //w:=Alabels.IndexOf(#1+chr(lw)+ExpressionItem);
              w:=labels[lw].IndexOf(ExpressionItem);
              if w<0 then
              begin
                MakeError(33); exit;
              end;
              //w:=longint(ALabels.Objects[w]);
              w:=longint(Labels[lw].Objects[w]);
            end;
          end;
        end;
        Numeric:=ID[w].Value;
        Numeric:=ID[w].Value;
        if pass>1 then
        begin
          if InsideIfRef then
          begin
            if not If_Skip then ID[w].Referenced:=true;
          end else ID[w].Referenced:=true;
        end else
        begin
          if not InsideIfRef then ID[w].Referenced:=true;
        end;
        done:=true;
      end;
    end;
  end;
  result:=done;
end;
}
         if ExpressionItem [1] = LLChar then
            ExpressionItem := LocalLabel + ExpressionItem;
         W := Labels[LinkFileNumber].IndexOf(ExpressionItem);

         if InsideMacro and (W = -1) then
            begin
            W:=Labels[LinkfileNumber].IndexOf(MacroName + '&' + IntToStr(MacroExpansionNumber) + '&'+ExpressionItem);
            end;

         if W = -1 then
            begin
            if Pass > 1 then
               begin
               ErrorString := 'Identifier not defined: ' + ExpressionItem;
               DelayedError := true;
               Done := true;
               end
            else
               begin
               Numeric := 0;
               Done := true;
               end;
            end
         else
            begin
//            if ExpressionItem = '_ADRCACHE_' then
//               W := 0;

           //w:=longint(ALabels.Objects[w]);
            W := longint(Labels[LinkFileNumber].Objects[W]);   // Get location of label parameters in ID array
            if Pass > 1 then
               begin
               if ID[W].Extern then                      // if item is defined as external label then check for
                  begin                                  // item in Global list
                  W := Global.IndexOf(ExpressionItem);
                  if W = -1 then                         // if not in Global list
                     begin
                     if SyntaxCheck then
                        begin
                        W := 0;
                        end
                     else
                        MakeError(15);
                     end
                  else  // W <> -1  meaning item is in global list
                     begin
                     LW := longint(Global.Objects[W]);    // Get filenumber of file defining global
                     //w:=Alabels.IndexOf(#1+chr(lw)+ExpressionItem);
                     W := labels[LW].IndexOf(ExpressionItem);    // CHeck if item in Labels
                     if W < 0 then                               // if not in list
                        begin
                        MakeError(33);
                        exit;
                        end;
                     //w:=longint(ALabels.Objects[w]);
                     W := longint(Labels[LW].Objects[W]);       // get locatin of items parameters in ID array
                     end;
                  end;
               end;

            Numeric:=ID[W].Value;
            if pass > 1 then
               begin
               if InsideIfRef then
                  begin
                  if not If_Skip then
                     ID[w].Referenced:=true;
                  end
               else
                  ID[w].Referenced:=true;
               end
            else
               begin
               if not InsideIfRef then
                  ID[w].Referenced:=true;
               end;
            done:=true;
            end;
         end;
      end;
   result:=done;
   end;

{==============================================================================}

function TryExpression(var Value: longint): boolean;

   var  Expr: string;
        I: word;
        NumericValid: boolean;

   begin
   result := false;
   Expr := '';
   I := 1;

   while GetExpressionItem(I, NumericValid) do
      begin
      if NumericValid then
         begin
         Expr := Expr + IntToStr(Numeric) + ExpressionOperator;
         end
      else
         begin
         Expr := Expr + ExpressionOperator;
         end;
      end;

   if EvaluateExpression(Value, Expr) then
      result := true
   else if Pass = 1 then
      result := true;
   end;

{==============================================================================}

{function getExpression: boolean;
var
  n: word;
  s: String;
begin
  result:=false;
  ExpressionType:=ExUnknown;
  if ParseItem(spSpaceNotTerminator) then
  begin
    if (CItem[1]='R') and (length(CItem)<4) then
    begin
      RegisterCode:=StrToIntDef(copy(CItem,2,2),$FF);
      if RegisterCode<32 then
      begin
        result:=true;
        ExpressionType:=ExRegister8;
        exit;
      end;
    end;
    for n:=0 to 31 do {simple register expression}
{    begin
      if CItem=Regs[n] then
      begin
        RegisterCode:=n;
        result:=true;
        ExpressionType:=ExRegister8;
        exit;
      end;
    end;
    if length(CItem)>2 then
    begin
      for n:=0 to 31 do {register name substitutions}
{      begin
        if CItem=RegisterSubs[n] then
        begin
          RegisterCode:=n;
          result:=true;
          ExpressionType:=ExRegister8;
          exit;
        end;
      end;
      if Copy(CItem,1,2)='Z+' then
      begin
        ExpressionType:=ExIndexedZ;
        CItem:=Copy(CItem,3,255);  {fall through to tryexpression}
 {     end else
      if Copy(CItem,1,2)='Y+' then
      begin
        ExpressionType:=ExIndexedY;
        CItem:=Copy(CItem,3,255);  {fall through to tryexpression}
 {     end;
    end;
    if TryExpression(Numeric) then
    begin
      if ExpressionType=ExUnknown then ExpressionType:=ExExpression;
      result:=true;
    end;
  end;
end;
}
function GetExpression: boolean;

   var  N: word;

   begin
   result := false;
   ExpressionType := ExUnknown;
   if ParseItem (spSpaceNotTerminator) then
      begin
      if (CItem[1] = 'R') and (length(CItem) < 4) then
         begin
         RegisterCode := StrToIntDef(copy(CItem, 2, 2), $FF);
         if RegisterCode < 32 then
            begin
            result := true;
            ExpressionType := ExRegister8;
            exit;
            end;
         end;

      for N := 0 to 31 do //simple register expression
         begin
         if CItem = Regs[N] then
            begin
            RegisterCode := N;
            result := true;
            ExpressionType := ExRegister8;
            exit;
            end;
         end;

      if length(CItem) > 2 then
         begin
         for N := 0 to 31 do //register name substitutions
            begin
            if CItem = RegisterSubs[N] then
               begin
               RegisterCode := N;
               result := true;
               ExpressionType := ExRegister8;
               exit;
               end;
            end;

         if copy(CItem, 1, 2) = 'Z+' then
            begin
            ExpressionType := ExIndexedZ;
            CItem := Copy(CItem, 3, 255);  //fall through to tryexpression
            end
         else if copy(CItem, 1, 2) = 'Y+' then
            begin
            ExpressionType := ExIndexedY;
            CItem := copy(CItem, 3, 255);  //fall through to tryexpression
            end;
         end;

      if TryExpression(Numeric) then
         begin
         if ExpressionType = ExUnknown then
            ExpressionType := ExExpression;
         result:=true;
         end;
      end;
   end;

{==============================================================================}

function GetExpressionComma: boolean;

   begin
   if GetExpression and Comma then
      result:=true
   else
      result:=false;
   end;

{==============================================================================}

function GetExpressionNoComma: boolean;

   begin
   if GetExpression and not Comma then
      result:=true
   else
      result:=false;
   end;

{==============================================================================}

{procedure sp_LLChar;

   begin
   if parseItem(true) and not Comma then
      begin
      if length(CItem)=1 then
         begin
         if CharInSet (CItem[1], ['?','!','@','#','%','^','&','[',']','{','}{','=','_','\','~']) then
{            LLChar:= char (CItem[1]);
         end
      else
         MakeError(0);
      end
   else
      MakeError(0);
   end;

{==============================================================================}

procedure sp_Macro; {This will run only during pass 1}

   var  i: word;
        EndIt: boolean;

   begin
   if InsideMacro then
      begin
      MakeError(9); exit;
      end;
   if LineHasNoLabel then
      begin
      MakeError(8); exit;
      end
   else
      begin
      AddTheLabel:=false;
      CurrentMacro:=MacroCount;
      if LastLabel[length(LastLabel)] = ':' then
//ss         dec(LastLabel[0]);
         DecStringLength (LastLabel);
      MacroCall.Add (LastLabel);
      if CurrentfileIsInclude then
         begin
         inc(IncludeLineNumber);
         if IncludeLineNumber=AssemblerSource.Count then
            exit;
         end
      else
         begin
         inc(LineNumber);
         if LineNumber=AssemblerSource.Count then
            exit;
         end;

      endit:=false;
         repeat
         if CurrentfileIsInclude then
            begin
            OriginalLine := Includes [IncludeLineNumber];
//            StrToShort (Include [IncludeLineNumber], OriginalLine);
            inc(IncludeLineNumber);
            if IncludeLineNumber=AssemblerSource.Count then
               endit:=true;
            end
         else
            begin
            OriginalLine:=AssemblerSource[lineNumber];
//            StrToShort (AssemblerSource[lineNumber], OriginalLine);
            inc(LineNumber);
            if LineNumber=AssemblerSource.Count then endit:=true;
            end;
         i:=1;
         if length(OriginalLine)>0 then
            begin
            CurrentLine:=Uppercase(OriginalLine);
            MacroCall.Add (#$FF+CurrentLine);   //ss
            end;

         until (pos('ENDMACRO',CurrentLine) > 0) or Endit;
      MacroCall.Add(#$FF+' ENDMACRO');
      end;
   InstructionError:=false;
   ParserToOFile:=false;
   end;

{$I Mo1.pas}

{==============================================================================}

const
//  NumberOfMnemonics = 189;

  Ma: array[0..NumberOfMnemonics-1] of pointer =
  {0}(@M_CALL,
  {1} @M_CALLZ, {2} @M_JMP, {3} @M_JMPCC, {4} @M_JMPCS, {5} @M_JMPEQ,
  {6} @M_JMPNE, {7} @M_LDB, {8} @M_LDL, {9} @M_LDLI, {10} @M_LDW,
  {11} @M_LDWI, {12} @M_LPM, {13} @M_POP, {14} @M_POPBA, {15} @M_POPBB,
  {16} @M_POPLA, {17} @M_POPLB, {18} @M_POPWA, {19}@M_POPWB, {20} @M_PUSH,
  {21} @M_PUSHBA, {22} @M_PUSHBB, {23} @M_PUSHLA, {24} @M_PUSHLB, {25} @M_PUSHWA,
  {26} @M_PUSHWB, {27} @M_PUSHZ, {28} @M_STB, {29} @M_STL, {30} @M_STW,
  {31} @D_ABS, {32} @A_ADC, {33} @A_ADD, {34} @A_ADIW, {35} @A_AND,
  {36} @A_ANDI, {37} @A_ASR, {38} @A_BCLR, {39} @A_BLD, {40} @A_BRBC,
  {41}@A_BRBS, {42} @A_BRCC, {43} @A_BRCS, {44} @A_BREQ, {45} @A_BRGE,
  {46} @A_BRHC, {47} @A_BRHS, {48} @A_BRID, {49} @A_BRIE, {50} @A_BRLO,
  {51} @A_BRLT, {52} @A_BRMI, {53} @A_BRNE, {54} @A_BRPL, {55} @A_BRSH,
  {56} @A_BRTC, {57} @A_BRTS,{58} @A_BRVC, {59} @A_BRVS, {60} @A_BSET,
  {61} @A_BST, {62} @A_CALL, {63} @A_CBI, {64} @A_CBR, {65} @A_CLC,
  {66} @A_CLH, {67} @A_CLI, {68} @A_CLN, {69} @A_CLR, {70} @A_CLS,
  {71} @A_CLT, {72} @A_CLV, {73} @A_CLZ, {74} @D_CODE, {75} @D_CODESP,
  {76} @D_COFF_FE, {77} @D_COFF_FN, {78} @D_COFF_SM, {79} @D_COFF_VAR,{80} @A_COM,
  {81} @D_CONST, {82} @A_CP, {83} @A_CPC, {84} @A_CPI, {85} @A_CPSE,
  {86} @D_DATA, {87} @D_DB, {88} @D_DCA, {89} @A_DEC, {90} @D_DEF,
  {91} @D_DEFREG, {92} @D_DL, {93} @D_DS, {94} @D_DVAR, {95} @D_DW,
  {96} @D_E2DATA, {97} @A_ELPM, {98} @D_END, {99} @D_ENDIF, {100} @A_EOR,
  {101} @D_EQU, {102} @D_EXLPM, {103} @D_EXT, {104} @D_EXTERN, {105} @D_File,
  {106} @A_FMUL, {107} @A_FMULS, {108} @A_FMULSU, {109} @D_GLOBAL, {110} @A_ICALL,
  {111} @D_IFFALSE, {112} @D_IFREF, {113} @D_IFTRUE, {114} @A_IJMP, {115} @D_IMPORT,
  {116} @A_IN, {117} @A_INC, {118} @D_Include, {119} @A_JMP, {120} @A_LD,
  {121} @A_LDD, {122} @A_LDI, {123} @A_LDS, {124} @D_LIBRARY, {125} @D_LINE,
  {126} @D_LINK, {127} @D_LLCHAR, {128} @D_LOAD, {129} @A_LPM, {130} @A_LSL,
  {131} @A_LSR, {132} @sp_Macro, {133} @D_MemErr, {134} @A_MOV, {135} @A_MOVW,
  {136} @A_MUL, {137} @A_MULS, {138} @A_MULSU, {139} @A_NEG, {140} @A_NOP,
  {141} @A_OR, {142} @D_ORG, {143} @A_ORI, {144} @A_OUT, {145} @A_POP,
  {146} @D_PUBLIC, {147} @A_PUSH, {148} @A_RCALL, {149} @D_REGADD,  {150} @D_REGEND,
  {151} @D_REGNOT, {152} @D_REGUSE, {153} @D_REPORT, {154} @A_RET, {155} @A_RETI,
  {156} @A_RJMP, {157} @A_ROL, {158} @A_ROR, {159} @D_RunEnd, {160} @A_SBC,
  {161} @A_SBCI, {162} @A_SBI, {163} @A_SBIC, {164} @A_SBIS, {165} @A_SBIW,
  {166} @A_SBR, {167} @A_SBRC, {168} @A_SBRS, {169} @A_SEC, {170} @A_SEH,
  {171} @A_SEI, {172} @A_SEN, {173} @A_SER, {174} @A_SES, {175} @A_SET,
  {176} @A_SEV, {177} @A_SEZ, {178} @A_SLEEP, {179} @A_SPM, {180} @A_ST,
  {181} @A_STD, {182} @A_STS, {183} @A_SUB, {184} @A_SUBI, {185} @A_SWAP,
  {186} @D_TRIM, {187} @A_TST, {188} @A_WDR);

  MMap: array[0..NumberOfMnemonics-1] of TProcedureCall =
  (M_CALL,M_CALLZ,M_JMP,M_JMPCC,M_JMPCS,M_JMPEQ,M_JMPNE,M_LDB,M_LDL,
   M_LDLI,M_LDW,M_LDWI,M_LPM,M_POP,M_POPBA,M_POPBB,M_POPLA,M_POPLB,M_POPWA,M_POPWB,
   M_PUSH,M_PUSHBA,M_PUSHBB,M_PUSHLA,M_PUSHLB,M_PUSHWA,M_PUSHWB,M_PUSHZ,
   M_STB,M_STL,M_STW,
   D_ABS, A_ADC, A_ADD, A_ADIW, A_AND, A_ANDI, A_ASR, A_BCLR, A_BLD, A_BRBC,
   A_BRBS, A_BRCC, A_BRCS, A_BREQ, A_BRGE, A_BRHC, A_BRHS, A_BRID, A_BRIE,
   A_BRLO, A_BRLT, A_BRMI, A_BRNE, A_BRPL, A_BRSH, A_BRTC, A_BRTS, A_BRVC,
   A_BRVS, A_BSET, A_BST, A_CALL, A_CBI, A_CBR, A_CLC, A_CLH,
   A_CLI, A_CLN, A_CLR, A_CLS, A_CLT, A_CLV, A_CLZ, D_CODE, D_CODESP, D_COFF_FE, D_COFF_FN, D_COFF_SM, D_COFF_VAR, A_COM, D_CONST,
   A_CP, A_CPC, A_CPI, A_CPSE, D_DATA, D_DB, D_DCA, A_DEC, D_DEF, D_DEFREG, D_DL, D_DS, D_DVAR,
   D_DW, D_E2DATA, A_ELPM,
   D_END, D_ENDIF, A_EOR, D_EQU, D_EXLPM, D_EXT, D_EXTERN, D_File, A_FMUL, A_FMULS, A_FMULSU, D_GLOBAL, A_ICALL, D_IFFALSE,
   D_IFREF, D_IFTRUE, A_IJMP, D_IMPORT, A_IN, A_INC, D_Include, A_JMP,
   A_LD, A_LDD, A_LDI, A_LDS, D_LIBRARY, D_LINE, D_LINK, D_LLCHAR, D_LOAD, A_LPM, A_LSL, A_LSR,
   sp_Macro, D_MemErr,
   A_MOV, A_MOVW, A_MUL, A_MULS, A_MULSU, A_NEG, A_NOP, A_OR, D_ORG, A_ORI, A_OUT, A_POP, D_PUBLIC, A_PUSH,
   A_RCALL, D_REGADD, D_REGEND, D_REGNOT, D_REGUSE, D_REPORT, A_RET, A_RETI, A_RJMP, A_ROL, A_ROR, D_RunEnd, A_SBC,
   A_SBCI, A_SBI, A_SBIC, A_SBIS, A_SBIW, A_SBR, A_SBRC, A_SBRS, A_SEC,
   A_SEH, A_SEI, A_SEN, A_SER, A_SES, A_SET, A_SEV, A_SEZ, A_SLEEP, A_SPM,
   A_ST, A_STD, A_STS, A_SUB, A_SUBI, A_SWAP, D_TRIM, A_TST, A_WDR);

{  Ms: array[0..NumberOfMnemonics-1] of string =                                 //ss [7]
  ('.CALL','.CALLZ','.JMP','.JMPCC','.JMPCS','.JMPEQ','.JMPNE','.LDB','.LDL',
   '.LDLI','.LDW','.LDWI','.LPM','.POP','.POPBA','.POPBB','.POPLA','.POPLB','.POPWA','.POPWB',
   '.PUSH','.PUSHBA','.PUSHBB','.PUSHLA','.PUSHLB','.PUSHWA','.PUSHWB','.PUSHZ',
   '.STB','.STL','.STW',
   'ABS','ADC','ADD','ADIW','AND','ANDI','ASR','BCLR','BLD','BRBC',
   'BRBS','BRCC','BRCS','BREQ','BRGE','BRHC','BRHS','BRID','BRIE',
   'BRLO','BRLT','BRMI','BRNE','BRPL','BRSH','BRTC','BRTS','BRVC',
   'BRVS','BSET','BST','CALL','CBI','CBR','CLC','CLH',
   'CLI','CLN','CLR','CLS','CLT','CLV','CLZ','CODE','CODESP','COFF_FE','COFF_FN','COFF_SM','COFF_VR','COM','CONST',
   'CP','CPC','CPI','CPSE','DATA','DB','DCA','DEC','DEF','DEFREG','DL','DS','DVAR','DW','E2DATA','ELPM',
   'END','ENDIF','EOR','EQU','EXLPM','EXT','EXTERN','FILE','FMUL','FMULS','FMULSU','GLOBAL','ICALL','IFFALSE',
   'IFREF','IFTRUE','IJMP','IMPORT','IN','INC','INCLUDE','JMP',
   'LD','LDD','LDI','LDS','LIBRARY','LINE','LINK','LLCHAR','LOAD','LPM','LSL','LSR','MACRO','MEMERR',
   'MOV','MOVW','MUL','MULS','MULSU','NEG','NOP','OR','ORG','ORI','OUT','POP','PUBLIC','PUSH',
   'RCALL','REGADD','REGEND','REGNOT','REGUSE','REPORT','RET','RETI','RJMP','ROL','ROR','RUNEND','SBC',
   'SBCI','SBI','SBIC','SBIS','SBIW','SBR','SBRC','SBRS','SEC',
   'SEH','SEI','SEN','SER','SES','SET','SEV','SEZ','SLEEP','SPM',
   'ST','STD','STS','SUB','SUBI','SWAP','TRIM','TST','WDR');
}
{==============================================================================}

function Reconstruct: string;

   var  L: byte;
        LP: word;

   begin
{   LinePointer := 1;
   result := '';
   TempLabelsChanged := LabelsChanged; {for EQU}
 {  Xml := GetXmlData (CurrentLine, LinePointer);
   if XmlType = '9' then
//   if CurrentLine[1] = #10 then
      begin
      LinePointer:=5;
      end
   else if XmlType = '1' then
      begin
      Xml := GetXmlData (CurrentLine, LinePointer);
         begin
         CItem := XML;
         if CItem[length(CItem)]=':' then
            SetLength (Citem, CItem.Length-1);

         result:= CItem + ':';
         LP := LinePointer;
         if GetXmlData (CurrentLine, LinePointer, XML) = '5' then
            begin
            LinePointer := LP;
            exit;
            end;
         end;
      end;
   inc(LinePointer);
   result:=Result+#9+Ms[ord(CurrentLine[LinePointer])-1] + copy(CurrentLine, LinePointer,255);
   inc(LinePointer);
  } end;
{==============================================================================}

procedure CallMnemonicID;

   var  ProcCall:  TProcedureCall;

   begin
   ProcCall := MMap [MnemonicID];
   ProcCall;
   end;

{{==============================================================================}
{procedure DoFirstPass;

var
  lb,lh: byte;
  LocalLabelID: string;  //[1];

function GetMnemonic: boolean;
var
  I,J,K: word;
begin
  result:=true;
  I := 1;
  J := NumberOfMnemonics;   (*BINARY SEARCH*)
  repeat
    K := (I+J) DIV 2;
    if CItem <= Ms[K-1] then J := K-1;
    if CItem >= Ms[K-1] then I := K+1;
  until I > J;
  if I-1 > J then MnemonicID:=(K-1)*4 else result:=false;
end;

begin

   LinePointer:=1;
  if length(CurrentLine)=0 then exit;
  Dollar:=AssemblyPointer;
  Marker:=AssemblyPointer+CodeLoadAt;
  LineHasLabel:=false;
  TempLabelsChanged:=LabelsChanged; //for EQU
  if CurrentLine[1]=';' then exit;
  if CurrentLine[1]>' ' then
  begin
    ParseItem(spSpaceIsTerminator);
    LocalLabelID:='';
    if LLChar<>' ' then
    begin
      if CItem[1]=LLChar then
      begin
        CItem:=LocalLabel+CItem;
        if CItem[length(CItem)]=':' then DecStringLength (CItem); //then dec(CItem[0]);
      end else
      begin
        if CItem[length(CItem)]=':' then
        begin
          LocalLabel:=CItem;
          DecStringLength (LocalLabel); //dec(LocalLabel[0]);
          LocalLabelID:=':';
        end;
      end;
    end;
    if CItem[length(CItem)]=':' then
    begin
      DecStringLength (CItem); //dec(CItem[0]);
    end;
    AddLabel(CItem, Dollar, ltInternal);
    LastLabel:=CItem;
    LineHasLabel:=true;
  end;
  InstructionError:=true;
  delayedError:=false;
  if ParseItem(spSpaceIsTerminator) then
  begin
    if GetMnemonic then
    begin
      if pass=1 then
      begin
//        if LineHasLabel then Os:=#1+chr(length(LastLabel)+length(LocalLabelID))+LastLabel+LocalLabelID
        if LineHasLabel then Os := '<\D1>'+'<\T'+LastLabel+LocalLabelID + '>'
		else
		   Os:='';
        Os:=Os+'<\D2>'+'<\M' + IntToStr (MnemonicID) + '>';
        ParserToOFile:=true;
      end;
      asm
        mov eax,0
        mov ax,MnemonicID
        mov ecx,offset Ma
        add eax,ecx
        mov eax,[eax]
        call eax
      end;
      if not InstructionError then
      begin
        if RedefinedLabel then MakeError(3) else
        if GetExpression then MakeError(0);
      end else MakeError(0);
      if delayedError then
      begin
        MakeError(100);
      end;
    end else
    begin
      MacroPointer:=MacroCall.IndexOf(CItem);
      if MacroPointer>-1 then
      begin
        MacroName:=CItem;
        NumberOfMacroArgs:=0;
        while ParseItem(spSpaceNotTerminator) and (NumberOfMacroArgs<9) do
        begin
          MacroArg[NumberOfMacroArgs]:=CItem;
          inc(NumberOfMacroArgs);
        end;
        ExpandMacro:=true;
        inc(MacroPointer);
        inc(MacroExpansionNumber);
        WriteOpln('8'+'EF'+'L'+MacroName);
      end else MakeError(1);
    end;
  end else
  begin
    if LineHasLabel then
    begin
///      Os:=#1+chr(length(LastLabel)+length(LocalLabelID))+LastLabel+LocalLabelID+#5;
      Os:='<\D1>'+'<\T'+LastLabel+LocalLabelID+'><\D5>';
      writeOpln(OS);
    end;
  end;
  if not error and ParserToOFile then
  begin
    if CurrentFileIsInclude then
    begin
      lb:=(IncludeLineNumber mod 254)+1;
      lh:=(IncludeLineNumber div 254)+1;
    end else
    begin
      lb:=(LineNumber mod 254)+1;
      lh:=(LineNumber div 254)+1;
    end;
//    writeOpln(#10+char(SourceFileNumber)+char(lb)+char(lh)+OS);
    writeOpln('<\D9>'+'<\F'+IntToStr (SourceFileNumber)+'<\S'+IntToStr (LineNumber) + '>'+OS);
  end;
  ParserToOFile:=false;
end;

{==============================================================================}
{==============================================================================}
{==============================================================================}
{                           FIRST PASS                                         }
{==============================================================================}
{==============================================================================}
{==============================================================================}

procedure ProcessLinePass1;

   var
   //lb,lh: byte;
        LocalLabelID: string;                                                    //ss [1];
        Z: word;
        SourceLine:  word;

{====================}

      function GetMnemonic: boolean;

         var  I: integer;

         begin
         result:=true;
         I := MsList.IndexOf (CItem);
         if I > -1 then
            MnemonicID := I
         else
            result:=false;
         end;

{====================}

   begin   // Process Line Pass 1 \\
   LinePointer:=1;
   if length(CurrentLine) = 0 then
      exit;
   Dollar := AssemblyPointer;
   Marker:=AssemblyPointer+CodeLoadAt;
   LineHasLabel:=false;
   TempLabelsChanged:=LabelsChanged; {for EQU}

   if CurrentLine[1] = ';' then                                                    // Comment
      exit;

   if CurrentLine[1] > ' ' then         // local label
      begin
      ParseItem(spSpaceIsTerminator);
      LocalLabelID := '';
      if LLChar <> ' ' then
         begin
         if  (CItem[1]) = LLChar then
            begin
            CItem := LocalLabel + CItem;
            if CItem[length(CItem)] = ':' then
               DecStringLength (CItem);
            end
         else
            begin
            if CItem[length(CItem)] = ':' then
               begin
               LocalLabel := CItem;
               DecStringLength (LocalLabel);
               LocalLabelID := ':';
               end;
            end;
         end;

      if CItem[length(CItem)] = ':' then
         SetLength (CItem, CItem.Length-1);

      AddLabel(CItem, Dollar, ltInternal);
      LastLabel := CItem;
      LineHasLabel := true;
      end;   // if CurrentLine[1] > ' '

   if LineNumber = 38 then
      Z := 0;
   InstructionError:=true;
   DelayedError:=false;
   if ParseItem(spSpaceIsTerminator) then
      begin
      MnemonicId := MsList.IndexOf (CItem);

      if MnemonicID <> -1  then
         begin
         if Pass = 1 then
            begin
            if LineHasLabel then
               Os := '<\D1>' + '<\T' + LastLabel+LocalLabelID + '>'
            else
               Os:='';

            Os := Os + '<\D2>' + '<\M' + IntToStr(MnemonicID) + '>';
            ParserToOFile:=true;
            end;


         CallMnemonicID;
         if not InstructionError then
            begin
            if RedefinedLabel then
               MakeError(3)
            else if GetExpression then
               MakeError(0);
            end
         else
            MakeError(0);

         if delayedError then
            begin
            MakeError(100);
            end;
         end  // if MnemonicID
      else  // MnemonicID < 0 (failed)
         begin
         MacroPointer := MacroCall.IndexOf(CItem);
         if MacroPointer >- 1 then
            begin
            MacroName := CItem;
            NumberOfMacroArgs := 0;
            while ParseItem(spSpaceNotTerminator) and (NumberOfMacroArgs < 9) do
                begin
                MacroArg[NumberOfMacroArgs] := CItem;
                inc(NumberOfMacroArgs);
                end;

            ExpandMacro := true;
            inc(MacroPointer);
            inc(MacroExpansionNumber);
            WriteOpln(#$80 + char(MacroExpansionNumber mod 255) + char(MacroExpansionNumber div 255) + char(length (MacroName)) + MacroName);
            end
         else
            MakeError(1);
         end;
      end
   else  // ParseItem = false
      begin
      if LineHasLabel then
         begin
         OS := '<\D1>' + '<\T' + LastLabel + LocalLabelID + '>' + '<\D5>';
         writeOpln(OS);
         end;
      end;

   if not error and ParserToOFile then
      begin
      if CurrentFileIsInclude then
         begin
         SourceLine := IncludeLineNumber;
//         lb:=(IncludeLineNumber mod 254)+1;
//         lh:=(IncludeLineNumber div 254)+1;
         end
      else
         begin
         SourceLine := LineNumber;
//         lb:=(LineNumber mod 254)+1;
//         lh:=(LineNumber div 254)+1;
         end;

//      WriteOpLn (#10 + char(SourceFileNumber) + char (lb) + char(lh) + OS);
      WriteOpLn ('<\D9>' + '<\F' + IntToStr(SourceFileNumber) + '><\S' + IntToStr (SourceLine){(lb)} + '>' {; //<\H' + IntToStr(lh)} + OS);
      end;
   ParserToOFile:=false;
   end;

{==============================================================================}

procedure ParseLine (Line: string);

   var  I:  word;
        L:  word;

   begin
   for I := 1 to XMLFieldCount do
      begin
      XMLField[I].DataType := #0;
      XMLField[I].Data := '';
      end;

   I := 1;
   L := 1;
   while L <= length (Line) do
      begin
      XMLField [I] := GetXmlData (Line, L);
      inc (I);
      end;
   XMLField [I].DataType := 'X';
   end;

{==============================================================================}
{==============================================================================}
{==============================================================================}
{                           PROCESS PASS N                                     }
{==============================================================================}
{==============================================================================}
{==============================================================================}

procedure ProcessLinePassN;

   var
        Str1:  shortstring;
        E:    integer;

   label fx;

   begin
   SymLineWaiting := '';
   LinePointer:=1;
   Dollar:=AssemblyPointer;
   Marker:=AssemblyPointer+CodeLoadAt;
   LineHasLabel:=false;
   TempLabelsChanged:=LabelsChanged; {for EQU}
   if LineNumber = 7 then
      E := 0;
   ParseLine (CurrentLine);
   if LineNumber = 81 then
      E := 0;



//===== MACROS
   if XMLField [LinePointer].Data = 'D7' then
      begin
      MacroExpansionNumber:=ord(CurrentLine[2])+ord(CurrentLine[3])*255;
      StrToShort (MacroName, Str1);
//mov      move(CurrentLine[4],MacroName[0],ord(MacroName[0])+1);
      move(Str1[4], Str1[0],length (MacroName)+1);
      InsideMacro:=true;
      exit;
      end
   else if XMLField [LinePointer].Data = 'D8' then //CurrentLine[1]=#$81 then
      begin
      InsideMacro:=false; exit;
      end;
//===== END MACROS

//   if CurrentLine[1]=#10 then
   if XMLField [LinePointer].Data = 'D9' then                                                         // Get source file number
      begin
      inc (LinePointer);
      if XMLField [LinePointer].DataType = 'F' then
         begin
         val (XMLField [LinePointer].Data, SourceFileNumber, E);
         if E > 0 then
            begin
            ErrorString := 'Invalid source file number';
            MakeError (101);
            exit;
            end;
         end
      else
         begin
         ErrorString := 'Missing source file number';
         MakeError (101);
         exit;
         end;

      inc (LinePointer);
      if XMLField [LinePointer].DataType = 'S' then
         begin
         val (XMLField [LinePointer].Data, SourceLineNumber, E);
         if E > 0 then
            begin
            ErrorString := 'Invalid source line number';
            MakeError (101);
            exit;
            end;
         end
      else
         begin
         ErrorString := 'Missing source line number';
         MakeError (101);
         exit;
         end;

      LinePointer := 4;
      end;

   if XMLField [LinePointer].Data = 'D1' then
      begin
      inc (LinePointer);
//      LinePointer := 2;
      if XMLField [LinePointer].DataType = 'T' then
         CItem :=  XMLField[LinePointer].Data
      else
         begin
         ErrorString := 'Data field missing';
         MakeError (101);
         exit;
         end;

      if LLChar <> ' ' then
         begin
         if  (CItem[1]) = LLChar then
            begin
            CItem := LocalLabel + CItem;
            if CItem[length(CItem)]=':' then
               DecStringLength (CItem);
            end
         else
            begin
            if CItem[length(CItem)] = ':' then
               begin
               LocalLabel := CItem;
               DecStringLength (LocalLabel);
               end;
            end;
         end
      else if CItem[length(CItem)]=':' then
         begin
         DecStringLength (CItem);
         end;

      AddLabel(CItem, Dollar, ltInternal);
      LastLabel := CItem;
      LineHasLabel:=true;
      inc (LinePointer);
      if XMLField [LinePointer].Data = 'D5' then
//      if CurrentLine[LinePointer]=#5 then
         goto fx;
      end;

   InstructionError:=true;
   delayedError:=false;
   inc(LinePointer);
   if XMLField [LinePointer].DataType = 'M' then
      begin
      val (XMLField [LinePointer].Data, MnemonicID, E);
      if E > 0 then
         begin
         ErrorString := 'Invalid Mnemonic ID';
         MakeError (101);
         exit;
         end;
      end
   else
      begin
      ErrorString := 'Missing Mnemonic ID';
      MakeError (101);
      exit;
      end;

   if LineNumber = 38 then
      E := 0;
//   if MnemonicID = 119 then
//     E := 0;

   if MnemonicID = 117 then
      E := 0;
   inc(LinePointer);
   CallMnemonicID;

{     asm
      mov eax,0
      mov ax,MnemonicID
      mov ecx,offset Ma
      add eax,ecx
      mov eax,[eax]
      call eax
      end;
}

   if InstructionError then
      MakeError(0);
   if delayedError then
      MakeError(100);

fx:
   if (SymLineWaiting <> '') and FinalPass and WriteSymbolFile then
      begin
      Writeln(SymFile,SymLineWaiting);
      end;
   end;

{==============================================================================}

procedure PointersZero;

   begin
   CurrentCodeMap:=0;
   NumberOfCodeMaps:=0;
   CurrentDataMap:=0;
   NumberOfDataMaps:=0;
   CurrentABSMap:=0;
   NumberOfABSMaps:=0;
   CurrentConstMap:=0;
   NumberOfConstMaps:=0;
   CurrentE2Map:=0;
   NumberOfE2Maps:=0;
   CodeMap[0].FromAdr:=0;
   CodeMap[0].ToAdr:=0;
   DataMap[0].FromAdr:=0;
   DataMap[0].ToAdr:=0;
   ABSMap[0].FromAdr:=0;
   ABSMap[0].ToAdr:=0;
   E2Map[0].FromAdr:=0;
   E2Map[0].ToAdr:=0;
   ConstMap[0].FromAdr:=0;
   ConstMap[0].ToAdr:=0;
   CodeAssemblyPointer:=0;
   DataAssemblyPointer:=0;
   ABSAssemblyPointer:=0;
   ConstAssemblyPointer:=0;
   E2AssemblyPointer:=0;
   lineNumber:=0; EndIt:=false; AssemblyPointer:=0; BufferPointer:=0;
   if pass = 1 then
      begin
      ALabels.Clear;
      Global.clear;
      NumberOfIDs:=0;
      end;
   end;

{==============================================================================}

procedure LoadFixedDefs;

   var  I: word;
        V: longint;
        S: string;                                                               //ss [128];

   label  LineDone;

      function ParseDef: boolean;

         var  Done: boolean;
              C: char;                                                              //ss ansichar

         begin
         result := false;
         Done := false;
         CItem := '';
         while (I <= length(CurrentLine)) and ((CurrentLine[I] = ' ') or (CurrentLine[I] = #9)) do
            inc(I);
         while (I <= length(Currentline)) and not Done do
            begin
            C:= (CurrentLine[I]);
            inc(I);
            if c = ';' then
               Done:=true
            else if C = ' ' then
               Done:=true
            else if C = #9 then
               Done:=true
            else if C = '=' then
               begin
               if CItem <> '' then
                  begin
                  dec(I);
                  Done := true;
                  end
               else
                  begin
                  CItem := '=';
                  Done := true;
                  end;
               end
            else
               CItem := CItem + C;
            end;

         if CItem <> '' then
            result:=true;
         end;

   begin
   if fileExists(ExePath + 'lib\' + Project[prjCpuDef]) then
      begin
      assign(TFile, ExePath + 'lib\' + Project[prjCpuDef]);
      reset(TFile);
      while not eof(TFile) do
         begin
         readln(TFile,CurrentLine);
         CurrentLine:=Uppercase(CurrentLine);
         if length(CurrentLine) > 0 then
            begin
            I:=1;
            while ParseDef do
               begin
               if CItem = ';' then
                  goto LineDone
               else if CItem = '.DEVICE' then
                  begin
                  if ParseDef then
                     begin
                     DeviceCode := CItem;
                     goto LineDone;
                     end;
                  end;
               if CItem = '.EQU' then
                  begin
                  if ParseDef then
                     begin
                     S := CItem;
                     if ParseDef then
                        begin
                        if CItem = '=' then
                           begin
                           if ParseDef then
                              begin
                              if TryExpression(v) then
                                 begin
                                 AddLabel(S, V, ltInternal);
                                 goto LineDone;
                                 end;
                              end;
                           end;
                        end;
                     end;
                  end;
               end;
LineDone:
            end;
         end;
      CloseFile(TFile);
      end
   else
      begin
      ShowMessage('Project file does not contain a reference to a valid CPU '
       +'type definition file. Please select a CPU using "options"');
      end;
   end;

{==============================================================================}
{==============================================================================}
{==============================================================================}
{                              ASSEMBLE                                        }
{==============================================================================}
{==============================================================================}
{==============================================================================}

procedure Assemble;

   var  Arg: word;
        I,C: longint;
        MacroLineDone: boolean;

   begin
   LLChar := ' ';
   LocalLabel := '';
   if Pass = 1 then
      begin
      Labels [LinkfileNumber] := TStringlist.Create;
      Labels [LinkfileNumber].Sorted := true;
      if LinkFileNumber > NumberOfLinkFiles then
         NumberOfLinkFiles := LinkFileNumber;
      I := NumLabelsChanged;                                                     // Save NumLabels Changed
      LoadFixedDefs;                                                             // Load CPU definition file

      if smallstack then
         AddLabel('LARGESTACK', 0, ltInternal)                                        //8-bit stack pointer in Options
      else
         AddLabel('LARGESTACK', 1, ltInternal);

      if HasMult then
         AddLabel ('CPUHASMULTIPLY', 1, ltInternal)                                   // 8x8 multiply in Options
      else
         AddLabel ('CPUHASMULTIPLY', 0, ltInternal);

      NumLabelsChanged := I;                                                     // Restore NumLabelsChanges
      end;

   LLChar:=' ';
   ActiveIFStatements := 0;
   FindIfEnd := false;
   ParserToOFile := false;
   CurrentFileIsInclude := false;
   InsideMacro := false;
   ExpandingMacro := false;
   ActiveArea := CodeActive;
   InsideIfRef := false;
   lineNumber := 0;
   EndIt := false;
   If_Skip := false;
   CodeAssemblyPointerAtSOF := CodeAssemblyPointer;                              // Pointers zeroed before calling Assemble
   DataAssemblyPointerAtSOF := DataAssemblyPointer;
   ConstAssemblyPointerAtSOF := ConstAssemblyPointer;
   E2AssemblyPointerAtSOF := E2AssemblyPointer;
   MacroExpansionNumber := 0;
   ExpandMacro := false;
   CurrentTrackingVariable := -1;

   for C := 0 to 31 do
      RegisterSubs [C] := '';
   D_Code;
   C := AssemblerSource.Count;
   Error := false;
   Segment := CodeSegment;

   while (LineNumber < C) and not Error and not EndIt do                         // Process assembly source
      begin
//      if LineNumber = 525 then
//         C := 31;
      if (Pass = 1) and FromSource then
         begin
         if CurrentFileIsInclude then
            begin
            OriginalLine := Includes [IncludeLineNumber];
            if IncludeLineNumber and $F = 0 then
               Application.ProcessMessages;
            end
         else
            begin
            if ExpandMacro then
               dec(LineNumber)
            else
               OriginalLine := AssemblerSource[LineNumber];

            MacroLineDone := false;
            while ExpandMacro and not MacroLineDone do
               begin
               OriginalLine := MacroCall [MacroPointer];
               if OriginalLine [1] = #$FF then
                  OriginalLine := copy(OriginalLine, 2, 255);
               if pos('ENDMACRO', MacroCall[MacroPointer+1]) > 0 then
                  ExpandMacro := false;
               inc(MacroPointer);
               I:=1;
               while (I < length(OriginalLine)) and (not Error) and ExpandMacro do
                  begin
                  if OriginalLine[I] = '@' then
                     begin
                     if I+1 <= length(OriginalLine) then
                        begin
                        if CharInSet (OriginalLine[I+1], ['1'..'9']) then
                           begin
                           Arg := ord(OriginalLine[I+1])-$31;
                           if Arg+1 > NumberOfMacroArgs then
                              MakeError(23)
                           else
                              begin
                              delete(OriginalLine, I, 2);
                              insert(MacroArg[arg], OriginalLine, I);
                              I := I + length(MacroArg[Arg]);
                              end; // if Arg+1
                           end
                        else
                           MakeError(22); // if CharInSet
                        end
                     else
                        MakeError(22); // if I+1
                     end
                  else
                     inc(i); // if (OrginalLine
                  end; // while (I < length (OriginalLine

               MacroLineDone := true;
               InsideMacro := ExpandMacro;
               if not InsideMacro then
                  WriteOpLn(#$81);
               end; // While ExpandMacro

            if lineNumber and $F = 0 then
               Application.ProcessMessages;
            end; // if CurrentFileIsInclude

         I := 1;
         CurrentLine := uppercase(OriginalLine);
         end
      else
         begin
         OriginalLine := AssemblerSource[LineNumber];
         CurrentLine := OriginalLine;
         end;


      if (Pass = 1) and FromSource then
         begin
         ProcessLinePass1;
         end
      else
         ProcessLinePassN;

      if (Pass = 1) and FromSource then
         begin
         if not error then
            begin
            if CurrentFileIsInclude then
               inc(IncludeLineNumber)
            else
               inc(lineNumber);
            end;

         if CurrentFileIsInclude then
            begin
            if (IncludeLineNumber>=Includes.Count) or Endit then
               begin
               CurrentFileIsInclude:=false;
               Endit := false;
               inc(LineNumber);
               SourceFileNumber := OldSourceFileNumber;
               end;
            end;
         end // if (Pass = 1
      else
         inc(LineNumber);
      end; // while (LineNumber

   if not Error then
      if ByteInStorage then
         InsertByte(0);

   if ActiveIFStatements > 0 then
      MakeError(30);

   if not Error and FinalPass then
      begin
      CodeMap[CurrentCodeMap].ToAdr := CodeAssemblyPointer;
      DataMap[CurrentDataMap].ToAdr := DataAssemblyPointer;
      ABSMap[CurrentABSMap].ToAdr := ABSAssemblyPointer;
      ConstMap[CurrentConstMap].ToAdr := ConstAssemblyPointer;
      E2Map[CurrentE2Map].ToAdr := E2AssemblyPointer;
      end;
   end;

{==============================================================================}
{==============================================================================}
{==============================================================================}
{                             RUN ASSEMBLER                                    }
{==============================================================================}
{==============================================================================}
{==============================================================================}

procedure RunAssembler (F: string);

   var
        ch: char;

   begin
//   cibp := 0;
   CH := #0;
   SyntaxCheck := true;
   Linkfilenumber := 1;
   ExeSize := 0;
   ExtendedLPM := false;
   RunWhenDoneFiles.Clear;
   InfoList.Clear;
   RegsUsed.Clear;
   AsmFilePath := ExtractFilePath(F);
   SourceFiles.Clear;
   SourceFiles.Add(f);
   SourceFileNumber:=SourceFiles.Count;
   AssemblerSource.LoadFromFile(F);
   Status('Assemble: ' + F);
   Pass := 1;
   LastPass := 2;
   Error := false;
   FinalPass := false;
   CreateObject := false;
   RSLFilename :=  (ChangeFileExt(F, '.RSL'));
//   assignfile(OFile, changeFileExt(F, '.RRC'));
//   rewrite(OFile, 1);
   OpLines.Clear;
//   RRCVersion:=CurrentRRCVersion;
//   blockwrite(OFile, RRCVersion, 4);

   repeat
      if Pass = 2 then
         begin
         Application.ProcessMessages;
//      assignfile(OFile, changeFileExt(F, '.RRC'));
//      reset(OFile, 1);
//      blockread(OFile, RRCVersion, 4, Res);
         AssemblerSource.clear;
         AssemblerSource.LoadFromFile (RSLFilename);
//      CharsInBuffer := 0;
{      repeat
         S := '';
         repeat
            if CharsInBuffer = 0 then
               begin
               blockread(OFile, chb, 1024, Res);
               CharsInBuffer := res;
               cibp:=0;
               end;

            if CharsInBuffer > 0 then
               begin
               ch := char (chb[cibp]);
               inc(cibp);
               dec(CharsInBuffer);
               if ch <> #$FF then
                  S := S + ch;
               end;

         until (ch = #$FF) or (Res = 0);
         if Res <> 0 then
            begin
            AssemblerSource.Add(S);
            end;
      until res=0;
      closefile(OFile);
 }       end;  // end if Pass = 2 \\

      FirstCodeSpecial := true;
      LabelsChanged := false;
      Status('Processing pass ' + IntToStr(Pass));
      FromSource := true;
      PointersZero;
      Assemble;

      if Pass = 1 then
         begin
         OpLines.SaveToFile (RSLFilename);
//      closefile(OFile);
         end;

      if not error then
         inc(Pass);

   until error or (Pass > LastPass) or ((not LabelsChanged) and (Pass > 2));

   Labels[1].free;
   if not Error then
      begin
      Status('Completed. No fatal errors...');
      end
   else
      begin
      if Pass = 1 then
         begin
         if not CurrentFileIsInclude then
            ShowErrorPosition(F, LineNumber,0)
         else
            ShowErrorPosition(IncludeFileName, IncludeLineNumber, 0);
         end
      else
         begin
         ShowErrorPosition(SourceFiles[SourceFileNumber-1], SourceLineNumber, 0);
      end;

//      AssemblerStatus.Show;
      end;
   end;

{==============================================================================}

function AppendToCoff(var BFile, FL: file): longint;

   var  I, Res: integer;
        Buff: array[0..255] of byte;

   begin
   reset(FL, 1);
   I := FileSize(FL);
   result := I;
   while I > 0 do
      begin
      blockread(FL, Buff, 256, Res);
      if Res > 0 then
         blockwrite(BFile, Buff, Res);
      I := I - Res;
      end;
   closefile(FL);
   end;

{==============================================================================}

function AppendToCoffx(var BFile, FL: file): longint;

   var  I, Res: integer;
{$A-}
        Buff: TCoffSymbol;
{$A+}
   begin
   reset(FL, 18);
   I := FileSize(FL);
   result := I;
   while I > 0 do
      begin
      blockread(FL, Buff, 1, Res);
      if Buff.Value >= $FFFFFF then
         begin
         Buff.Value := Buff.Value - $FFFFFF + LineOffset;
         end;
      if Res > 0 then
         begin
         blockwrite(BFile, Buff, 18);
         end;
      dec(I);
      end;
   closefile(FL);
   end;

{==============================================================================}
{==============================================================================}
{==============================================================================}
{                              MAKE HEX FILE                                   }
{==============================================================================}
{==============================================================================}
{==============================================================================}

procedure GenerateHexFile;

   const BufferSize = 262144;

   var  HexFile:  THexFile;
        Data:     array of byte;
        DataSize: integer;

   begin
   HexFile := THexFile.Create (nil);
   HexFile.FillByte := $FF;
   HexFile.Filename := OutputFilename + '.rom';
   SetLength (Data, BufferSize);
   HexFile.LoadBin (Data);
   DataSize := HexFile.DataBits + 1;
   SetLength (Data, DataSize);
   HexFile.Filename := OutputFilename + '.hex';
   HexFile.SaveHex (Data);
   HexFile.Free;
   end;


{==============================================================================}
{==============================================================================}
{==============================================================================}
{                            SAVE EXECUTABLE                                   }
{==============================================================================}
{==============================================================================}
{==============================================================================}

 procedure SaveExecutable;

   var
      BFile: file;
      NumWritten,
      B, N, I,BlockSize,
      ADR0,ADR1,ADR2: longint;
      C: word;
      S: string;
      SX: string;  //[20];
      CKS, SZ, X,CKSX,CurrentBase: byte;
      NotFF: boolean;
      ExeList: TStringList;
      ProjectPath:  string;

   begin
   inc(ExeSize);
//   AssignFile(TFile, Project[prjPath] + Project[prjTitle] + '.HEX');
//   rewrite(TFile);
   CurrentBase:=0;
{   ExeList := TStringList.Create;
   I := 0;
   while I < CurrentConstMap + 1 do
      begin
      B := ConstMap[I].FromAdr;
      repeat
         SZ := 0;

         if ConstMap[I].ToAdr-B >= 16 then
            begin
            CKS := $10;
            S := ':10';
            end
         else
            begin
            CKS := ConstMap[I].ToAdr-B;
            S:= ':' + IntToHex(ConstMap[I].ToAdr-B, 2);
            end;

         if B shr 16 <> CurrentBase then
            begin
            CurrentBase := B shr 16;
            SX := ':02000002' + IntToHex(CurrentBase shl 12, 4);
            CKSX := 256-(4 + CurrentBase shl 4);
            SX := SX + IntToHex(CKSX, 2);
            ExeList.Add (SX);
//            writeln(TFile, SX);
            end;

          S := S + IntToHex(B and $FFFF, 4) + '00';
          CKS := CKS + lo(B) + hi(B);
          NotFF := false;

         while (SZ < 16) and (B < ConstMap[I].ToAdr) do
            begin
            S := S + IntToHex(Buffer^[B], 2);
            CKS := CKS + Buffer^[B];
            if Buffer^[B] <> $FF then
               NotFF := true;
            inc(B);
            inc(SZ);
            end;

         if CKS = 0 then
            S := S + '00'
         else
            S := S + IntToHex($100-CKS, 2);

         if (SZ > 0) and NotFF then
            ExeList.Add (S); //writeln(TFile, S);
         until SZ < 16;
      inc(I);
      end; // while I

   I := 0;
   while I < CurrentABSMap+1 do
      begin
      B := ABSMap[I].FromAdr;
      repeat
         SZ := 0;
         CKS := 0;

         if ABSMap[I].ToAdr-B >= 16 then
            begin
            CKS := $10;
            S := ':10';
            end
         else
            begin
            CKS := ABSMap[I].ToAdr-B;
            S := ':' + IntToHex(ABSMap[I].ToAdr-b, 2);
            end;

         if B shr 16 <> CurrentBase then
            begin
            CurrentBase := B shr 16;
            SX := ':02000002' + IntToHex(CurrentBase shl 12, 4);
            CKSX := 256-(4+CurrentBase shl 4);
            SX := SX + IntToHex(CKSX, 2);
            ExeList.Add (SX); //writeln(TFile, SX);
            end;

         S := S + IntToHex(B and $FFFF, 4) + '00';
         CKS := CKS + lo(B) + hi(B);
         NotFF := false;
         while (SZ < 16) and (B < ABSMap[I].ToAdr) do
            begin
            S := S + IntToHex(Buffer^[b], 2);
            CKS := CKS + Buffer^[B];
            if Buffer^[B]<>$FF then
               NotFF:=true;
            inc(B);
            inc(SZ);
            end;

         if CKS = 0 then
            S := S + '00'
         else
            S := S + IntToHex($100-CKS, 2);

         if (SZ > 0) and NotFF then
            ExeList.Add (S); // writeln(TFile,S);
         until SZ < 16;
      inc(I);
      end;

   I := 0;
   while I < CurrentCodeMap+1 do
      begin
      B := CodeMap[I].FromAdr;
      repeat
         SZ := 0;
         CKS := 0;

         if CodeMap[I].ToAdr-B >= 16 then
            begin
            CKS := $10;
            S := ':10';
            end
         else
           begin
           CKS := CodeMap[I].ToAdr-B;
           S := ':' + IntToHex(CodeMap[I].ToAdr-B, 2);
           end;

         if b shr 16 <> CurrentBase then
            begin
            CurrentBase := B shr 16;
            SX := ':02000002' + IntToHex(CurrentBase shl 12, 4);
            CKSX := 256-(4+CurrentBase shl 4);
            SX := SX + IntToHex(CKSX, 2);
            ExeList.Add (SX); // writeln(TFile, SX);
            end;

         S := S + IntToHex(B and $FFFF,4) + '00';
         CKS := CKS + lo(B) + hi(B);
         NotFF := false;
         while (SZ < 16) and (B < CodeMap[I].ToAdr) do
            begin
            S := S + IntToHex(Buffer^[B],2);
            CKS := CKS + Buffer^[B];
            if Buffer^[B] <> $FF then
               NotFF:=true;
            inc(B);
            inc(SZ);
            end;

         if CKS = 0 then
            S := S + '00'
         else
            S := S + IntToHex($100-CKS, 2);

         if (sz>0) and NotFF then
            ExeList.Add (S); // writeln(TFile, S);
         until SZ < 16;
      inc(I);
      end;

   ExeList.Add (':00000001FF');
   ExeList.SaveToFile (Project[prjPath] + Project[prjTitle] + '.HEX');
   ExeList.Free;
}
//   writeln(TFile,':00000001FF');
//   CloseFile(TFile);
   if E2Size > 0 then
      begin
      Status('Created ' + OutputFilename + '_E2R.HEX');
      AssignFile(TFile, OutputFilename + '_E2R.HEX');

      rewrite(TFile);
      CurrentBase:=0;
      I := 0;
      B := 0;
      repeat
         SZ:=0;
         CKS := 0;

         if E2Size-B >= 16 then
            begin
            CKS := $10;
            S := ':10';
            end
         else
            begin
            CKS := E2Size - B;
            S := ':' + IntToHex(E2Size-B, 2);
            end;

         if B shr 16 <> CurrentBase then
            begin
            CurrentBase := B shr 16;
            SX := ':02000002' + IntToHex(CurrentBase shl 12, 4);
            CKSX := 256-(4+CurrentBase shl 4);
            SX := SX + IntToHex(CKSX, 2);
            writeln(TFile, SX);
            end;

         S := S + IntToHex(B and $FFFF, 4) + '00';
         CKS := CKS + lo(B)+hi(B);
         NotFF := false;

         while (SZ < 16) and (B < E2Size) do
            begin
            S := S + IntToHex(E2Buffer[B], 2);
            CKS := CKS + E2Buffer[B];
            inc(B);
            inc(SZ);
            end;

         if CKS = 0 then
            S := S + '00'
         else
            S := S + IntToHex($100-CKS, 2);

         if (SZ > 0) then
            writeln(TFile, S);
         until SZ < 16;

      writeln(TFile,':00000001FF');
      CloseFile(TFile);
      end;

   AssignFile(BFile, OutputFilename + '.ROM');
   rewrite(BFile, 1);
   if SwapBytes then
      begin
      I := 0;
      for N := 0 to ((BufferSize-1) div 2) do
         begin
         X := Buffer^[I];
         Buffer^[I] := Buffer^[I+1];
         Buffer^[I+1] := X;
         inc(I, 2);
         end;
      end;

   if ExeSize > TrimDown then
      BlockWrite(BFile,Buffer^[TrimDown], ExeSize - TrimDown,NumWritten);
   CloseFile(BFile);
   GenerateHexFile;
  winexec (@S[1], SW_SHOWNORMAL);
   C := RunWhenDoneFiles.Count;
   if C > 0 then
      SetCurrentDir(Project[prjPath]);
   I := 0;
   while I < C do
      begin
      S := RunWhenDoneFiles[I];
      if (S[1]='''') and (S[length(S)]='''') then
         S := copy(S, 2, length(S)-2);
      if FileExists(S) then
         begin
         Status('Running after merge ' + S);
         S := S + #0;
         Application.ProcessMessages;
         winexec(@s[1],SW_SHOWNORMAL);
         inc(I);
         end
      else
         begin
         S := Project[prjPath] + ExtractFileName(S);
         if FileExists(S) then
            begin
            Status('Running after merge ' + S);
            S := S + #0;
            Application.ProcessMessages;
            winexec(@s[1], SW_SHOWNORMAL);
            inc(I);
            end
         else
            begin
            Status('Warning: Run file "' + s + '" not found.');
            inc(i);
            end;
         end;
      end;

   if COFFEnabled then
      begin
      BlockSize:=((EXESize div 4) * 4) + 4;
      CoffHeader.Magic:=$A12;
      CoffHeader.Sections:=3;
      CoffHeader.TimeDate:=(Trunc(Now)-25569)*86400;
      CoffHeader.SymbolTablePointer:=0;
      CoffHeader.SymbolTableCount:=0;
      CoffHeader.NumberBytesOptional:=0;
      CoffHeader.Flags:= $F3; //$0B; //File has line numbers

      CoffHeader.Section1.name:='.text'#0;
      CoffHeader.Section1.PhysicalAddress:=0;
      CoffHeader.Section1.VirtualAddress:=0;
      CoffHeader.Section1.Size:=BlockSize;
      CoffHeader.Section1.RawDataPointer:=140;
      CoffHeader.Section1.RelocationPointer:=0;
      CoffHeader.Section1.LineNumberPointer:=0;
      CoffHeader.Section1.RelocationEntries:=0;
      CoffHeader.Section1.LineNumberEntries:=LineNumberEntries;
      CoffHeader.Section1.Flags:=$20;

      CoffHeader.Section2.name:='.data'#0;
      CoffHeader.Section2.PhysicalAddress:=0;
      CoffHeader.Section2.VirtualAddress:=0;
      CoffHeader.Section2.Size:=0;
      CoffHeader.Section2.RawDataPointer:=0;
      CoffHeader.Section2.RelocationPointer:=0;
      CoffHeader.Section2.LineNumberPointer:=0;
      CoffHeader.Section2.RelocationEntries:=0;
      CoffHeader.Section2.LineNumberEntries:=0;
      CoffHeader.Section2.Flags:=$80;

      CoffHeader.Section3.name:='.bss'#0;
      CoffHeader.Section3.PhysicalAddress:=0;
      CoffHeader.Section3.VirtualAddress:=0;
      CoffHeader.Section3.Size:=0;
      CoffHeader.Section3.RawDataPointer:=0;
      CoffHeader.Section3.RelocationPointer:=0;
      CoffHeader.Section3.LineNumberPointer:=0;
      CoffHeader.Section3.RelocationEntries:=0;
      CoffHeader.Section3.LineNumberEntries:=0;
      CoffHeader.Section3.Flags:=$80;

    assignFile(bFile, OutputFilename + '.COF');
    rewrite(bFile,1);
    BlockWrite(bfile,CoffHeader,sizeOf(CoffHeader));
    BlockWrite(bFile,Buffer^[0],BlockSize);

    adr0:=filesize(bfile);
    LineOffset:=adr0;
    appendToCoff(bfile,CoffLineFile);
    ADR1:=filesize(bfile);
    appendToCoffx(bfile,CoffSymFile);
    appendToCoff(bfile,CoffStaticFile);
    reset(CoffStringFile,1);
    i:=filesize(CoffStringFile);
    closefile(CoffStringFile);
    adr2:=fileSize(bfile);
    BlockWrite(bfile,i,4);
    appendToCoff(bfile,CoffStringFile);
    seek(bfile,8);
    BlockWrite(bfile,adr1,4);
    CoffSymbolIndex:=CoffSymbolIndex+StaticCount;
    BlockWrite(bfile,CoffSymbolIndex,4);
    seek(bfile,$30);
    BlockWrite(bfile,adr0,4);
    closefile(bfile);
    deletefile(LinkProject[1]+LinkProject[0]+'.CF1');
    deletefile(LinkProject[1]+LinkProject[0]+'.CF2');
    deletefile(LinkProject[1]+LinkProject[0]+'.CF4');
    deletefile(LinkProject[1]+LinkProject[0]+'.CF3');
    Status('Created ' + OutputFilename + '.COF');
  end;
end;


{==============================================================================}
{==============================================================================}
{==============================================================================}
{                                 LINK                                         }
{==============================================================================}
{==============================================================================}
{==============================================================================}

procedure Link;

   var  c,i,N: word;
        s:  string;
        fp: longint;

{====================}

      function SourceFileExists (Filename, Folder: string): boolean;

         begin
         result := false;
         if Folder = '' then
            if FileExists (Filename) then
               result := true
            else if FileExists (Folder + Filename) then
               result := true;
         end;

{====================}

   begin
   SyntaxCheck := false;
{   for N := 0 to NumberOfEditItems - 1 do                                        // NumberOfEditItems = 100
      begin
      if EditList[N].Used then                                                  // Save modified files
         if EditList [N] .Editor.Modified then
            begin
            if MessageDlg (ExtractFileName (EditList[N].Filename) + ' has changed. Save changes ?',
              mtConfirmation,[mbYes,mbNo],0)=mrYes then
               begin
               Editor.SaveFile (EditList [N].Filename, N);
               EditList [N].Editor.Modified := false;
               end;
            end;
      end;
}
   NumberOfLinkFiles := 0;
   fillchar (Buffer^, BufferSize, $FF);                                           // Project list format
   fillchar (E2buffer, $10000, $FF);                                              // 0 - Project name
   E2Size := 0;                                                                   // 1 - Path to project
   RegsUsed.Clear;                                                                // 2 - Standard targets
   RunWhenDoneFiles.Clear;                                                        // 3 - CPU DEF file
//   AssemblerStatus.Show;                                                          // 4 - Main File name
   InfoList.Clear;                                                                // 5..N - Addition filss
   C := Project.Count;
   ExtendedLPM := false;
   TrimDown := 0;

   if C <= ProjectFileStart then
      begin
      Status ('No sources in project...');
      error := true;
      exit;
      end;

   SourceFiles.Clear;
   LinkProject.Assign (Project);

// Verify presence and location of source files
{   for I := 4 to LinkProject.Count-1 do
      begin
      if uppercase (ExtractFileExt (LinkProject [I])) = '.PAS' then
         LinkProject [I] := ChangeFileExt (LinkProject [I], '.ASM');

      if not FileExists (Project [I]) then
         begin
         S := LinkProject [1] + ExtractFilename (LinkProject [I]);
         if FileExists (S) then
            LinkProject[I] := S
         else if not FileExists ('Lib\' + ExtractFilename (LinkProject [I])) then
            begin
            Status (LinkProject [I] + ' not found');
            exit;
            end;
         end;
      end;
}


   LastPass := 50;
   Pass := 1;
   FinalPass := false;
   Error:=false;

   repeat
      LinkFileNumber := 1;
      LabelsChanged := false;
      NumLabelsChanged := 0;
      PointersZero;
      ExeSize := 0;
      FirstCodeSpecial := true;
      HighestConstAssemblyPointer := 0;

      if Pass = 1 then
         Status ('Assembly phase, creating RSL files (where required)')
      else if Pass = 2 then
         Status ('Validation and mapping phase')
      else
         if Pass <> LastPass then
            Status ('Fitting attempt ' + IntToStr (Pass-2));

      if Pass = LastPass then
         begin
         FinalPass := true;
         if WriteSymbolFile then
            begin
            assign (SymFile, OutputFilename + '.SYM');
            rewrite (SymFile);
            writeln (SymFile,'Embedded Pascal AVR, Format version 1');
            end;

         if COFFEnabled then
            begin
            assign (CoffLineFile, LinkProject[1] + LinkProject[0] + '.CF1');
            rewrite (CoffLineFile, 1);
            assign (CoffSymFile, LinkProject[1] + LinkProject[0] + '.CF2');
            rewrite (CoffSymFile,1);
            assign (CoffStaticFile, LinkProject[1] + LinkProject[0] + '.CF4');
            rewrite (CoffStaticFile,1);
            assign (CoffStringFile, LinkProject[1] + LinkProject[0] + '.CF3');
            rewrite (CoffStringFile,1);
            CoffSymbolIndex := 0;
            LineNumberEntries := 0;
            StaticCount := 0;
            end;

         Status ('Code creation phase');
         if OBJEnabled then
            CreateObject := true;
         C := SourceFiles.Count;
         I := 0;

         while I < C do
            begin
            if ExtractFileExt (SourceFiles [I])= '.RSL' then
               CreateObject := false;
            inc(I);
            end;

         if CreateObject then
            begin
            assign(ObjectFile,Project[prjPath]+Project[prjTitle]+'.obj');
            rewrite(ObjectFile,1);
            OBJHeader.FileNames:=0;
            OBJHeader.Records1:=0;
            OBJHeader.Records2:=0;
            OBJHeader.Records3:=0;
            OBJHeader.Records4:=$1A;
            OBJHeader.Bytes:=9;
            OBJHeader.NumberNames:=0;
            move(AVRStr,OBJHeader.AVR,16);
            BlockWrite(ObjectFile,OBJHeader,26);
            end;
         end  // if Pass = LastPass
      else
         CreateObject := false;

      I := ProjectFileStart;
      C := LinkProject.Count;
      AddLinkFiles := 0;

      while (I < C) and not Error do
         begin
         if Pass = LastPass then
            Status(ExtractFilename(LinkProject[i])+' at start Code '+IntToHex(CodeAssemblyPointer,4)+'H Data '+IntToHex(DataAssemblyPointer,2)+'H');

         if LinkProject[I] <> ExtractFilename (LinkProject[I]) then
            LinkFile(LinkProject[I])
         else
            LinkFile (LinkProject[1] + LinkProject[I]);

         if Pass = LastPass then
            Status(ExtractFilename(LinkProject[i])+' at end Code '+IntToHex(CodeAssemblyPointer,4)+'H Data '+IntToHex(DataAssemblyPointer,2)+'H');

         inc (I);
         inc (LinkFileNumber);
         C := C + AddLinkFiles;
         AddLinkFiles := 0;
         end;

      if LabelsChanged and not FinalPass then
         begin
         if Pass > 2 then
            Status ('Fitting attempt ended with ' + IntToStr (NumLabelsChanged) + ' unresolved references');
         end
      else if not LabelsChanged then
         if not FinalPass and not Error then
            Status ('Fitting attempt succeeded');
      HighestConst := HighestConstAssemblyPointer;

      if Pass = LastPass then
         begin
         if (not Error) and CreateObject then
            begin
            FP := FilePos (ObjectFile);
            C := SourceFiles.Count;
            I := 0;

            while I < C do
               begin
               S := SourceFiles[I] + #0;
//               StrToShort (SourceFiles[i], s);
//               S := S + #0;
               BlockWrite(ObjectFile, S [1], length(S));
               inc(I);
               end;

            Seek (ObjectFile, 9);
            BlockWrite (ObjectFile, C, 1);
            Seek (ObjectFile, 0);
            FP := (FP shr 24) or ((FP and $00FF0000) shr 8) or ((FP and $0000FF00) shl 8) or ((FP and $FF) shl 24);
            BlockWrite (ObjectFile, FP, 4);
            CloseFile (ObjectFile);
            end
         else if CreateObject then
            closeFile(ObjectFile);

         if WriteSymbolFile then
            closeFile(SymFile);
         end;

      inc (Pass);
      if (not FinalPass) and (not LabelsChanged) and (Pass > 2) then
         Pass := LastPass;
   until error or FinalPass;

   if CoffEnabled then
      begin
      closefile(CoffLineFile);
      closefile(CoffSymFile);
      closefile(CoffStaticFile);
      closefile(CoffStringFile);
      end;

   if not Error then
      SaveExecutable;
   if not Error then
      Status('Code merging complete. No Errors')
   else
      Status('Code creation not completed');
   if not Error then
      begin
      if CreateObject then
         Status('Created ' + OutputFilename + '.OBJ')
      else
        Status(OutputFilename + '.OBJ'+' not created due to missing source');
      Status('Created ' + OutputFilename + '.ROM');

      if TrimDown > 0 then
         Status('Trimmed first '+IntToHex(TrimDown,5)+'H bytes from output ROM file');
      Status('Created ' + OutputFilename + '.HEX');
      end;

   if WriteSymbolFile then
      begin
      if not error then
         Status('Created ' + OutputFilename + '.SYM');
      end;

   while NumberOfLinkFiles > 0 do
      begin
      Labels[NumberOfLinkFiles].free;
      dec(NumberOfLinkFiles);
      end;

   if not error then
      begin
      if E2Size>0 then
         begin
         assign(OFile,Project[prjPath]+Project[prjTitle]+'.E2R');
         rewrite(OFile,1);
         blockwrite(OFile,E2Buffer,E2Size);
         closeFile(OFile);
         Status('Project has initialized E2ROM data');
         Status('Created '+Project[prjPath]+Project[prjTitle]+'.E2R');
         end;
      assign(SymFile,Project[prjPath]+Project[prjTitle]+'.REF');
      rewrite(SymFile);
      i := 0;
      c := InfoList.count;
      while i < c do
         begin
         writeln(SymFile,InfoList[i]);
         inc(i);
         end;
      c:=ALabels.count;
      if c = 0 then
         WriteLn(SymFile,'No conditional code on reference in project')
      else
         begin
         writeln(SymFile,'Project has conditional code references');
         i := 0;
         while i < c do
            begin
            writeln(SymFile,'Referenced label: '+ALabels[i]);
            inc(i);
            end;
         end;
      closeFile(SymFile);
      Status('Created ' + OutputFilename + '.REF');
      Status('ROM size  : '+IntToStr(ExeSize)+' bytes');
      Status('E2ROM size: '+IntToStr(E2Size)+' bytes. (For image)');
      end;

   if FirstCodeSpecial then
      begin
      Showmessage('Warning: Your project does not have a constant section declared with "CODESP". One of your '+
                  'sources needs to declare the location of the contant section if it is required by your project. '+
                  'A missing "CODESP" can result in the overwriting of code by constants as code cannot be located '+
                  'after the constant section by the linker due to missing information.');
      end;
   AssemblerSource.clear;
   end;

{==============================================================================}

//**procedure LinkFile(f: string);                                               // short
procedure LinkFile (Filename: string);

//   var  CharsInBuffer: word;
//     TFile: TextFile;
var       S: string;
//       CH: char;
       IX: longint;

   label LoadAsRSL, AllOK;

   begin
//   CH := #0;
//   CIBP := 0;
   CurrentFile := Filename;
   if not FileExists (Filename) then
      begin
      if FileExists (ChangeFileExt (Filename, '.Asm')) then
         goto AllOK;

      if FileExists (ChangeFileExt (Filename, '.RSL')) then
         goto AllOK;

      S := ExtractFilename (Filename);

      if FileExists (ExePath+ 'Lib\' + ChangeFileExt (S, '.Asm')) then
        begin
        Filename := ExePath + 'Lib\' + ChangeFileExt (S, '.Asm');
        goto allOK;
        end;

      if FileExists (ExePath + 'Lib\' + changeFileExt (S, '.RSL')) then
         begin
         Filename := ExePath + 'Lib\' + ChangeFileExt (S, '.RSL');
         goto allOK;
         end;

      if FileExists (ExePath + 'Default\' + ChangeFileExt (S,'.Asm')) then
         begin
         Filename := ExePath + 'Default\' + ChangeFileExt (S, '.Asm');
         goto allOK;
         end;

      if FileExists (ExePath + 'Default\' + ChangeFileExt (S, '.RSL')) then
         begin
         Filename := ExePath + 'Default\' + ChangeFileExt (S, '.RSL');
         goto allOK;
         end;

      Status ('File "' + Filename + '" cannot be located');
      Error := true;
      exit;
      end;

AllOK:
   if Pass = 1 then
      begin
      if uppercase (ExtractFileExt (Filename)) = '.PAS' then
         Filename := ChangeFileExt(Filename, '.ASM')
      else if uppercase (ExtractFileExt(Filename)) = '.RSL' then
         goto LoadAsRSL;

{      if not FileExists (Filename) then
         begin
         S := Exepath + 'Lib\' + ExtractFilename (Filename);
         if  FileExists (S) then
            Filename := S
         else
            begin
            Status (S + ' not found');
            exit;
            end;
         end;
}
      AsmFilePath := ExtractFilePath(Filename);
      SourceFiles.Add (Filename);
      SourceFileNumber := SourceFiles.Count;
      CurrentFile := Filename;
      OpLInes.Clear;
      AssemblerSource.LoadFromFile (Filename);
      FromSource:=true;
      end
   else
      begin
LoadAsRSL:
      Application.ProcessMessages;

      if Pass = 1 then
         begin
         SourceFiles.Add (Filename);
         SourceFileNumber := SourceFiles.Count;
         end
      else
         begin
         IX := SourceFiles.IndexOf (Filename);
         if IX > -1 then
            SourceFileNumber := IX + 1
         else
            SourceFileNumber := 1;
         end;

      Filename := ChangeFileExt (Filename, '.RSL');
      CurrentFile := Filename;
//      reset (OFile, 1);
      OpLines.Clear;
      AssemblerSource.Clear;
      AssemblerSource.LoadFromFile (Filename);
//      blockread (OFile, RRCVersion, 4, Res);
//      if Res = 4 then
//         begin
//         if RRCVersion <> CurrentRRCVersion then
//            begin
//            Status ('File "'+ Filename + '" as RRC. Incompatible file version. Please re-assemble from source');
//            error := true;
//            exit;
//            end;
//         end;

//      CharsInBuffer := 0;

{      repeat
         S := '';
         repeat
            if CharsInBuffer = 0 then
               begin
               BlockRead (OFile, chb, 1024, Res);
               CharsInBuffer := Res;
               cibp := 0;
               end;

            if CharsInBuffer > 0 then
               begin
               Ch := char (chb [cibp]);
               inc (cibp);
               dec (CharsInBuffer);
               if Ch <> #$FF then
                  S := S + Ch;
               end;
         until (Ch = #$FF) or (Res = 0);

         if Res <> 0 then
            begin
            AssemblerSource.Add (S);
            end;
      until Res = 0;

      CloseFile (OFile);
}
      FromSource := false;
      end;

   if FinalPass then
      Status ('Link: ' + ExtractFilename (Filename));
//   else
//      Status ('Assembling ' + ExtractFilename (Filename) + ' - Pass ' + IntToStr (Pass));
   Application.ProcessMessages;
//   if (Pass = 1) and FromSource then
//      begin
//     AssignFile (OFile,ChangeFileExt (Filename, '.RRC'));
//     rewrite (OFile, 1);
//     RRCVersion := CurrentRRCVersion;
//     blockwrite (OFile, RRCVersion,4);
//     end;

   Error:=false;
//   RSLFIlename := ChangeFileExt (Filename, '.RSL');
   Assemble;
   if (Pass = 1) and FromSource then
      OpLines.SaveToFile (ChangeFileExt (Filename, '.RSL'));
//      begin
//      Closefile (OFile);
//      end;

   if Error then
      begin
      if Pass = 1 then
         begin
         if not CurrentFileIsInclude then
            ShowErrorPosition(Filename ,LineNumber,0)
         else
            ShowErrorPosition (IncludeFileName, IncludeLineNumber, 0);
         end
      else
         begin
         ShowErrorPosition (SourceFiles [SourceFileNumber - 1], SourceLineNumber, 0);
         end;
//      AssemblerStatus.Show;
      end;
   end;

{==============================================================================}

end.
