unit EditorFrame;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, SynEditHighlighter,
  SynHighlighterPas, SynEdit, ComCtrls, SynEditMiscClasses, SynEditSearch,
  SynEditTypes, SynEditRegexSearch;

type
  PfmEditorFrame = ^TfmEditorFrame;
  TfmEditorFrame = class(TFrame)
    SynEdit: TSynEdit;
    SynEditRegexSearch1: TSynEditRegexSearch;
    procedure SynEditChange(Sender: TObject);
    procedure SynEditStatusChange(Sender: TObject; Changes: TSynStatusChanges);
    procedure SynEditSpecialLineColors(Sender: TObject; Line: Integer;
      var Special: Boolean; var FG, BG: TColor);
  private
    { Private declarations }
    EditorFilename:   string;
    FileModified:     boolean;
    ErrorLine:        integer;
    LineToClear:      integer;

  public
    { Public declarations }

    function GetFilename: string;
    function GetLineNumber: integer;
    function HasBeenModified: boolean;
    function LineCount: integer;
    function LinesDisplayed: integer;

    procedure Initialize;
    procedure SetBackground (C: TColor);
    procedure LoadFile (Filename: string);
    procedure SaveFile;
    procedure SaveFileAs (Filename: string);
    procedure CloseFile;
    procedure SearchFor (S: string; Options: TSynSearchOptions);
    procedure Replace (S, R: string; Options: TSynSearchOptions);
    procedure Cut;
    procedure Copy;
    procedure Paste;
    procedure SetFont (F: TFont);
    procedure SetCaretPosition (X, Y: integer);
    procedure HighlightLine (Line: integer);
    procedure ClearError;


  end;

implementation

{$R *.dfm}

uses MainForm, EditorDefs;


{==============================================================================}

function TfmEditorFrame.GetFilename: string;

   begin
   Result := EditorFilename;
   end;

{==============================================================================}

function TfmeditorFrame.GetLineNumber: integer;

   begin
   Result := SynEdit.CaretY;
   end;

{==============================================================================}

function TfmEditorFrame.LineCount: integer;

   begin
   Result := SynEdit.Lines.Count;
   end;

{==============================================================================}

function TfmEditorFrame.LinesDisplayed: integer;

   begin
   Result := SynEdit.LinesInWindow;
   end;

{==============================================================================}

procedure TfmEditorFrame.SetBackground (C: TColor);

   begin
   Color := C;
   SynEdit.Color := C;
   end;

{==============================================================================}

procedure TfmEditorFrame.ClearError;

   begin
   ErrorLine := -1;
//   SynEdit.Repaint;
   end;

{==============================================================================}

procedure TfmEditorFrame.HighlightLine (Line: integer);

   var  P:  TBufferCoord;

   begin
//   SynEdit.Validate;

   P.Line := Line;
   P.Char := 1;
   SynEdit.CaretXY := P;
   ErrorLine := Line;
   LineToClear := Line;
   SynEdit.SetFocus;
//   SynEdit.SelStart := Line;
//   SynEdit.SelEnd := Line+1;
   end;

{==============================================================================}

function TfmEditorFrame.HasBeenModified: boolean;

   begin
   Result := FileModified;
   end;

{==============================================================================}

procedure TfmEditorFrame.SetCaretPosition (X, Y: integer);

   begin
   if Y < SynEdit.Lines.Count then
      begin
      SynEdit.CaretY := Y;
      if X < length (SynEdit.Lines [Y]) then
         SynEdit.CaretX := X;
      end;
   end;

{==============================================================================}

procedure TfmEditorFrame.Initialize;

   begin
   ErrorLine := -1;
   Align := alClient;
   SynEdit.Align := alClient;
   SynEdit.SearchEngine := TSynEditSearch.Create (SynEdit);
   SynEdit.Lines.Clear;
   Color := BackgroundColor;
   Visible := true;
   SynEdit.Font.Name := EditorFont.Name;
   SynEdit.Font.Size := EditorFont.Size;
   FileModified := false;
   fmMainForm.ShowFileModified (false);
   end;

{==============================================================================}

procedure TfmEditorFrame.SaveFile;

   begin
   SynEdit.Lines.SaveToFile (EditorFilename);
   FileModified := false;
   fmMainForm.ShowFileModified (false);
   end;

{==============================================================================}

procedure TfmEditorFrame.SaveFileAs (Filename: string);

   begin
   EditorFilename := Filename;
   SaveFile;
   end;

{==============================================================================}

procedure TfmEditorFrame.LoadFile (Filename: string);

   begin
   SynEdit.Lines.LoadFromFile (Filename);
   EditorFilename := Filename;
   fmMainForm.ShowFileModified (false);
   end;

{==============================================================================}

procedure TfmEditorFrame.CloseFile;

   var  S:   string;

   begin
   if FileModified then
      begin
      S := ExtractFilename (EditorFilename);
      if MessageDlg (S + ' has been modified. Save changes?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
         SaveFile;
      end;
   end;

{==============================================================================}

procedure TfmEditorFrame.SearchFor (S: string; Options: TSynSearchOptions);

   begin
//  SynEdit.SearchEngine := SynEditSearch;
  SynEdit.SearchReplace (S, '', Options);
  end;

{==============================================================================}

procedure TfmEditorFrame.Replace (S, R: string; Options: TSynSearchOptions);

   begin
   SynEdit.SearchReplace (S, R, Options);
   end;

{==============================================================================}

procedure TfmEditorFrame.Cut;

   begin
   SynEdit.CutToClipboard;
   end;

{==============================================================================}

procedure TfmEditorFrame.Copy;

   begin
   SynEdit.CopyToClipboard;
   end;

{==============================================================================}

procedure TfmEditorFrame.Paste;

   begin
   SynEdit.PasteFromClipboard;
   end;

{==============================================================================}

procedure TfmEditorFrame.SetFont (F: TFont);

   begin
   SynEdit.Font := F;
   end;

procedure TfmEditorFrame.SynEditChange(Sender: TObject);

   begin
   FileModified := true;
   fmMainForm.ShowFileModified (true);
   ClearError;
   end;


procedure TfmEditorFrame.SynEditSpecialLineColors(Sender: TObject;
  Line: Integer; var Special: Boolean; var FG, BG: TColor);

   begin
   if Line = LineToClear then
      begin
      FG := clWhite;
      BG := clRed;
      if Line = ErrorLine then
         Special := true
      else
         Special := false;
      LineToClear := -1;
      end;
   end;

procedure TfmEditorFrame.SynEditStatusChange(Sender: TObject;
  Changes: TSynStatusChanges);

   begin
   fmMainForm.StatusBar.Panels [0].Text := IntToStr (SynEdit.DisplayY) + ' : ' + IntToStr (SynEdit.DisplayX);
   ErrorLine := -1;
   end;

{==============================================================================}

   end.
