unit init;

interface

uses AsmDefs,PascalDefs,SysUtils;

procedure DoInitialize;
procedure EnterStandardFunctions;

const
  Errors : array[0..129] of string =                                             //ss [40]
          ( 'Undefined identifier',
            'Multiple definition of this identifier',
            'Expected an identifier',
            '"Procedure" or "Program" expected',
            'Expected closing parenthesis ")"',
{  5 }      'Expected a colon ":"',
            'Incorrectly used symbol',
            'Expected identifier or the symbol "VAR"',
            'Expected the symbol "OF"',
            'Expected an opening paranthesis "("',
{ 10 }      'Expected identifier, "ARRAY" or "RECORD"',
            'Expected an opening bracket "["',
            'Expected a closing bracket "]"',
            'Expected ".." without intervening blanks',
            'Expected a SEMICOLON ";"',
{ 15 }      'Unsupported function result type',
            'Expected a equal sign "="',
            'Expected a boolean expression',
            'Loop variable is of the wrong type',
            'Invalid type for loop',
{ 20 }      'Syntax error',
            'The number is to large',
            'Expect period "." or semicolon ";"',
            'Bad type for a case statement',
            'Illegal character',
{ 25 }      'Illegal constant or constant identifier',
            'Illegal array subscript (check type)',
            'Illegal bounds for an array index',
            'Indexed variable must be ARRAY or STRING',
            'Expected a type identifier',
{ 30 }      'Undefined type',
            'Var with field selector must be record',
            'Expected type "BOOLEAN"',
            'Illegal type for arithmetic expression',
            'Expected integer for "DIV" or "MOD"',
{ 35 }      'Incompatible types for comparison',
            'Parameter types do not match',
            'Expected a variable',
            'String type expected',
            'Number of parameters do not match',
{ 40 }      'Illegal parameters "READ"',
            'Illegal parameters TO "WRITE"',
            'Parameter must be of type "REAL"',
            'Parameter must be of type "INTEGER"',
            'Expected variable or constant',
{ 45 }      'Expected a variable or procedure',
            'Types must match in an assignment',
            'Expected constant or identifier',
            'Argument of std. function of wrong type',
            'Expected a constant or OVERLAYADD',
{ 50 }      'Illegal symbol for a constant',
            'Expected BECOMES ":="',
            'Expected "THEN"',
            'Expected "UNTIL"',
            'Expected "DO"',
{ 55 }      'Expected "TO" OR "DOWNTO"',
            'Expected "BEGIN"',
            'Expected "END"',
            'Expected ID, CONST, "NOT" OR "("',
            'Cannot evaluate expression',
{ 60 }      'Invalid character in source ',
            'Constant out of range',
            'Invalid type for case label',
            'Subrange types must be the same',
            '".." not permitted',
{ 65 }      'String or char expected',
            'Invalid use of External keyword',
            'Label identifier expected',
            'Goto target must be inside current block',
            'Expected caret (^) on left side',
{ 70 }      'Expected caret (^) on right side',
            'Caret on both sides ???',
            'Expected a pointer variable',
            'Invalid type for pointer assignment',
            'Main procedure cannot have parameters',
{ 75 }      'Typed constant cannot be pointer',
            'Expected TRUE or FALSE',
            'Illegal constant',
            'Expected a comma ","',
            'Expected correct field name',
{ 80 }      'Unsupported operation',
            'Unsupported standard pascal function',
            'Invalid target for inc/dec',
            'Numeric constant or variable expected',
            'Illegal type in expession',
{ 85 }      'Longint not permitted here',
            'Invalid filename',
            'Include file not found',
            'To many nested include files',
            'Invalid compiler directive',
{ 90 }      'Continue statement not applicable here',
            'Break statement not applicable here',
            'Cannot evaluate expression',
            'Expected constant or CODEADD',
            'Expected constant or DATAADD',
{ 95 }      'I/O or bit address uncertain',
            'String constant exceeds line',
            'Word or Integer required to store pointer',
            'Illegal E2 type reference. Use E2Blockread',
            'Expected reference to E2 variable',
{100}       'Invalid I/O address ($20-$5F expected)',
            'Invalid word',
            'Unterminated conditional (missing endif)',
            'Unexpected end of sourcefile',
            'Expected LONGINT',
{105}       'Expected RECORD',
            'Invalid typecast (object size missmatch)',
            'Invalid typecast (object not castable)',
            'To many dimensions (more than defined)',
            'Invalid device definition file',
{110}       'To many forward declarations',
            'Compile canceled by user',
            'Direct access of E2 variable',
            'Expected a character',
            'Type missmatch in FP operation',
{115}       'Unsupported compare operation (Types ?)',
            'Expected CODE, DATA, or PREV',
            'Destination cannot be code memory',
            'Expected bit address 0-7',
            'Memory allocation error. File too large',
{120}       'Top level only (no local procs/funcs)',
            'Sharing is active. Use {#S-}',
            'Dynamic vars active. Use (#K-}',
            'Error 123',
            'Resolution not possible (overflow)',
{125}       'Expected "Interface"',
            'Expected identifier for TShare type',
            'Declaration differs from forward',
            'GLOBAL keyword not allowed here',
            'EXTERNAL keyword not allowed here');

implementation

uses BlockA,AsmStatus,Compiler, StringConversion;

(*-------------------------------------------------------INITIALIZE---*)

procedure DoInitialize;

var
  C : Char;
  n: word;

begin

(*
    =================
     character types
    =================
 *)

  SPS['+'] := PLUS;        SPS['-'] := MINUS;
  SPS['*'] := TIMES;       SPS['/'] := RDIV;
  SPS['('] := LPARENT;     SPS[')'] := RPARENT;
  SPS['='] := EQL;         SPS[','] := COMMA;
  SPS['['] := LBRACK;      SPS[']'] := RBRACK;
  SPS['"'] := NEQ;         SPS['&'] := ANDSY;
  SPS[';'] := SEMICOLON;

  for C := CHR( ORDMINCHAR ) to CHR( ORDMAXCHAR ) do case C of

    'A'..'Z','@' : CharTypes[C] := LETTER;
    'a'..'z' : CharTypes[C] := LOWCASE;
    '0'..'9' : CharTypes[C] := NUMBER;

    '+', '-', '*', '/', '(', ')', '$', '=', ' ', ',',
    '.', '''','[', ']', ':', '^', '_', ';', '{', '}',
    '<', '>', '%', '#', '!' : CharTypes[C] := SPECIAL;

    else CharTypes[C] := ILLEGAL;

  end;

(*
    ===========
       Sets
    ===========
*)

  CONSTBEGSYS := [ PLUS,MINUS,BYTECON,WORDCON,INTCON,LINTCON,SINGLECON,CHARCON,STRINGCON,IDENT    ];
  TYPEBEGSYS  := [ IDENT,ARRAYSY,RECORDSY,STRINGSY,CARET];
  BLOCKBEGSYS := [ CONSTSY,E2CONSTSY,TYPESY,VARSY,E2VARSY,PROCSY,FUNCSY,BEGINSY,LABELSY ];
  FACBEGSYS   := [ BYTECON,INTCON,WORDCON,LINTCON,SINGLECON,CHARCON,STRINGCON,IDENT,LPARENT,NOTSY,MINUS,PLUS];
  STATBEGSYS  := [ BEGINSY,WITHSY,IFSY,WHILESY,REPEATSY,FORSY,CASESY,GOTOSY,ASMSY,EXITSY,BREAKSY,CONTINUESY ];
  StandardTypes    := [ NOTYP,BYTES,WORDS,INTS,LINTS,SINGLES,BOOLS,CHARS,CSTRINGS];

(*
   ===========
     Scalars
   ===========
*)

  ProgramLocationCounter := 0;
  LineLength := 0;
  CharacterCounter := 0;
  LastCharFromSource := ' ';
  ERRPOS :=  0;
  IdTableIndex := -1;
  ArrayTableIndex :=  0;
  BlockTableIndex :=  1;
  DISPLAY[0] := 1;
  SKIPFLAG := FALSE;
  LINECOUNT := -1;

  LastLabelID:=1;
  CurrentProcedureIndex:=0;
  CallParameters:=false;
  AsmOutput.Clear;
  AsmConstList.Clear;
  Externals.Clear;
  Globals.Clear;
  StringConstants.Clear;
  Forwards.Clear;
  IdentifierNames.Clear;
  StringConstantsUsed.Clear;
  DataSize:=0;
  E2DataSize:=0;
  CodeSize:=0;
  CurrentIncludeFile:=0;
  FirstEOL:=true;
  ParserLookAhead:='';
  DelphiOnlyComment:=false;
  ParserOptimizer:=true;
  InsideConstantDeclaration:=false;
  SharedData:=false;
  NextVarResetsShare:=false;
  ReferenceList.Clear;
//  BList.Clear;
  ConditionalSkip:=false;
  ConditionalElse:=false;
  AllowAnycast:=false;
  MacroCall.clear;
  NumberOfRecordings:=0;
  for n:=0 to NumberOfRecordingPointers do RecordingPointer[n].f:=0;
  ProgramNameInLabel:=true;
  TCast.Typ:=NOTYP;
  ErrorLine:=0;
  LastLineNumber:=$FFFFFFF;
  PointerBend:=DataMem;
  PreviousPointerBend:=DataMem;
  PreviousDynamicVars:=false;
  DynamicVars:=false;
  MaxStringSize:=0;
  UsesAddStrings:=false;
  RegisterTrack:=true;
  InsideUsesClause:=false;
  IntLineLength:=0;
  NumberOfWithPointers:=0;
  for n:=1 to NumberOfIncludeFiles do
    IncludeFile[n].IncludeErrorLine:=0;


    end; { INITIALIZE }

{==============================================================================}

procedure EnterStandardFunctions;

(*--------------------------------------------------------ENTER-----
   the following procedures enter the apropriate type
   into the associated table for that type.
*)
{====================}

      procedure ENTER( X0: string; X1: POBJECT; X2: TYPES; X3,X4: word);             //ss  short

         begin
         IdTableIndex := IdTableIndex+1;       (* enter standard identifier *)
         with IdentifierTable[IdTableIndex] do
            begin
            {NAME := X0;}
            NameLookup := IdentifierNames.Count;                                  // HASID = word
            IdentifierNames.Add(X0);
            HASHID := GetHashId (X0);

//mov       move(X0,HASHID,2);
//       StrToShort (X0, Str);
//       move (Str, HASHID, 2);
            LINK := IdTableIndex-1;
            OBJ  := X1;
            TYP  := X2;
            REF  := X4;
            NORMAL := TRUE;
            LEV  := 0;
            ADR  := X3;
            Value:= X3;
            Size := X3;
         end;
      end; { ENTER }

{====================}

      procedure LoadFixedDefs;          // Load CPU INC definition file

         type
            TDefType = (dtNone, dtIO, dtBit, dtVector, dtCPU, dtCode);

         var  I: word;
              V: longint;
              S: string;                                                                   //ss [128];
              TFile: TextFile;
              DefType: TDefType;

         label   LineDone;

{==========}

         function ParseDef: boolean;

            var  Done: boolean;
                 C: char;

            begin
            result := false;
            done := false;
            CItem := '';
            while (I <= length(StringAssembly)) and ((Stringassembly[I] = ' ') or (StringAssembly[I] = #9)) do
               inc(I);
            while (I <= length(StringAssembly)) and not Done do
               begin
               C := StringAssembly[I];
               inc(I);
               if C = ';' then
                  Done := true
               else if C = ' ' then
                  Done := true
               else if C = #9 then
                  Done := true
               else if  c = '=' then
                  begin
                  if CItem <> '' then
                     begin
                     dec(I);
                     Done := true;
                     end
                  else
                     begin
                     CItem := '=';
                     Done := true;
                     end;
                  end
               else
                  CItem := CItem + C;
               end;
            if CItem <> '' then
               result := true;
            end;

{==========}

         begin  // LoadFixedDefs \\
         AdvancedLPM := false;
         if FileExists(ExePath + 'lib\' + Project[3]) then
            begin
            assign(TFile, ExePath + 'lib\' + Project[3]);
            reset(TFile);
//            Mode := 0;
            DefType := dtNone;
            DeviceCode := 'Unknown';
            IDAtIODefStart := IDTableIndex;
            while not eof(TFile) do
               begin
               readln(TFile, StringAssembly);
               StringAssembly := uppercase(StringAssembly);
               if length(StringAssembly) > 0 then
                  begin
                  I := 1;
                  while ParseDef do
                     begin
                     if CItem = ';' then
                        goto LineDone
                     else if CItem = '.DEVICE' then
                        begin
                        if ParseDef then
                           begin
                           DeviceCode := CItem;
                           goto LineDone;
                           end;
                        end;

                     if CItem = '.IODEF' then
                        DefType := dtIO
//                        Mode := 1
                     else if CItem = '.BITDEF' then
                        DefType := dtBit
//                        Mode := 2
                     else if CItem = '.VECTORS' then
                        DefType := dtVector
//                        Mode := 3
                     else if CItem = '.CPUDEF' then
                        Deftype := dtCPU
//                        Mode := 4
                     else if CItem = '.CODEDEF' then
                        DefType := dtCode;
//                        Mode := 5;

                     if CItem = '.EQU' then
                        begin
                        if ParseDef then
                           begin
                           S := '_' + CItem;
                           if ParseDef then
                              begin
                              if CItem = '=' then
                                 begin
                                 if ParseDef then
                                    begin
                                    if TryExpression(V) then
                                       begin
                                       case DefType of
//                                       if DefType = dtIO then
  //                                     if Mode = 1 then
                                       dtIO:
                                          begin
                                          if (V >= 0) and (V <= $3F) then
                                             begin
                                             ENTER(S, TVariable, Bytes, 1, 0); //io def
                                             with IdentifierTable[IdTableIndex] do
                                                begin
                                                Typ := Bytes;
                                                Obj := TVariable;
                                                Ref := 0;
                                                IsAbsolute := 1;
                                                Address := V + $20;
                                                end;
                                             end
                                          else if V >= $60 then
                                             begin
                                             ENTER(S, TVariable, Bytes, 1, 0); //io def
                                             with IdentifierTable[IdTableIndex] do
                                                begin
                                                Typ := Bytes;
                                                Obj := TVariable;
                                                Ref := 0;
                                                IsAbsolute := 1;
                                                Address := V;
                                                end;
                                             end;
                                          end;

                                       dtBit: //else if DefType = dtBit then //Mode = 2 then
                                          begin
                                          if (V >= 0) and (V <= 7) then
                                             begin
                                             ENTER(S, TConstant, Bytes, 1, 0); //bit address
                                             IdentifierTable[IdTableIndex].TYP := Bytes;
                                             IdentifierTable[IdTableIndex].IsAbsolute := 1;
                                             IdentifierTable[IdTableIndex].Address := BitMask[V];
                                             IdentifierTable[IdTableIndex].Value := V;
                                             IdentifierTable[IdTableIndex].REF := 0;
                                             ENTER('_' + S, TConstant, Bytes, 1, 0); //bit address
                                             IdentifierTable[IdTableIndex].TYP := Bytes;
                                             IdentifierTable[IdTableIndex].IsAbsolute := 1;
                                             IdentifierTable[IdTableIndex].Address := BitMask[V];
                                             IdentifierTable[IdTableIndex].Value := BitMask[V];
                                             IdentifierTable[IdTableIndex].REF := 0;
                                             end;
                                          end;
                                       dtVector: //else if DefType = dtVector then //Mode = 3 then
                                          begin
                                          ENTER(S, TVariable, Bytes, 1, 0); //vector address
                                          IdentifierTable[IdTableIndex].IsAbsolute := 1;
                                          IdentifierTable[IdTableIndex].Address := V * 2;
                                          end;
                                       dtCPU: //else if DefType = dtCPU then //Mode = 4 then
                                          begin
                                          ENTER(S, TConstant, Lints, 1, 0); //vector address
                                          IdentifierTable[IdTableIndex].TYP := Lints;
                                          IdentifierTable[IdTableIndex].Value := V;
                                          IdentifierTable[IdTableIndex].ADR := V;
                                          IdentifierTable[IdTableIndex].Address := V;
                                          IdentifierTable[IdTableIndex].REF := 0;
                                          //IdentifierTable[IdTableIndex].IsAbsolute := 1;
                                          end;
                                       dtCode: //else if DefType = dtCode then // Mode = 5 then
                                          begin
//                                          ENTER(S, TConstant, Lints, 1, 0); //vector address
//                                          IdentifierTable[IdTableIndex].TYP := Lints;
//                                          IdentifierTable[IdTableIndex].Value := V;
//                                          IdentifierTable[IdTableIndex].ADR := V;
//                                          IdentifierTable[IdTableIndex].REF := 0;
                                          if S = '_ADVANCEDLPM' then
                                             if V > 0 then
                                                AdvancedLPM := true;
                                          end;
                                       end;
                                       goto LineDone;
                                       end;
                                    end;
                                 end;
                              end;
                           end;
                        end;
                     end;
LineDone:
                  end;
               end;
            CloseFile(TFile);
            end
         else
            begin
            CError(109);
            end;
         end;

{====================}

   begin  // EnterStandardFunctions \\
   ENTERARRAY(INTS, 0, 255); {default result type for functions returning strings}
   ArrayTable[1].ELTYP := CHARS;
   ENTER('',  TVARIABLE, NOTYP,  0, 0);
   ENTER('FALSE', TCONSTANT, BOOLS,  0, 0);
   ENTER('TRUE', TCONSTANT, BOOLS,  1, 0);
   ENTER('NIL', TCONSTANT, WORDS, 0, 0);
   ENTER('REAL', TTYPE, SINGLES,  4, 0);
   ENTER('SINGLE', TTYPE, SINGLES, 4, 0);
   ENTER('CHAR', TTYPE, CHARS,  1, 0);
   ENTER('BOOLEAN', TTYPE, BOOLS,  1, 0);
   ENTER('INTEGER', TTYPE, INTS ,  2,0 );
   ENTER('BYTE', TTYPE, BYTES,  1,0 );
   ENTER('LONGINT', TTYPE, LINTS,  4,0 );
   ENTER('WORD', TTYPE, WORDS, 2, 0 );
   ENTER('SHORTINT', TTYPE, SINTS, 1, 0);
   ENTER('TRIPLET', TTYPE, TRIPPLETS, 3,0);
   ENTER('POINTER', TTYPE, WORDS, 2, 0 );
   ENTER('TSHARE', TTYPE, TSHARE, 0, 0 );
   ENTER('ABS',  TFunc, SINGLES,  0, 0 );
  {ENTER('SQR',  TFunc, REALS,  2,0 );}
  ENTER('ODD',  TFunc, BOOLS,  4,0 );
  ENTER('CHR',  TFunc, CHARS,  5,0 );
  ENTER('ORD',  TFunc, LINTS,   6,0 );
  ENTER('SUCC',  TFunc, CHARS,  7,0 );
  ENTER('PRED',  TFunc, CHARS,  8,0 );
  {ENTER('ROUND',  TFunc, INTS,   9,0 );
  ENTER('TRUNC',  TFunc, INTS,  10,0 );
  ENTER('SIN',  TFunc, REALS, 11,0 );
  ENTER('COS',  TFunc, REALS, 12,0 );
  ENTER('EXP',  TFunc, REALS, 13,0 );
  ENTER('LN',  TFunc, REALS, 14,0 );
  ENTER('SQRT',  TFunc, REALS, 15,0 );
  ENTER('ARCTAN',  TFunc, REALS, 16,0 );
  ENTER('EOF',  TFunc, BOOLS, 17,0 );
  ENTER('EOLN',  TFunc, BOOLS, 18,0 );
  ENTER('RANDOM',  TFunc, INTS,  19,0 );}
  ENTER('SIZEOF',  TFunc, LINTS,  20, 0 );
  ENTER('INP',  TFunc, BYTES,  21, 0 );
  ENTER('NEW',  TProcs, NOTYP, 13, 0 );
  ENTER('E2READ', TFunc, INTS, 22, 0 );
  ENTER('ADDR', TFunc, WORDS, 23, 0 );
  ENTER('@', TFunc, WORDS, 24, 0 );
  ENTER('INC', TProcs, NOTYP, 5, 0 );
  ENTER('DEC', TProcs, NOTYP, 6, 0 );
  ENTER('MOVE', TProcs, NOTYP, 7, 0 );
  ENTER('FILLCHAR', TProcs, NOTYP, 8, 0 );
  ENTER('E2WRITE', TProcs, NOTYP, 9, 0 );
  ENTER('E2BLOCKREAD', TProcs, NOTYP, 10, 0 );
  ENTER('E2BLOCKWRITE', TProcs, NOTYP, 17, 0 );
  ENTER('LENGTH', TFunc, BYTES, 25, 0 );
  ENTER('COPY', TFunc, ARRAYS, 26, 1);
  ENTER('INTTOSTR',TFunc, ARRAYS, 27, 1);
  ENTER('INTTOHEX',TFunc, ARRAYS, 28, 1);
  ENTER('INSERT',TProcs, NOTYP, 11, 0);
  ENTER('DELETE',TProcs, NOTYP, 12, 0);
  ENTER('STRTOINT',TFunc, LINTS, 29, 0);
  ENTER('STRTOINTDEF',TFunc, LINTS, 30, 0);
  ENTER('LO',TFunc, BYTES, 31, 0);
  ENTER('HI',TFunc, BYTES, 32, 0);
  ENTER('LOWORD',TFunc, INTS, 33, 0);
  ENTER('HIWORD',TFunc, INTS, 34, 0);
  ENTER('UPCASE',TFunc, CHARS, 35, 0);
  ENTER('POS',TFunc, INTS, 36, 0);
  ENTER('POSN',TFunc, INTS, 37, 0);
  ENTER('MARK',TProcs, NOTYP, 14, 0);
  ENTER('RELEASE',TProcs, NOTYP, 15, 0);
  ENTER('GETMEM',TProcs, NOTYP, 16, 0);
  ENTER('WDR',TProcs, NOTYP, 18, 0);
  ENTER('SLEEP',TProcs, NOTYP, 19, 0);
  ENTER('OUT',TProcs, NOTYP, 20, 0);
  ENTER('ASINT',TFunc, INTS, 39, 0);
  ENTER('ASWORD',TFunc, WORDS, 40, 0);
  ENTER('ASBYTE',TFunc, BYTES, 41, 0);
  ENTER('ASLONGINT',TFunc, LINTS, 42, 0);
  ENTER('SWAP',TFunc, BYTES, 45, 0);
  ENTER('IOBITHIGH',TFunc, BOOLS, 46, 0);
  ENTER('IOBITLOW',TFunc, BOOLS, 47, 0);
  ENTER('SWAP_WORD',TFunc, WORDS, 43, 0);
  ENTER('SWAP_LONGINT',TFunc, LINTS, 44, 0);
  ENTER('INTERRUPT_ENTRY',TProcs, NOTYP, 21, 0);
  ENTER('INTERRUPT_EXIT',TProcs, NOTYP, 22, 0);
  ENTER('EI',TProcs, NOTYP, 23, 0);
  ENTER('DI',TProcs, NOTYP, 24, 0);
  ENTER('SETIOBIT',TProcs, NOTYP, 25, 0);
  ENTER('CLRIOBIT',TProcs, NOTYP, 26, 0);
  ENTER('EXTENDTOINTEGER',TFunc, INTS, 52, 0);
  ENTER('EXTENDTOLONGINT',TFunc, LINTS, 53, 0);
  ENTER('BITLOW',TFunc, BOOLS, 54, 0);
  ENTER('BITHI',TFunc, BOOLS, 55, 0);
  ENTER('TOGGLEBIT',TProcs, NOTYP, 27, 0);
  ENTER('STRADD',TProcs, NOTYP, 46, 0);
  ENTER('BREAKER',TProcs, NOTYP, 47, 0);
  ENTER('HEAPPTR',TVariable,words,2,0);
  IdentifierTable[IdTableIndex].Extern := true;
  IdentifierTable[IdTableIndex].TypedConstant := false;
  IdentifierTable[IdTableIndex].Block := 0;
  LoadFixedDefs;
  ENTER('', TProcs, NOTYP,  0,0 );
   end; { ENTERSTDFCNS }

end.

