//
//      M01.PAS CONTAINS FUNCTIONS TO CREATE MACHINE CODE FROM ASSEMBLY
//
{==============================================================================}

procedure RegisterTrack(R: byte);    // R is register code 0 to 31

   var  W: cardinal;                                                                       //longint

   begin
   if CurrentTrackingVariable >= 0 then
      begin
      W := cardinal(RegsUsed.Objects[CurrentTrackingVariable]) or LongMask[R];                  //longint
      RegsUsed.Objects[CurrentTrackingVariable] := pointer(W);
      end;
   end;

{==============================================================================}

function IsRegister: boolean;

   begin
   if ExpressionType = ExRegister8 then
      result:=true
   else if ExpressionType = ExExpression then
      begin
      if FinalPass then
         if not (Numeric in [0..31]) then //(Numeric < 0) or (Numeric > $1F) then
            MakeError(32);
      RegisterCode := Numeric and $1F;
      Result:=true;
      end
   else
      result:=false;
   end;

{==============================================================================}

procedure EndCurrentMap;

   begin
   case ActiveArea of
      CodeActive:
         CodeMap[CurrentCodeMap].ToAdr := CodeAssemblyPointer;
      DataActive:
         DataMap[CurrentDataMap].ToAdr := DataAssemblyPointer;
      ABSActive:
         ABSMap[CurrentABSMap].ToAdr := ABSAssemblyPointer;
      E2Active:
         E2Map[CurrentE2Map].ToAdr := E2AssemblyPointer;
      ConstActive:
         ConstMap[CurrentConstMap].ToAdr := ConstAssemblyPointer;
      end;
   end;

{==============================================================================}

procedure D_ABS;

   begin
   if FinalPass then
      EndCurrentMap;

   if GetExpressionNoComma then
      begin
      if ExpressionType = ExExpression then
         begin
         if ByteInStorage then
            InsertByte(0);
         InstructionError := false;
         AssemblyPointer := Numeric;
         ABSAssemblyPointer := Numeric;
         ActiveArea := ABSActive;
         end;
      end;

   if not FinalPass then
      exit;
   if NumberOfABSMaps > 0 then
      begin
      if AssemblyPointer <> ABSMap[CurrentABSMap].ToAdr then
         begin
         if CurrentABSMap < NumberOfMaps then
            begin
            inc(CurrentABSMap);
            inc(NumberOfABSMaps);
            ABSMap[CurrentABSMap].FromAdr := AssemblyPointer;
            ABSMap[CurrentABSMap].ToAdr := AssemblyPointer;
            end;
         end;
      end
   else
      begin
      ABSMap[CurrentABSMap].FromAdr := AssemblyPointer;
      ABSMap[CurrentABSMap].ToAdr := AssemblyPointer;
      inc(NumberOfABSMaps);
      end;
   end;

{==============================================================================}

function RegSplit55(D, R: word): word;

   begin
//rdddddrrrr
   result:=((D and $1F) shl 4) or (R and $F) or ((R and $10) shl 5);
   end;

function RegSplit44(d,r: word): word;
begin
//ddddrrrr
  result:=((d and $F) shl 4) or (r and $F);
end;

function RegSplit65(d,p: word): word;
begin
//rdddddrrrr
  result:=((d and $1F) shl 4) or (p and $F) or ((p and $30) shl 5);
end;

function RegConstant(r,v: word): word;
begin
//vvvvrrrrvvvv
  result:=((r-16) shl 4) or (v and $F) or ((v and $F0) shl 4);
end;

{==============================================================================}

procedure A_ADC;

   var  Reg: word;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         Reg := RegisterCode;
         RegisterTrack(RegisterCode);
         if GetExpressionNoComma then
            begin
            if IsRegister then
               begin
               Code2($1C00 or RegSplit55(Reg, RegisterCode));
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure A_ADD;

   var  Reg: word;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         Reg := RegisterCode;
         RegisterTrack(RegisterCode);
         if GetExpressionNoComma then
            begin
            if IsRegister then
               begin
               Code2($0C00 or RegSplit55(Reg, RegisterCode));
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure A_ADIW;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         if RegisterCode in [24,26,28,30] then
            begin
            RegisterTrack(RegisterCode);
            RegisterTrack(RegisterCode+1);
            if GetExpressionNoComma then
               begin
               if ExpressionType=ExExpression then
                  begin
                  if Numeric in [0..63] then // (Numeric>=0) and (Numeric<=63) then
                     begin
                     Code2($9600 or ((RegisterCode-24) shl 3) or (Numeric and $F) or ((Numeric and $30) shl 2));
                     end
                  else
                     MakeError(26);
                  end;
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure A_AND;

   var  Reg: word;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         Reg := RegisterCode;
         RegisterTrack(RegisterCode);
         if GetExpressionNoComma then
            begin
            if IsRegister then
               begin
               Code2($2000 or RegSplit55(Reg, RegisterCode));
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure A_ANDI;

   begin
   if GetExpressionComma then
      if IsRegister then
         if RegisterCode > 15 then
            RegisterTrack(RegisterCode);
            if GetExpressionNoComma then
               if ExpressionType = ExExpression then
                  begin
                  if (Numeric >= 0) and (Numeric <= 255) then
                     Code2($7000 or RegConstant(RegisterCode,Numeric))
                  else
                     MakeError(26);
                  end;
   end;

{==============================================================================}

procedure A_ASR;

   begin
   if GetExpressionNoComma then
      begin
      if IsRegister then
         begin
         RegisterTrack(RegisterCode);
         Code2($9405 or (RegisterCode shl 4));
         end;
      end;
   end;

{==============================================================================}

procedure A_BCLR;

   begin
   if GetExpressionNoComma then
      begin
      if ExpressionType=ExExpression then
         begin
         if Numeric in [0..7] then //(Numeric>=0) and (Numeric<=7) then
            begin
            Code2($9488 or (Numeric shl 4));
            end
         else
            MakeError(26);
         end;
      end;
   end;

{==============================================================================}

procedure A_BLD;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         RegisterTrack(RegisterCode);
         if GetExpressionNoComma then
            begin
            if ExpressionType = ExExpression then
               begin
               if Numeric in [0..7] then  // Numeric>=0) and (Numeric<=7) then
                  begin
                  Code2($F800 or (RegisterCode shl 4) or Numeric);
                  end
               else
                  MakeError(26);
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure A_BRBC;

   var  Bit: word;

   begin
   if GetExpressionComma then
      begin
      if ExpressionType = ExExpression then
         begin
         if Numeric in [0..7] then //(Numeric>=0) and (Numeric<=7) then
            begin
            Bit := Numeric;
            if GetExpressionNoComma then
               begin
               if ExpressionType = ExExpression then
                  begin
                  if RelativeRange(Dollar+2, Numeric) then
                     begin
                     Code2($F400 or (BranchDisplacement shl 3) or Bit);
                     end;
                  end;
               end;
            end
         else
            MakeError(26);
         end;
      end;
   end;

{==============================================================================}

procedure A_BRBS;

   var  Bit: word;

   begin
   if GetExpressionComma then
      begin
      if ExpressionType = ExExpression then
         begin
         if Numeric in [0..7] then //(Numeric>=0) and (Numeric<=7) then
            begin
            Bit := Numeric;
            if getExpressionNoComma then
               begin
               if ExpressionType = ExExpression then
                  begin
                  if RelativeRange(Dollar+2, Numeric) then
                     begin
                     Code2($F000 or (BranchDisplacement shl 3) or Bit);
                     end;
                  end;
               end;
            end
         else
            MakeError(26);
         end;
      end;
   end;

{==============================================================================}

procedure D_CODE;                                                                // Called from Assemble

   begin
   if FinalPass then
      EndCurrentMap;

   if ByteInStorage then
      InsertByte(0);

   InstructionError := false;
   AssemblyPointer := CodeAssemblyPointer;
   ActiveArea := CodeActive;

   if not FinalPass then
      exit;

   if NumberOfCodeMaps > 0 then
      begin
      if AssemblyPointer <> CodeMap[CurrentCodeMap].ToAdr then
         begin
         if CurrentCodeMap < NumberOfMaps then
            begin
            inc(CurrentCodeMap);
            inc(NumberOfCodeMaps);
            CodeMap[CurrentCodeMap].FromAdr := AssemblyPointer;
            CodeMap[CurrentCodeMap].ToAdr := AssemblyPointer;
            end;
         end;
      end
   else
      begin
      CodeMap[CurrentCodeMap].FromAdr := AssemblyPointer;
      CodeMap[CurrentCodeMap].ToAdr := AssemblyPointer;
      inc(NumberOfCodeMaps);
      end;
   end;

{==============================================================================}

procedure D_CODESP;

   begin
   if FinalPass then
      EndCurrentMap;
   if ByteInStorage then
      InsertByte(0);

   if FirstCodeSpecial then
      begin
      FirstCodeSpecial := false;
      AssemblyPointer := HighestConst;
      if odd(AssemblyPointer) then
         inc(AssemblyPointer);
      end
   else
      begin
      AssemblyPointer := CodeAssemblyPointer;
      MakeError(24);
      exit;
      end;

   InstructionError := false;
   CodeAssemblyPointer := AssemblyPointer;
   ActiveArea := CodeActive;
   if not FinalPass then
      exit;
   if NumberOfCodeMaps > 0 then
      begin
      if AssemblyPointer <> CodeMap[CurrentCodeMap].ToAdr then
         begin
         if CurrentCodeMap < NumberOfMaps then
            begin
            inc(CurrentCodeMap);
            inc(NumberOfCodeMaps);
            CodeMap[CurrentCodeMap].FromAdr := AssemblyPointer;
            CodeMap[CurrentCodeMap].ToAdr := AssemblyPointer;
            end;
         end;
      end
   else
      begin
      CodeMap[CurrentCodeMap].FromAdr := AssemblyPointer;
      CodeMap[CurrentCodeMap].ToAdr := AssemblyPointer;
      inc(NumberOfCodeMaps);
      end;
   end;

{==============================================================================}

procedure D_CONST;

   begin
   if FinalPass then
      EndCurrentMap;
//  if ActiveArea<>ConstActive then
//    if ByteInStorage then InsertByte(0);
   InstructionError := false;
   if GetExpressionNoComma then
      begin
      if ExpressionType = ExExpression then
         begin
         ConstAssemblyPointer := Numeric;
         AssemblyPointer := Numeric;
         end;
      end
   else
      begin
      if (CodeAssemblyPointer>0) and (ConstAssemblyPointer=0) then
         AssemblyPointer := CodeAssemblyPointer
      else
         AssemblyPointer := ConstAssemblyPointer;

      ConstAssemblyPointer := AssemblyPointer;
      end;

   if AssemblyPointer > HighestConstAssemblyPointer then
      HighestConstAssemblyPointer := AssemblyPointer;
   ActiveArea := ConstActive;
   if not FinalPass then
      exit;
   if NumberOfConstMaps > 0 then
      begin
      if AssemblyPointer<>ConstMap[CurrentConstMap].ToAdr then
         begin
         if CurrentConstMap < NumberOfMaps then
            begin
            inc(CurrentConstMap);
            inc(NumberOfConstMaps);
            ConstMap[CurrentConstMap].FromAdr := AssemblyPointer;
            ConstMap[CurrentConstMap].ToAdr := AssemblyPointer;
            end;
         end;
      end
   else
      begin
      ConstMap[CurrentConstMap].FromAdr := AssemblyPointer;
      ConstMap[CurrentConstMap].ToAdr := AssemblyPointer;
      inc(NumberOfConstMaps);
      end;
   end;

{==============================================================================}

procedure D_E2Data;

   begin
   if FinalPass then
      begin
      EndCurrentMap;
      end;

   if ByteInStorage then
      InsertByte(0);
   InstructionError := false;
   AssemblyPointer := E2AssemblyPointer;
   ActiveArea := E2Active;
   if not FinalPass then
      exit;

   if NumberOfE2Maps > 0 then
      begin
      if AssemblyPointer <> E2Map[CurrentE2Map].ToAdr then
         begin
         if CurrentE2Map < NumberOfMaps then
            begin
            inc(CurrentE2Map);
            inc(NumberOfE2Maps);
            E2Map[CurrentE2Map].FromAdr := AssemblyPointer;
            E2Map[CurrentE2Map].ToAdr := AssemblyPointer;
            end;
         end;
      end
   else
      begin
      E2Map[CurrentE2Map].FromAdr := AssemblyPointer;
      E2Map[CurrentE2Map].ToAdr := AssemblyPointer;
      inc(NumberOfE2Maps);
      end;
   end;

{==============================================================================}

procedure A_BRCC;

   begin
   if GetExpressionNoComma then
      begin
      if ExpressionType=ExExpression then
         begin
         if RelativeRange(Dollar+2, Numeric) then
            begin
            Code2($F400 or (BranchDisplacement shl 3));
            end
         else
            begin
            if not FinalPass then
               Code2(0);
            end;
         end;
      end;
   end;


procedure A_BRCS;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F000 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BREQ;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F001 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRGE;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F404 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRHC;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F405 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRHS;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F005 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRID;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F407 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRIE;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F007 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRLO;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F000 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure D_DATA;
begin
  if FinalPass then
  begin
    EndCurrentMap;
  end;
  if ByteInStorage then InsertByte(0);
  InstructionError:=false;
  AssemblyPointer:=DataAssemblyPointer;
  ActiveArea:=DataActive;
  if not FinalPass then exit;
  if NumberOfDataMaps>0 then
  begin
    if AssemblyPointer<>DataMap[CurrentdataMap].ToAdr then
    begin
      if CurrentDataMap<NumberOfMaps then
      begin
        Inc(CurrentDataMap);
        inc(NumberOfDataMaps);
        DataMap[CurrentDataMap].FromAdr:=AssemblyPointer;
        DataMap[CurrentDataMap].ToAdr:=AssemblyPointer;
      end;
    end;
  end else
  begin
    DataMap[CurrentDataMap].FromAdr:=AssemblyPointer;
    DataMap[CurrentDataMap].ToAdr:=AssemblyPointer;
    inc(NumberOfDataMaps);
  end;
end;

procedure A_BRLT;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F004 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRMI;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F002 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRNE;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F401 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRPL;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F402 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRSH;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F400 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRTC;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F406 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRTS;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F006 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure D_END;
begin
  if pass=1 then Endit:=true;
  InstructionError:=false;
end;

procedure A_BRVC;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F403 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

procedure A_BRVS;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F003 or (BranchDisplacement shl 3));
      end else
      begin
        if not finalpass then Code2(0);
      end;
    end;
  end;
end;

{==============================================================================}

procedure D_EXT;

   begin
   while ParseItem(spSpaceIsTerminator) do
      if Pass = 1 then
         AddLabel(CItem, 0, ltExternal);
   InstructionError := false;
   end;

{==============================================================================}

procedure D_EXTERN;

   begin
   D_Ext;
   end;

procedure A_BSET;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if (Numeric>=0) and (Numeric<=7) then
      begin
        Code2($9408 or (Numeric shl 4));
      end else MakeError(26);
    end;
  end;
end;

procedure A_BST;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      if GetExpressionNoComma then
      begin
        if ExpressionType=ExExpression then
        begin
          if Numeric in [0..7] then //(Numeric>=0) and (Numeric<=7) then
          begin
            Code2($FA00 or (RegisterCode shl 4) or Numeric);
          end else MakeError(26);
        end;
      end;
    end;
  end;
end;

procedure A_CALL;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      Code2($940E);
      InsertWord(Numeric shr 1);
    end;
  end;
end;

procedure A_CBI;
var
  IO: word;
begin
  if GetExpressionComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if Numeric in [0..31] then //(Numeric>=0) and (Numeric<=31) then
      begin
        IO:=Numeric;
        if GetExpressionNoComma then
        begin
          if Numeric in [0..7] then //(Numeric>=0) and (Numeric<=7) then
          begin
            Code2($9800 or (IO shl 3) or Numeric);
          end;
        end;
      end else MakeError(26);
    end;
  end;
end;

procedure A_CBR;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      if RegisterCode > 15 then
      begin
        if GetExpressionNoComma then
        begin
          if Numeric in [0..255] then //(Numeric>=0) and (Numeric<=255) then
          begin
            Code2($7000 or RegConstant(RegisterCode,Numeric xor $FF));
          end else MakeError(26);
        end;
      end;
    end;
  end;
end;

{==============================================================================}

procedure D_GLOBAL;

   var
      I: integer;

   begin
   while ParseItem(spSpaceIsTerminator) do
      if Pass = 1 then
         AddGlobal(CItem);
   InstructionError:=false;
   end;

{==============================================================================}

procedure D_RegUse;

   var
      W: cardinal;
      X: longint;

   begin
   if ParseItem(spSpaceIsTerminator) then
      begin
      if CItem[1] <> '#' then
         CItem := {#1} '!!' + char(LinkFileNumber+64) + CItem;
      X := RegsUsed.IndexOf(CItem);
      if X = -1 then
         begin
         W := 0;
         RegsUsed.AddObject(CItem, pointer(W));
         CurrentTrackingVariable := RegsUsed.IndexOf(CItem);
         end
      else
         CurrentTrackingVariable := X;
      InstructionError:=false;
      end;
   end;

{==============================================================================}

procedure D_RegNot;

   var
      W,X: cardinal;                                                                 // longint
      R: byte;
      I:  integer;

   begin
   if ParseItem(spSpaceIsTerminator) then
      begin
      if CItem = '##' then
         begin
         if CurrentTrackingVariable > -1 then
            CItem := RegsUsed[CurrentTrackingVariable]
         else
            begin
            MakeError(27); exit;
            end;
         end
      else
         begin
         if CItem[1] <> '#' then
            CItem:={#1} '!!'  + char(LinkFileNumber+64) + CItem;
         end;

      I := RegsUsed.IndexOf(CItem);
      if Comma then
         begin
         if GetExpressionNoComma then
            begin
            if IsRegister then
               begin
               R := RegisterCode;
               end
            else
               exit;
            end
         else
            exit;
         end
      else
         exit;

      if I <= -1 then
         begin
         if (Pass > 1) and not SyntaxCheck then
            begin
            MakeError(25);
            exit;
            end;
         end
      else
         begin
         W := 0;
         RegsUsed.Objects[I] := pointer(cardinal(RegsUsed.Objects[I]) and (longMask[R] xor $FFFFFFFF));   // longint
         end;
      InstructionError:=false;
      end;
   end;

{==============================================================================}

procedure D_RegEnd;

   begin
   CurrentTrackingVariable := -1;
   InstructionError := false;
   end;

{==============================================================================}

procedure D_RegAdd;

   var  X,Y: longint;                                                                 //longint;

   begin
   if ParseItem(spSpaceIsTerminator) then
      begin
      if CItem = '##' then
         begin
         if CurrentTrackingVariable > -1 then
            CItem := RegsUsed[CurrentTrackingVariable]
         else
            begin
            MakeError(27); exit;
            end;
         end
      else
         begin
         if CItem[1] <> '#' then
            CItem := {#1} '!!' + char(LinkFileNumber+64) + CItem;
         end;

      X := RegsUsed.Indexof(CItem);
      if X <= -1 then
         begin
         if (Pass > 1) and not SyntaxCheck then
            begin
            ShowMessage(CItem + ' ' + IntToStr(X));
            MakeError(25);
            exit;
            end;
         end;

      if Comma then
         begin
         if ParseItem(spSpaceIsTerminator) then
            begin
            if CItem[1] <> '#' then
               CItem := {#1} '!!' + char(LinkFileNumber+64) + CItem;
            Y := RegsUsed.Indexof(CItem);
            if Y <= -1 then
               begin
               if (Pass > 1) and not SyntaxCheck then
                  begin
                  MakeError(25);
                  end
               else
                  InstructionError := false;
               exit;
               end;

            if (X > -1) and (Y > -1) then
               RegsUsed.Objects[X] := pointer(cardinal(RegsUsed.Objects[Y]) or cardinal(RegsUsed.Objects[X]));   // longint
            InstructionError:=false;
            end;
         end;
      end;
   end;

procedure A_CLC;
begin
  Code2($9488);
end;

procedure A_CLH;
begin
  Code2($94D8);
end;

procedure D_IMPORT;
begin
  D_Ext;
end;

procedure A_CLI;
begin
  Code2($94F8);
end;

procedure A_CLN;
begin
  Code2($94A8);
end;

procedure A_CLR;
begin
  if getExpressionNoComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      Code2($2400 or RegSplit55(RegisterCode,RegisterCode));
    end;
  end;
end;

procedure A_CLS;
begin
  Code2($94C8);
end;

procedure A_CLT;
begin
  Code2($94E8);
end;

procedure A_CLV;
begin
  Code2($94B8);
end;

procedure A_CLZ;
begin
  Code2($9498);
end;

procedure A_COM;
begin
  if GetExpressionNoComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      Code2($9400 or (RegisterCode shl 4));
    end;
  end;
end;

{==============================================================================}

procedure A_CP;

   var
      Reg: word;

   begin
   if GetExpressionComma then
      if IsRegister then
         begin
         Reg := RegisterCode;
         if GetExpressionNoComma then
            if IsRegister then
               Code2($1400 or RegSplit55(Reg,RegisterCode));
         end;
   end;

procedure A_CPC;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      Reg:=RegisterCode;
      if GetExpressionNoComma then
      begin
        if IsRegister then
        begin
          Code2($0400 or RegSplit55(Reg,RegisterCode));
        end;
      end;
    end;
  end;
end;

{==============================================================================}

procedure A_CPI;

   begin
   if GetExpressionComma then
      if IsRegister then
         if RegisterCode > 15 then
            if GetExpressionNoComma then
               if ExpressionType=ExExpression then
                  begin
                  if (Numeric >= 0) and (Numeric <= 255) then
                     Code2($3000 or RegConstant(RegisterCode, Numeric))
                  else
                     MakeError(26);
                  end;
   end;

procedure A_CPSE;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      Reg:=RegisterCode;
      if GetExpressionNoComma then
      begin
        if IsRegister then
        begin
          Code2($1000 or RegSplit55(Reg,RegisterCode));
        end;
      end;
    end;
  end;
end;

procedure A_DEC;
begin
  if GetExpressionNoComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      Code2($940A or (RegisterCode shl 4));
    end;
  end;
end;

procedure A_EOR;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      Reg:=RegisterCode; RegisterTrack(RegisterCode);
      if GetExpressionNoComma then
      begin
        if IsRegister then
        begin
          Code2($2400 or RegSplit55(Reg,RegisterCode));
        end;
      end;
    end;
  end;
end;

procedure A_ICALL;
begin
  Code2($9509);
end;

procedure A_IJMP;
begin
  Code2($9409);
end;

procedure A_IN;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      Reg:=RegisterCode;
      RegisterTrack(Reg);
      if GetExpressionNoComma then
      begin
        if ExpressionType=ExExpression then
        begin
          if Numeric in [0..63] then //(Numeric>=0) and (Numeric<=63) then
          Code2($B000 or RegSplit65(Reg,Numeric)) else MakeError(26);
        end;
      end;
    end;
  end;
end;

{==============================================================================}

procedure A_INC;

   begin
   if GetExpressionNoComma then
      begin
      if IsRegister then
         begin
         RegisterTrack(RegisterCode);
         Code2($9403 or (RegisterCode shl 4));
         end;
      end;
   end;

{==============================================================================}

procedure A_JMP;

   begin
   if GetExpressionNoComma then
      if ExpressionType = ExExpression then
         begin
         Code2($940C);
         InsertWord(Numeric shr 1);
         end;
   end;

{==============================================================================}

procedure A_LD;

   var
      Reg: word;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         Reg := RegisterCode;
         RegisterTrack(RegisterCode);
         if ParseItem(spSpaceIsTerminator) then
            begin
            if CItem = 'X' then
               Code2($900C or (Reg shl 4))
            else if CItem = 'X+' then
               Code2($900D or (Reg shl 4))
            else if CItem = '-X' then
               Code2($900E or (Reg shl 4))
            else if CItem = 'Y' then
               Code2($8008 or (Reg shl 4))
            else if CItem = 'Y+' then
               Code2($9009 or (Reg shl 4))
            else if CItem = '-Y' then
               Code2($900A or (Reg shl 4))
            else if CItem = 'Z' then
               Code2($8000 or (Reg shl 4))
            else if CItem = 'Z+' then
               Code2($9001 or (Reg shl 4))
            else if CItem = '-Z' then
               Code2($9002 or (Reg shl 4))
            end;
         end;
      end;
   end;

procedure A_LDD;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      Reg:=RegisterCode; RegisterTrack(RegisterCode);
      if GetExpressionNoComma then
      begin
        if ExpressionType=ExIndexedY then
        begin
          Code2($8008 or (reg shl 4) or (Numeric and 7) or
            ((Numeric and $18) shl 7) or ((Numeric and $20) shl 8));
        end else
        if ExpressionType=ExIndexedZ then
        begin
          Code2($8000 or (reg shl 4) or (Numeric and 7) or
            ((Numeric and $18) shl 7) or ((Numeric and $20) shl 8));
        end;
      end;
    end;
  end;
end;

{==============================================================================}

procedure A_LDI;

   begin
   if GetExpressionComma then
      if IsRegister then
         if RegisterCode > 15 then
            begin
            RegisterTrack(RegisterCode);
            if GetExpressionNoComma then
               if ExpressionType = ExExpression then
                  begin
                  if (Numeric >= 0) and (Numeric <= 255) then
                     Code2($E000 or RegConstant(RegisterCode, Numeric))
                  else
                     MakeError(26);
                  end;
            end;
   end;

procedure A_LDS;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      reg:=RegisterCode; RegisterTrack(RegisterCode);
      if GetExpressionNoComma then
      begin
        Code2($9000 or (reg shl 4));
        InsertWord(Numeric);
      end;
    end;
  end;
end;

procedure A_LPM;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      Reg:=RegisterCode; RegisterTrack(RegisterCode);
      if ParseItem(spSpaceIsTerminator) then
      begin
        if CItem='Z' then
        begin
          Code2($9004 or (Reg shl 4));
        end else
        if CItem='Z+' then
        begin
          Code2($9005 or (Reg shl 4));
        end;
      end;
    end;
  end else
  begin
    Code2($95C8);
    RegisterTrack(0);
  end;
end;

procedure A_LSL;
begin
  if GetExpressionNoComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      Code2($0C00 or RegSplit55(registercode,registercode));
    end;
  end;
end;

procedure A_LSR;
begin
  if GetExpressionNoComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      Code2($9406 or (RegisterCode shl 4));
    end;
  end;
end;

procedure A_MOV;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      Reg:=RegisterCode;
      if GetExpressionNoComma then
      begin
        if IsRegister then
        begin
          RegisterTrack(Reg);
          Code2($2C00 or RegSplit55(Reg,RegisterCode));
        end;
      end;
    end;
  end;
end;

procedure A_MUL;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      Reg:=RegisterCode;
      RegisterTrack(0);
      RegisterTrack(1);
      if GetExpressionNoComma then
      begin
        if IsRegister then
        begin
          Code2($9C00 or RegSplit55(Reg,RegisterCode));
        end;
      end;
    end;
  end;
end;

procedure A_NEG;
begin
  if GetExpressionNoComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      Code2($9401 or (RegisterCode shl 4));
    end;
  end;
end;

procedure A_NOP;
begin
  Code2($0000);
end;

procedure A_OR;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      Reg:=RegisterCode; RegisterTrack(RegisterCode);
      if GetExpressionNoComma then
      begin
        if IsRegister then
        begin
          Code2($2800 or RegSplit55(Reg,RegisterCode));
        end;
      end;
    end;
  end;
end;

procedure A_ORI;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      if RegisterCode>15 then
      begin
        RegisterTrack(RegisterCode);
        if GetExpressionNoComma then
        begin
          if ExpressionType=ExExpression then
          begin
            if (Numeric>=0) and (Numeric<=255) then
              Code2($6000 or RegConstant(RegisterCode,Numeric)) else MakeError(26);
          end;
        end;
      end;
    end;
  end;
end;

procedure A_OUT;
var
  v: word;
begin
  if GetExpressionComma then
  begin
    if ExpressionType=ExExpression then
    begin
      v:=Numeric;
      if GetExpressionNoComma then
      begin
        if IsRegister then
        begin
//          if (V >= 0) and (V <= 63) then
          if V in [0..63] then
            Code2($B800 or RegSplit65(RegisterCode,v)) else makeError(26);
        end;
      end;
    end;
  end;
end;

procedure A_POP;
begin
  if GetExpressionNoComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      Code2($900F or (RegisterCode shl 4));
    end;
  end;
end;

procedure A_PUSH;
begin
  if GetExpressionNoComma then
  begin
    if IsRegister then
    begin
      Code2($920F or (RegisterCode shl 4));
    end;
  end;
end;

procedure A_RCALL;
begin
  if getExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange2K(dollar+2,Numeric) then
      begin
        Code2($D000 or BranchDisplacement);
      end else
        if not FinalPass then InstructionError:=false else MakeError(8);
    end;
  end;
end;

procedure D_ORG;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      InstructionError:=false;
      case ActiveArea of
      CodeActive:
      begin
        CodeAssemblyPointer:=(CodeAssemblyPointerAtSOF+Numeric) shl 1;
        AssemblyPointer:=CodeAssemblyPointer;
      end;
      DataActive:
      begin
        DataAssemblyPointer:=dataAssemblyPointerAtSOF+Numeric;
        AssemblyPointer:=DataAssemblyPointer;
      end;
      ABSActive:
      begin
        ABSAssemblyPointer:=Numeric shl 1;
        AssemblyPointer:=ABSAssemblyPointer;
      end;
      E2Active:
      begin
        E2AssemblyPointer:=(E2AssemblyPointerAtSOF+Numeric);
        AssemblyPointer:=E2AssemblyPointer;
      end;
      ConstActive:
      begin
        ConstAssemblyPointer:=(ConstAssemblyPointerAtSOF+Numeric) shl 1;
        AssemblyPointer:=ConstAssemblyPointer;
      end;
      end;
    end else MakeError(6);
  end;
end;

{==============================================================================}

procedure A_RET;

   begin
   Code2($9508);
   end;

procedure A_RETI;
begin
  Code2($9518);
end;

procedure A_RJMP;
begin
  if getExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange2K(dollar+2,Numeric) then
      begin
        Code2($C000 or BranchDisplacement);
      end else
        if not FinalPass then InstructionError:=false else
          MakeError(8);
    end;
  end;
end;

procedure A_ROL;
begin
  if GetExpressionNoComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      Code2($1C00 or RegSplit55(registercode,registercode));
    end;
  end;
end;

procedure A_ROR;
begin
  if GetExpressionNoComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      Code2($9407 or (RegisterCode shl 4));
    end;
  end;
end;

procedure A_SBC;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      Reg:=RegisterCode;
      RegisterTrack(RegisterCode);
      if GetExpressionNoComma then
      begin
        if IsRegister then
        begin
          Code2($0800 or RegSplit55(Reg,RegisterCode));
        end;
      end;
    end;
  end;
end;

procedure A_SBCI;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      if RegisterCode>15 then
      begin
        RegisterTrack(RegisterCode);
        if GetExpressionNoComma then
        begin
          if ExpressionType=ExExpression then
          begin
            if (Numeric>=0) and (Numeric<=255) then
              Code2($4000 or RegConstant(RegisterCode,Numeric)) else MakeError(26);
          end;
        end;
      end;
    end;
  end;
end;

procedure A_SBI;
var
  IO: word;
begin
  if GetExpressionComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if Numeric in [0..31] then //(Numeric>=0) and (Numeric<=31) then
      begin
        IO:=Numeric;
        if GetExpressionNoComma then
        begin
          if Numeric in [0..7] then //(Numeric>=0) and (Numeric<=7) then
          begin
            Code2($9A00 or (IO shl 3) or Numeric);
          end else MakeError(26);
        end;
      end else MakeError(26);
    end;
  end;
end;

procedure A_SBIC;
var
  IO: word;
begin
  if GetExpressionComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if Numeric in [0..31] then //(Numeric>=0) and (Numeric<=31) then
      begin
        IO:=Numeric;
        if GetExpressionNoComma then
        begin
          if Numeric in [0..7] then //(Numeric>=0) and (Numeric<=7) then
          begin
            Code2($9900 or (IO shl 3) or Numeric);
          end else MakeError(26);
        end;
      end else MakeError(26);
    end;
  end;
end;

procedure A_SBIS;
var
  IO: word;
begin
  if GetExpressionComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if Numeric in [0..31] then //(Numeric>=0) and (Numeric<=31) then
      begin
        IO:=Numeric;
        if GetExpressionNoComma then
        begin
          if Numeric in [0..7] then //(Numeric>=0) and (Numeric<=7) then
          begin
            Code2($9B00 or (IO shl 3) or Numeric);
          end else MakeError(26);
        end;
      end else MakeError(26);
    end;
  end;
end;

procedure A_SBIW;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      if RegisterCode in [24,26,28,30] then
      begin
        RegisterTrack(RegisterCode);
        RegisterTrack(RegisterCode+1);
        if GetExpressionNoComma then
        begin
          if ExpressionType=ExExpression then
          begin
            if Numeric in [0..63] then //(Numeric>=0) and (Numeric<=63) then
            begin
              Code2($9700 or ((RegisterCode-24) shl 3) or (Numeric and $F)
                or ((Numeric and $30) shl 2));
            end else MakeError(26);
          end;
        end;
      end;
    end;
  end;
end;

procedure A_SBR;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      if RegisterCode>15 then
      begin
        RegisterTrack(RegisterCode);
        if GetExpressionNoComma then
        begin
          if ExpressionType=ExExpression then
          begin
            if Numeric in [0..255] then //(Numeric>=0) and (Numeric<=255) then
              Code2($6000 or RegConstant(RegisterCode,Numeric)) else MakeError(26);
          end;
        end;
      end;
    end;
  end;
end;

procedure A_SBRC;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      if GetExpressionNoComma then
      begin
        if ExpressionType=ExExpression then
        begin
          if (Numeric>=0) and (Numeric<=7) then
          begin
            Code2($FC00 or (RegisterCode shl 4) or Numeric);
          end else MakeError(26);
        end;
      end;
    end;
  end;
end;

procedure A_SBRS;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      if GetExpressionNoComma then
      begin
        if ExpressionType=ExExpression then
        begin
          if (Numeric>=0) and (Numeric<=7) then
          begin
            Code2($FE00 or (RegisterCode shl 4) or Numeric);
          end else MakeError(26);
        end;
      end;
    end;
  end;
end;

procedure A_SEC;
begin
  Code2($9408);
end;

procedure A_SEH;
begin
  Code2($9458);
end;

procedure A_SEI;
begin
  Code2($9478);
end;

procedure A_SEN;
begin
  Code2($9428);
end;

procedure A_SER;
begin
  if GetExpressionNoComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      if (Registercode>=16) and (RegisterCode<=31) then
        Code2($EF0F or ((RegisterCode-16) shl 4));
    end;
  end;
end;

procedure D_PUBLIC;
begin
  D_Global;
end;

procedure A_SES;
begin
  Code2($9448);
end;

procedure A_SET;
begin
  Code2($9468);
end;

procedure A_SEV;
begin
  Code2($9438);
end;

procedure A_SEZ;
begin
  Code2($9418);
end;

procedure A_SLEEP;
begin
  Code2($9588);
end;

{==============================================================================}

procedure A_ST;

   var  Reg: word;
        S: string;                                                                     //ss[10];

   begin
   if ParseItem(spSpaceIsTerminator) and Comma then
      begin
      S := CItem;
      if GetExpressionNoComma then
         if IsRegister then
            begin
            Reg := RegisterCode;
            if S = 'X' then
               begin
               Code2($920C or (Reg shl 4));
               end
            else if S = 'X+' then
               begin
               Code2($920D or (Reg shl 4));
               end
            else if S = '-X' then
               begin
               Code2($920E or (Reg shl 4));
               end
            else if S = 'Y' then
               begin
               Code2($8208 or (Reg shl 4));
               end
            else if S = 'Y+' then
               begin
               Code2($9209 or (Reg shl 4));
               end
            else if s='-Y' then
               begin
               Code2($920A or (Reg shl 4));
               end
            else if S  ='Z' then
               begin
               Code2($8200 or (Reg shl 4));
               end
            else if S = 'Z+' then
               begin
               Code2($9201 or (Reg shl 4));
               end
            else if S = '-Z' then
               begin
               Code2($9202 or (Reg shl 4));
            end;
         end;
      end;
   end;

procedure A_STD;
var
  nm: longint;                                                                  // longint
begin
  if GetExpressionComma then
  begin
    if ExpressionType=ExIndexedY then
    begin
      nm:=Numeric;
      if GetExpressionNoComma then
      begin
        if IsRegister then
          Code2($8208 or (registercode shl 4) or (Nm and 7) or
            ((Nm and $18) shl 7) or ((Nm and $20) shl 8));
      end;
    end else
    if ExpressionType=ExIndexedZ then
    begin
      nm:=Numeric;
      if GetExpressionNoComma then
      begin
        if IsRegister then
          Code2($8200 or (registercode shl 4) or (Nm and 7) or
            ((Nm and $18) shl 7) or ((Nm and $20) shl 8));
      end;
    end;
  end;
end;

procedure A_STS;
var
  v: longint;                                                                   // longint
begin
  if GetExpressionComma then
  begin
    if ExpressionType=ExExpression then
    begin
      v:=Numeric;
      if GetExpressionNoComma then
      begin
        if IsRegister then
        begin
          Code2($9200 or (registerCode shl 4));
          InsertWord(v);
        end;  
      end;
    end;
  end;
end;

procedure A_SUB;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      Reg:=RegisterCode; RegisterTrack(RegisterCode);
      if GetExpressionNoComma then
      begin
        if IsRegister then
        begin
          Code2($1800 or RegSplit55(Reg,RegisterCode));
        end;
      end;
    end;
  end;
end;

procedure A_SUBI;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      if RegisterCode>15 then
      begin
        RegisterTrack(RegisterCode);
        if GetExpressionNoComma then
        begin
          if ExpressionType=ExExpression then
          begin
            if (Numeric>=0) and (Numeric<=255) then
              Code2($5000 or RegConstant(RegisterCode,Numeric)) else MakeError(26);
          end;
        end;
      end;
    end;
  end;
end;

procedure A_SWAP;
begin
  if GetExpressionNoComma then
  begin
    if IsRegister then
    begin
      RegisterTrack(RegisterCode);
      Code2($9402 or (registercode shl 4));
    end;
  end;
end;

procedure A_TST;
begin
  if GetExpressionNoComma then
  begin
    if IsRegister then
    begin
      Code2($2000 or RegSplit55(registercode,registercode));
    end;
  end;
end;

procedure A_WDR;
begin
  Code2($95A8);
end;

{=============================================================================}

procedure D_Link;

   begin
   if ParseItem(spSpaceIsTerminator) then
      begin
      if CItem = 'CODEADD' then
         begin
         InstructionError:=false;
         end
      else
         begin
         if TryExpression(Numeric) then
            begin
            CodeLoadAt := 0;
            CodeAssemblyPointer := Numeric;
            if ActiveArea = CodeActive then
               AssemblyPointer := CodeAssemblyPointer;
            CodeAssemblyPointerAtSOF := CodeAssemblyPointer;
            InstructionError := false;
            end
         else
            exit;
         end;  // if CItem

      if Comma then
         if ParseItem(spSpaceIsTerminator) then
            begin
            if CItem = 'DATAADD' then
               begin
               InstructionError := false;
               end
            else if TryExpression(Numeric) then
               begin
               DataAssemblyPointer := Numeric;
               if ActiveArea = DataActive then
                  AssemblyPointer := DataAssemblyPointer;
               DataAssemblyPointerAtSOF := DataAssemblyPointer;
               InstructionError := false;
               end
            else
               exit;
            end;  // if ParseItem

      if Comma then
         if ParseItem(spSpaceIsTerminator) then
            begin
            if TryExpression(Numeric) then
               begin
               CodeLoadAt := Numeric - CodeAssemblyPointer;
               InstructionError:=false;
               end
            else
               exit;
            end;
      end; // if ParseItem
   end;


procedure D_DS;
var
  e2: word;
begin
  if GetExpressionNoComma then
  begin
    e2:=E2Size;
    if Numeric>0 then
    begin
      if ActiveArea=DataActive then
      begin
        while Numeric>0 do
        begin
          inc(AssemblyPointer);
          inc(DataAssemblyPointer);
          InstructionError:=false;
          dec(Numeric);
        end;
      end else
      while (AssemblyPointer<BufferSize) and (Numeric>0) do
      begin
        InsertByte(0); dec(Numeric);
      end;
    end else InstructionError:=false;
    if ActiveArea=E2Active then E2Size:=e2;
  end;
end;

{==============================================================================}

procedure D_DB;

   var
      L,I: word;

   begin
   while ParseItem(spSpaceIsTerminator) and not Error do
      begin
      if (length(CItem) > 3) and (CItem[1] = '''') and (not (CItem[3] = '''')) then
         begin
         L := length(CItem);
         I := 2;

         while (I <= L) and (CItem[I] <> '''') do
            begin
            InsertByte(ord(CItem[I]));
            inc(I);
            end;

         if CItem[I] <> '''' then
            MakeError(14);
         end
      else
         begin
         if (length(CItem) = 3) and (CItem[1] = '''') and (CItem[3] = '''') then
            InsertByte(ord(CItem[2]))
         else if TryExpression(Numeric) then
            InsertByte(Numeric);
         end;
      end;
   end;

procedure D_DW;
var
  l,i: word;
begin
  while ParseItem(spSpaceIsTerminator) and not error do
  begin
    if (length(CItem)>3) and (CItem[1]='''') and (not (CItem[3]='''')) then
    begin
      l:=length(CItem);
      i:=2;
      while (i<=l) and (CItem[i]<>'''') do
      begin
        InsertByte(ord(CItem[i])); InsertByte(0);
        inc(i);
      end;
      if CItem[i]<>'''' then MakeError(14);
    end else
    begin
      if (length(CItem)=3) and (CItem[1]='''') and (CItem[3]='''') then
      begin
        InsertByte(ord(CItem[2])); InsertByte(0);
      end else
      if TryExpression(Numeric) then
      begin
        InsertByte(Numeric);
        InsertByte(Numeric shr 8);
      end;
    end;
  end;
end;

procedure D_DCA;
var
  l,i: word;
begin
  while ParseItem(spSpaceIsTerminator) and not error do
  begin
    if (length(CItem)>3) and (CItem[1]='''') and (not (CItem[3]='''')) then
    begin
      l:=length(CItem);
      i:=2;
      while (i<=l) and (CItem[i]<>'''') do
      begin
        InsertByte(ord(CItem[i])); InsertByte(0);
        inc(i);
      end;
      if CItem[i]<>'''' then MakeError(14);
    end else
    begin
      if (length(CItem)=3) and (CItem[1]='''') and (CItem[3]='''') then
      begin
        InsertByte(ord(CItem[2])); InsertByte(0);
      end else
      if TryExpression(Numeric) then
      begin
        Numeric:=Numeric div 2;
        InsertByte(Numeric);
        InsertByte(Numeric shr 8);
      end;
    end;
  end;
end;

procedure D_DL;
var
  l,i: word;
begin
  while ParseItem(spSpaceIsTerminator) and not error do
  begin
    if (length(CItem)>3) and (CItem[1]='''') and (not (CItem[3]='''')) then
    begin
      l:=length(CItem);
      i:=2;
      while (i<=l) and (CItem[i]<>'''') do
      begin
        InsertByte(ord(CItem[i])); InsertByte(0); InsertByte(0); InsertByte(0);
        inc(i);
      end;
      if CItem[i]<>'''' then MakeError(14);
    end else
    begin
      if (length(CItem)=3) and (CItem[1]='''') and (CItem[3]='''') then
      begin
        InsertByte(ord(CItem[2]));
        InsertByte(0); InsertByte(0); InsertByte(0);
      end else
      if TryExpression(Numeric) then
      begin
        InsertByte(Numeric);
        InsertByte(Numeric shr 8);
        InsertByte(Numeric shr 16);
        InsertByte(Numeric shr 24);
      end;
    end;
  end;
end;

procedure D_EQU;
var
  w: longint;                                                                   // longint
  c: char;
begin
  if not LineHasLabel then
  begin
    MakeError(16); exit;
  end;
  LabelsChanged:=TempLabelsChanged;
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      w:=Labels[LinkfileNumber].IndexOf(LastLabel);
      if w<>-1 then
      begin
        if ID[w].Value<>PreviousValue then dec(NumLabelsChanged);
        w:=longint(Labels[LinkfileNumber].Objects[w]);                          // longint
        ID[w].Value:=PreviousValue;
        if Numeric<>ID[w].Value then
        begin
          ID[W].Value:=Numeric;
          LabelsChanged:=true;
          inc(NumLabelsChanged);
        end;
        if FinalPass and WriteSymbolFile then
        begin
          if ID[W].Referenced then
          begin
            w:=ID[w].value;
            if ActiveArea=CodeActive then c:='C' else c:='D';
            SymLineWaiting:=c+':'+IntToHex(w,8)+' '+LastLabel;
          end;
        end;
      end;
      InstructionError:=false;
    end;
  end;
end;

procedure D_DVAR;
var
  w: longint;                                                                   // longint
  c: char;
begin
  if not LineHasLabel then
  begin
    MakeError(16); exit;
  end;
  LabelsChanged:=TempLabelsChanged;
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      w:=Labels[LinkfileNumber].IndexOf(LastLabel);
      if w<>-1 then
      begin
        if ID[w].Value<>PreviousValue then dec(NumLabelsChanged);
        w:=longint(Labels[LinkfileNumber].Objects[w]);                           // longint
        ID[w].Value:=PreviousValue;
        ID[w].DynamicVar:=true;
        if Numeric<>ID[w].Value then
        begin
          ID[W].Value:=Numeric;
          LabelsChanged:=true;
          inc(NumLabelsChanged);
        end;
        if FinalPass and WriteSymbolFile then
        begin
          if ID[W].Referenced then
          begin
            w:=ID[w].value;
            if ActiveArea=CodeActive then c:='C' else c:='D';
            SymLineWaiting:=c+':'+IntToHex(w,8)+' '+LastLabel;
          end;
        end;
      end;
      InstructionError:=false;
    end;
  end;
end;

procedure D_DEF; //Same as equ but allows redefinitions
var
  w: longint;                                                                   // longint
  c: char;
begin
  if not LineHasLabel then
  begin
    MakeError(16); exit;
  end;
  LabelsChanged:=TempLabelsChanged;
  RedefinedLabel:=false;
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      w:=Labels[LinkfileNumber].IndexOf(LastLabel);
      if w<>-1 then
      begin
        if ID[w].Value<>PreviousValue then dec(NumLabelsChanged);
        w:=longint(Labels[LinkfileNumber].Objects[w]);                          // longint
        ID[w].Value:=PreviousValue;
        if Numeric<>ID[w].Value then
        begin
          ID[W].Value:=Numeric;
          LabelsChanged:=true;
          inc(NumLabelsChanged);
        end;
        if FinalPass and WriteSymbolFile then
        begin
          if ID[W].Referenced then
          begin
            w:=ID[w].value;
            if ActiveArea=CodeActive then c:='C' else c:='D';
            SymLineWaiting:=c+':'+IntToHex(w,8)+' '+LastLabel;
          end;
        end;
      end;
      InstructionError:=false;
    end;
  end;
end;

procedure D_IFTRUE;
begin
  {if ActiveIfStatements>0 then
  begin
    MakeError(18); exit;
  end;}
  inc(ActiveIfStatements);
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if not IF_Skip then
        if Numeric=0 then
        begin
          IF_Skip:=true;
          If_SkipLevel:=ActiveIfStatements;
        end else IF_Skip:=false;
      InstructionError:=false;
    end;
  end;
end;

procedure D_IFFalse;
begin
  {if ActiveIfStatements>0 then
  begin
    MakeError(18); exit;
  end;}
  inc(ActiveIfStatements);
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if not IF_Skip then
        if Numeric<>0 then
        begin
          IF_Skip:=true;
          If_SkipLevel:=ActiveIfStatements;
        end else IF_Skip:=false;
      InstructionError:=false;
    end;
  end;
end;

{==============================================================================}

procedure D_IFREF;

   var
      W: longint;
      dIf_Skip: boolean;

   begin
   if ActiveIfStatements > 0 then
      begin
      MakeError(18);
      exit;
      end;

   inc(ActiveIfStatements);

   if Pass = 1 then
      dIf_Skip := false
   else
      dIf_Skip := true;

   InsideIfRef := true;
   If_SkipLevel := 1;

   repeat
      if ParseItem(spSpaceNotTerminator) then
         begin
         W := Labels[LinkfileNumber].IndexOf(CItem);
         if W = -1 then
            begin
            if Pass = 1 then
               InstructionError := false
            else
               begin
               ExpressionItem := CItem;
               MakeError(2);
               exit;
               end;
            end  // if W = -1
         else
            begin  // if W <> -1
            W := longint(Labels[LinkfileNumber].Objects[W]);
            if ID[W].Referenced then
               begin
               dIf_Skip := false;
               if FinalPass then
                  begin
                  W := ALabels.IndexOf(CItem);
                  if W = -1 then
                     ALabels.Add(CItem);
                  end;
               end  // if ID[W]
            else if SyntaxCheck then
               dIf_Skip := false;
            InstructionError := false;
            end; // if W = -1
         end;  // if ParseItem
      until not Comma;

   If_Skip := dIf_Skip;
   end;

procedure D_EndIf;
begin
  if ActiveIfStatements>0 then
  begin
    if If_Skip then
      if ActiveIfStatements=If_SkipLevel then
    begin
      If_Skip:=false;
    end;
    dec(ActiveIfStatements);
    InstructionError:=false;
    if ActiveIfStatements=0 then InsideIfRef:=false;
  end else MakeError(17);
end;

{==============================================================================}

procedure D_Load;

   var  W,SZ,Written: longint;                                                     // longint
        S: string;                                                                     //ss short
        bFile: file;

   begin
   if GetExpressionComma then
      begin
      if ExpressionType = ExExpression then
         begin
         W := Numeric;

         if W > $FFFFFF then
            begin
            MakeError(19);
            exit;
            end;

         if ParseItem(spSpaceIsTerminator) and not Comma then
            begin
            S := ExtractFileName(CItem);
            S := Project[1] + S;
            InstructionError := false;
            if FileExists(S) then
               begin
               if If_Skip then
                  exit;

               if FinalPass then
                  try
                  AssignFile(bFile, S);
                  reset(bFile, 1);
                  SZ := FileSize(bFile);
                  BlockRead(bfile, Buffer[W + CodeLoadAt], SZ, Written);
                  if Exesize < W + CodeLoadAt + SZ then
                     ExeSize := W + CodeLoadAt + SZ;
                  CloseFile(bfile);
                  Status(ExtractFilename(CurrentFile) + ' Loading ' + S +' at '+IntToHex(W, 6) + 'H');
                  except on EInOutError do
                     begin
                     ErrorString:='Cannot load ' + S;
                     MakeError(100);
                     CloseFile(bfile);
                     end;
                  end; // try
               end
            else
               begin
               Status(ExtractFilename(CurrentFile) + ' Warning: Load file not found: ' + S);
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure D_RUNEnd;

   begin
   if ParseItem(spSpaceIsTerminator) and not comma then
      begin
      if FinalPass then
         RunWhenDoneFiles.Add(CItem);
      InstructionError:=false;
      end;
   end;

procedure D_Include;
var
  c,n,i: word;
  S:  string;

begin
  if pass>1 then
  begin
    ParseItem(spSpaceIsTerminator); InstructionError:=false; exit;
  end;
  if not CurrentFileIsInclude then
  begin
    if ParseItem(spSpaceIsTerminator) and not Comma then
    begin
      CItem:=ExtractFileName(CItem);
{      if pass=1 then
      for n:=0 to NumberOfEditItems-1 do
      begin
        if EditList[n].Used then
        if Uppercase(ExtractFileName(EditList[n].Filename))=
          Uppercase(CItem) then
        begin
          if EditList[n].Editor.Modified then
          Editor.SaveFile(EditList[n].Filename,n);
          break;
        end;
      end;
}
      CItem:=Project[1]+CItem;
      IncludeFileName:=CItem;
      if FileExists(CItem) then
      begin
        Includes.LoadFromFile(CItem);
        if Includes.Count>0 then
        begin
          CurrentFileIsInclude:=true;
          IncludelineNumber:=-1;
          InstructionError:=false;
        end;
        i:=0; c:=SourceFiles.Count;
        while i<c do
        begin
          if SourceFiles[i]=CItem then
          begin
            OldSourceFileNumber:=SourceFileNumber;
            SourceFileNumber:=i+1;
            break;
          end;
          inc(i);
        end;
        if i=c then
        begin
          SourceFiles.Add(CItem);
          OldSourceFileNumber:=SourceFileNumber;
          SourceFileNumber:=i+1;
        end;
      end else MakeError(20);
    end;
  end else MakeError(21);
end;

{==============================================================================}

procedure D_Library;

   var  C,I: word;
        XItem: string;

   begin
   if (Pass > 1) or SyntaxCheck then
      begin
      ParseItem(spSpaceIsTerminator);
      InstructionError := false;
      exit;
      end;

   InstructionError := false;
   if ParseItem(spSpaceIsTerminator) and not Comma then
      begin
      CItem := ChangeFileExt(CItem,'.RSL');
      XItem := ExtractFileName(CItem);
      C := LinkProject.Count;
      I := ProjectFileStart;
      Found := false;

      while I < C do
         begin
         if uppercase(ExtractFileName(LinkProject[I]))= uppercase(XItem) then
         begin
            Found := true;
            break;
            end;
         inc(I);
         end;

      if not Found then
         begin
         CItem := ChangeFileExt(CItem, '.ASM');
         XItem := ExtractFileName(CItem);
         C := LinkProject.Count;
         I := ProjectFileStart;
         Found := false;

         while I < C do
            begin
            if uppercase(ExtractFileName(LinkProject[I])) =  uppercase(XItem) then
               begin
               Found := true;
               break;
               end;
            inc(I);
            end;
         end;

      if not Found then
         begin
         inc(AddLinkFiles);
         LinkProject.Add(CItem);
         end;
      end;
   end;

{==============================================================================}

function MakeSymbol(S: string; Value: longint; Sect,Spec: word; StorageClass,Aux: byte): longint;  // longint

   var I: longint;                                                              // longint
       Str:   shortstring;

   begin
   S := S + #0;
   StrToShort (S, Str);
   if length(S) > 8 then
      begin
      I := 0;
      move(I, CoffSymbol.Name, 4);
      I := FileSize(CoffStringFile) + 4;
      move(I, CoffSymbol.Name[4], 4);
      BlockWrite(CoffStringFile, Str[1], length(Str));
      end
   else
      begin
      move(Str[1], CoffSymbol.Name, 8);
      end;

   CoffSymbol.Value := Value;
   CoffSymbol.Section := Sect;
   CoffSymbol.Spec := Spec;
   CoffSymbol.SClass := StorageClass;
   CoffSymbol.Aux := Aux;
   BlockWrite(CoffSymFile, CoffSymbol, 18);
   result := CoffSymbolIndex;
   inc(CoffSymbolIndex);
   end;

procedure MakeStaticSymbol(s: string; Value: longint; Sect,Spec: word; StorageClass,Aux: byte);
var
  i: longint;
begin
  s:=s+#0;
  if length(s)>8 then
  begin
    i:=0;
    move(i,CoffSymbol.name,4);
    i:=FileSize(CoffStringFile)+4;
    move(i,CoffSymbol.name[4],4);
    BlockWrite(CoffStringFile,s[1],length(s));
  end else
  begin
    move(s[1],CoffSymbol.Name,8);
  end;
  CoffSymbol.Value:=Value;
  CoffSymbol.Section:=Sect;
  CoffSymbol.Spec:=Spec;
  CoffSymbol.SClass:=StorageClass;
  CoffSymbol.Aux:=Aux;
  BlockWrite(CoffStaticFile,CoffSymbol,18);
  Inc(StaticCount);
end;

function MakeAux(x: longint; Value: longint; Sect,Spec: word; StorageClass,Aux: byte): longint;
var
  i: longint;
begin
  i:=0;
  move(i,CoffSymbol.name,4);
  move(x,CoffSymbol.name[4],4);
  CoffSymbol.Value:=Value;
  CoffSymbol.Section:=Sect;
  CoffSymbol.Spec:=0;
  CoffSymbol.SClass:=StorageClass;
  CoffSymbol.Aux:=Aux;
  BlockWrite(CoffSymFile,CoffSymbol,18);
  Result:=CoffSymbolIndex;
  inc(CoffSymbolIndex);
end;

//function MakeAuxFile(s: string): longint;
procedure MakeAuxFile(s: string);

begin
  if length(s)>12 then
  begin
    ErrorString:='Filename too long for COFF file: '+s;
    MakeError(100);
    exit;
  end;
  fillchar(CoffSymbol,SizeOf(CoffSymbol),0);
  move(s[1],CoffSymbol.Name,length(s));
  BlockWrite(CoffSymFile,CoffSymbol,18);
  inc(CoffSymbolIndex);
end;

procedure D_File;
begin
  if ParseItem(spSpaceIsTerminator) and not Comma then
  begin
    InstructionError:=false;
    if FinalPass and WriteSymbolFile then
    begin
      writeln(SymFile,'F:'+CItem);
      if COFFEnabled then
      begin
        CItem:=copy(CItem,2,length(CItem)-2);
        MakeSymbol('.file', 0, $FFFE, 0, 103, 1);
        if uppercase(ExtractFilePath(CItem))<>uppercase(Project[1]) then
        begin
          ErrorString:='Source path differs from project file - not permitted for COFF format';
          MakeError(100);
        end;  
        MakeAUXFile(ExtractFilename(CItem));
      end;
    end;
  end;
end;

procedure D_Line;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if FinalPass and WriteSymbolFile then
      begin
        writeln(SymFile,'L:'+IntToHex(Dollar,8)+':'+IntToStr(Numeric));
        if COFFEnabled then
        begin
          CoffLineInfo.Line:=Numeric-ProcedureStartLine+1;
          CoffLineInfo.Address:=Dollar;
          BlockWrite(CoffLineFile,CoffLineInfo,6);
        end;
        inc(LineNumberEntries);
      end;
      InstructionError:=false;
    end;
  end;
end;

{==============================================================================}

procedure LDB(V: longint; R: byte);

   begin
   if (V >= 0) and (V < $20) then
      begin
      if R <> V then
         Code2($2C00 or RegSplit55(r,v))
      else
         InstructionError:=false;
      end
   else if (V > $1F) and (V < $60) then
      Code2($B000 or RegSplit65(R, V-32))
   else
      begin
      Code2($9000 or (R shl 4));
      Code2(V);
      end;
   end;

{==============================================================================}

procedure STB(V: longint; R: byte);

   begin
   if (V >= 0) and (V < $20) then
      begin
      if V <> R then
         begin
         Code2($2C00 or RegSplit55(V, R));
         RegisterTrack(R);
         end
      else
         InstructionError := false;
      end
   else if (V > $1F) and (V < $60) then
      begin
      Code2($B800 or RegSplit65(R, V-32));
      end
   else
      begin
      Code2($9200 or (R shl 4));
      Code2(V);
      end;
   end;

procedure LDW(v: longint; r: byte);                                             // longint
begin
  LDB(v,r);
  LDB(v+1,r+1);
end;

procedure LDL(v: longint; r: byte);                                             // longint
begin
  LDB(v,r);
  LDB(v+1,r+1);
  LDB(v+2,r+2);
  LDB(v+3,r+3);
end;

procedure STW(v: longint; r: byte);                                             // longint
begin
  STB(v,r);
  STB(v+1,r+1);
end;

procedure STL(v: longint; r: byte);                                             // longint
begin
  STB(v,r);
  STB(v+1,r+1);
  STB(v+2,r+2);
  STB(v+3,r+3);
end;

procedure M_LDW;
var
  reg: string;                                                                   //ss [2];
begin
  if ParseItem(spSpaceIsTerminator) then
  begin
    reg:=CItem;
    if GetExpressionNoComma then
    begin
      if ExpressionType=ExExpression then
      begin
        if reg='X' then
        begin
          LDW(Numeric,26);
          RegisterTrack(26);
          RegisterTrack(27);
        end else
        if reg='Y' then
        begin
          LDW(Numeric,28);
          RegisterTrack(28);
          RegisterTrack(29);
        end else
        if reg='Z' then
        begin
          LDW(Numeric,30);
          RegisterTrack(30);
          RegisterTrack(31);
        end else
        if reg='WA' then
        begin
          LDW(Numeric,16);
          RegisterTrack(16);
          RegisterTrack(17);
        end else
        if reg='WB' then
        begin
          LDW(Numeric,20);
          RegisterTrack(20);
          RegisterTrack(21);
        end;
      end;
    end;
  end;
end;

{==============================================================================}

procedure M_LDB;

   var  Reg: word;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         Reg := RegisterCode;
         RegisterTrack(RegisterCode);
         if GetExpressionNoComma then
            begin
            if ExpressionType = ExExpression then
               begin
               LDB(Numeric, Reg);
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure M_STW;

   var  Reg: string;                                                                   //ss [2];
        N: longint;
                                                                           // longint
   begin
   if GetExpressionComma then
      begin
      if ExpressionType = ExExpression then
         begin
         N := Numeric;
         if ParseItem(spSpaceIsTerminator) then
            begin
            Reg := CItem;
            if Reg = 'X' then
               STW(N, 26)
            else if Reg = 'Y' then
               STW(N, 28)
            else if Reg = 'Z' then
               STW(N, 30)
            else if Reg = 'WA' then
               STW(N, 16)
            else if Reg = 'WB' then
               STW(N, 20);
            end;
         end;
      end;
   end;

{==============================================================================}

procedure M_STL;
var
  reg: string;                                                                   //ss [2];
  n: longint;                                                                   // longint
begin
  if GetExpressionComma then
  begin
    if ExpressionType=ExExpression then
    begin
      n:=Numeric;
      if ParseItem(spSpaceIsTerminator) then
      begin
        reg:=CItem;
        if reg='LA' then
        begin
          STL(N,16);
        end else
        if reg='LB' then
        begin
          STL(N,20);
        end;
      end;
    end;
  end;
end;

{==============================================================================}

procedure M_STB;

   var
      N: longint;

   begin
   if GetExpressionComma then
      begin
      if ExpressionType = ExExpression then
         begin
         N := Numeric;
         if GetExpressionNoComma then
            if IsRegister then
               STB(N, RegisterCode);
         end;
      end;
   end;

procedure M_LDL;
var
  reg: string;                                                                   //ss [2];
begin
  if ParseItem(spSpaceIsTerminator) then
  begin
    reg:=CItem;
    if GetExpressionNoComma then
    begin
      if ExpressionType=ExExpression then
      begin
        if reg='LA' then
        begin
          LDL(Numeric,16);
          RegisterTrack(16);
          RegisterTrack(17);
          RegisterTrack(18);
          RegisterTrack(19);
        end else
        if reg='LB' then
        begin
          LDL(Numeric,20);
          RegisterTrack(20);
          RegisterTrack(21);
          RegisterTrack(22);
          RegisterTrack(23);
        end;
      end;
    end;
  end;
end;

{==============================================================================}

procedure M_LDWI;

   var  Reg: string;                                                                   //ss [2];

   begin
   if ParseItem(spSpaceIsTerminator) then
      begin
      Reg := CItem;
      if GetExpressionNoComma then
         begin
         if ExpressionType = ExExpression then
            begin
            if Reg = 'X' then
               begin
               Code2($E000 or RegConstant(26, lo(Numeric)));
               Code2($E000 or RegConstant(27, hi(Numeric)));
               RegisterTrack(26);
               RegisterTrack(27);
               end
            else if Reg = 'Y' then
               begin
               Code2($E000 or RegConstant(28, lo(Numeric)));
               Code2($E000 or RegConstant(29, hi(Numeric)));
               RegisterTrack(28);
               RegisterTrack(29);
               end
            else if Reg = 'Z' then
               begin
               Code2($E000 or RegConstant(30, lo(Numeric)));
               Code2($E000 or RegConstant(31, hi(Numeric)));
               RegisterTrack(30);
               RegisterTrack(31);
               end
            else if Reg = 'WA' then
               begin
               Code2($E000 or RegConstant(16, lo(Numeric)));
               Code2($E000 or RegConstant(17, hi(Numeric)));
               RegisterTrack(16);
               RegisterTrack(17);
               end
            else if Reg = 'WB' then
               begin
               Code2($E000 or RegConstant(20, lo(Numeric)));
               Code2($E000 or RegConstant(21, hi(Numeric)));
               RegisterTrack(20);
               RegisterTrack(21);
               end;
            end;
         end;
      end;
   end;

procedure M_LDLI;
var
  reg: string;                                                                   //ss [2];
begin
  if ParseItem(spSpaceIsTerminator) then
  begin
    reg:=CItem;
    if GetExpressionNoComma then
    begin
      if ExpressionType=ExExpression then
      begin
        if reg='WA' then
        begin
          Code2($E000 or RegConstant(16,(Numeric and $FF)));
          Code2($E000 or RegConstant(17,((Numeric shr 8) and $FF)));
          Code2($E000 or RegConstant(18,((Numeric shr 16) and $FF)));
          Code2($E000 or RegConstant(19,((Numeric shr 24) and $FF)));
          RegisterTrack(16);
          RegisterTrack(17);
          RegisterTrack(18);
          RegisterTrack(19);
        end else
        if reg='WB' then
        begin
          Code2($E000 or RegConstant(20,(Numeric and $FF)));
          Code2($E000 or RegConstant(21,((Numeric shr 8) and $FF)));
          Code2($E000 or RegConstant(22,((Numeric shr 16) and $FF)));
          Code2($E000 or RegConstant(23,((Numeric shr 24) and $FF)));
          RegisterTrack(20);
          RegisterTrack(21);
          RegisterTrack(22);
          RegisterTrack(23);
        end;
      end;
    end;
  end;
end;

procedure M_POPBA;
begin
  Code2($900F or (16 shl 4));
  RegisterTrack(16);
end;

procedure M_POPBB;
begin
  Code2($900F or (20 shl 4));
  RegisterTrack(20);
end;

procedure M_POPWA;
begin
  Code2($900F or (16 shl 4));
  Code2($900F or (17 shl 4));
  RegisterTrack(16);
  RegisterTrack(17);
end;

procedure M_POPWB;
begin
  Code2($900F or (20 shl 4));
  Code2($900F or (21 shl 4));
  RegisterTrack(20);
  RegisterTrack(21);
end;

procedure M_POPLA;
begin
  Code2($900F or (16 shl 4));
  Code2($900F or (17 shl 4));
  Code2($900F or (18 shl 4));
  Code2($900F or (19 shl 4));
  RegisterTrack(16);
  RegisterTrack(17);
  RegisterTrack(18);
  RegisterTrack(19);
end;

procedure M_POPLB;
begin
  Code2($900F or (20 shl 4));
  Code2($900F or (21 shl 4));
  Code2($900F or (22 shl 4));
  Code2($900F or (23 shl 4));
  RegisterTrack(20);
  RegisterTrack(21);
  RegisterTrack(22);
  RegisterTrack(23);
end;

procedure M_PUSHBA;
begin
  Code2($920F or (16 shl 4));
end;

procedure M_PUSHBB;
begin
  Code2($920F or (20 shl 4));
end;

procedure M_PUSHWA;
begin
  Code2($920F or (17 shl 4));
  Code2($920F or (16 shl 4));
end;

procedure M_PUSHZ;
begin
  Code2($920F or (31 shl 4));
  Code2($920F or (30 shl 4));
end;

procedure M_PUSHWB;
begin
  Code2($920F or (21 shl 4));
  Code2($920F or (20 shl 4));
end;

procedure M_PUSHLA;
begin
  Code2($920F or (19 shl 4));
  Code2($920F or (18 shl 4));
  Code2($920F or (17 shl 4));
  Code2($920F or (16 shl 4));
end;

procedure M_PUSHLB;
begin
  Code2($920F or (23 shl 4));
  Code2($920F or (22 shl 4));
  Code2($920F or (21 shl 4));
  Code2($920F or (20 shl 4));
end;

{==============================================================================}

procedure M_CALL;

   begin
   if GetExpressionNoComma then
      begin
      if ExpressionType = ExExpression then
         begin
         if (Numeric > Dollar+2+4094) or (Numeric < Dollar+2-4096) then
            begin
            if ROM4K then
               begin
               if RelativeRange2K(Dollar+2, Numeric) then
                  Code2($D000 or BranchDisplacement);
               end
            else if HasFarCall then
               begin
               Code2($940E);
               InsertWord(Numeric shr 1);
               end
            else
               begin
               Code2($E000 or RegConstant(30, lo(Numeric shr 1)));
               Code2($E000 or RegConstant(31, hi(Numeric shr 1)));
               RegisterTrack(30);
               RegisterTrack(31);
               Code2($9509); //ICall
               end;
            end
         else
            begin
            if RelativeRange2K(dollar+2,Numeric) then
               Code2($D000 or BranchDisplacement);
            end;
         end;
      end;
   end;

procedure M_CALLZ;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if (Numeric>Dollar+2+4094) or (Numeric<Dollar+2-4096) then
      begin
        if ROM4K then
        begin
          if RelativeRange2K(dollar+2,Numeric) then
          begin
            Code2($D000 or BranchDisplacement);
          end;
        end else
        if HasFarCall then
        begin
          Code2($940E);
          InsertWord(Numeric shr 1);
        end else
        begin
          Code2($D000 or 1); //rcall. This gets a return address onto the stack
          Code2($C000 or 5); //rjmp. The called routine will return here
          //The Rcall goes here
          Code2($E000 or RegConstant(23,lo(Numeric shr 1)));  //This is a jump using the stack
          Code2($920F or (23 shl 4));
          Code2($E000 or RegConstant(23,hi(Numeric shr 1)));
          Code2($920F or (23 shl 4));
          Code2($9508);
          //The rjmp goes here
          RegisterTrack(23);
        end;
      end else
      begin
        if RelativeRange2K(dollar+2,Numeric) then
        begin
          Code2($D000 or BranchDisplacement);
        end;
      end;
    end;
  end;
end;

{==============================================================================}

procedure M_JMP;

   begin
   if GetExpressionNoComma then
      begin
      if ExpressionType = ExExpression then
         begin
         if (Numeric > Dollar+2+4094) or (Numeric < Dollar+2-4096) then
            begin
            if ROM4K then
               begin
               if RelativeRange2K(Dollar+2, Numeric) then
                  begin
                  Code2($C000 or BranchDisplacement);
                  end;
               end
            else if HasFarCall then
               begin
               Code2($940C);
               InsertWord(Numeric shr 1);
               end
            else   // IJMP to address in Z
               begin
               Code2($E000 or RegConstant(30, lo(Numeric shr 1)));
               Code2($E000 or RegConstant(31, hi(Numeric shr 1)));
               RegisterTrack(30);
               RegisterTrack(31);
               Code2($9409); //IJmp
               end;
            end
         else
            begin
            if RelativeRange2K(Dollar+2, Numeric) then
               begin
               Code2($C000 or BranchDisplacement);
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure M_JMPEQ;

   begin
   if GetExpressionNoComma then
      begin
      if ExpressionType = ExExpression then
         begin
         if RelativeRange(Dollar+2, Numeric) then
            Code2($F001 or (BranchDisplacement shl 3)) //breq
         else
            begin
            if (Numeric > Dollar+4+4094) or (Numeric < Dollar+4-4096) then
               begin
               if ROM4K then
                  begin
                  if RelativeRange2K(Dollar+2, Numeric) then
                     Code2($C000 or BranchDisplacement);
                  end
               else if HasFarCall then
                  begin
                  Code2($F401 or (2 shl 3)); //brne
                  Code2($940C);
                  InsertWord(Numeric shr 1); //jmp
                  end
               else
                  begin
                  Code2($F401 or (3 shl 3)); //brne
                  Code2($E000 or RegConstant(30, lo(Numeric shr 1)));
                  Code2($E000 or RegConstant(31, hi(Numeric shr 1)));
                  RegisterTrack(30);
                  RegisterTrack(31);
                  Code2($9409); //IJmp
                  end;
               end
            else
               begin
               Code2($F401 or (1 shl 3)); //brne
               if RelativeRange2K(Dollar+4, Numeric) then
                  Code2($C000 or BranchDisplacement); //rjmp
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure M_JMPNE;

   begin
   if GetExpressionNoComma then
      begin
      if ExpressionType = ExExpression then
         begin
         if RelativeRange(Dollar+2, Numeric) then
            begin
            Code2($F401 or (BranchDisplacement shl 3)); //brne
            end
         else
            begin
            if (Numeric > Dollar+4+4094) or (Numeric < Dollar+4-4096) then
               begin
               if ROM4K then
                  begin
                  if RelativeRange2K(Dollar+2, Numeric) then
                     begin
                     Code2($C000 or BranchDisplacement);
                     end;
                  end
               else if HasFarCall then
                  begin
                  Code2($F001 or (2 shl 3)); //breq
                  Code2($940C);
                  InsertWord(Numeric shr 1); //jmp
                  end
               else
                  begin
                  Code2($F001 or (3 shl 3)); //breq
                  Code2($E000 or RegConstant(30, lo(Numeric shr 1)));
                  Code2($E000 or RegConstant(31, hi(Numeric shr 1)));
                  Code2($920F or (30 shl 4));
                  Code2($920F or (31 shl 4));
                  RegisterTrack(30);
                  RegisterTrack(31);
                  Code2($9409); //IJmp
                  end;
               end
            else
               begin
               Code2($F001 or (1 shl 3)); //breq
               if RelativeRange2K(Dollar+4, Numeric) then
                  begin
                  Code2($C000 or BranchDisplacement); //rjmp
                  end;
               end;
            end;
         end;
      end;
   end;

procedure M_JMPCS;
begin
  if GetExpressionNoComma then
  begin
    if ExpressionType=ExExpression then
    begin
      if RelativeRange(dollar+2,Numeric) then
      begin
        Code2($F000 or (BranchDisplacement shl 3)); //brcc
      end else
      begin
        if (Numeric>Dollar+4+4094) or (Numeric<Dollar+4-4096) then
        begin
          if ROM4K then
          begin
            if RelativeRange2K(dollar+2,Numeric) then
            begin
              Code2($C000 or BranchDisplacement);
            end;
          end else
          if HasFarCall then
          begin
            Code2($F400 or (2 shl 3)); //brcs
            Code2($940C);
            InsertWord(Numeric shr 1); //jmp
          end else
          begin
            Code2($F400 or (3 shl 3)); //brcs
            Code2($E000 or RegConstant(30,lo(Numeric shr 1)));
            Code2($E000 or RegConstant(31,hi(Numeric shr 1)));
            RegisterTrack(30);
            RegisterTrack(31);
            Code2($9409); //IJmp
          end;
        end else
        begin
          Code2($F400 or (1 shl 3)); //brcs
          if RelativeRange2K(dollar+4,Numeric) then
          begin
            Code2($C000 or BranchDisplacement); //rjmp
          end;
        end;
      end;
    end;
  end;
end;

{==============================================================================}

procedure M_JMPCC;

   begin
   if GetExpressionNoComma then
      begin
      if ExpressionType = ExExpression then
         begin
         if RelativeRange(Dollar+2, Numeric) then
            Code2($F400 or (BranchDisplacement shl 3)) //brcs
         else
            begin
            if (Numeric > Dollar+4+4094) or (Numeric < Dollar+4-4096) then
               begin
               if ROM4K then
                  begin
                  if RelativeRange2K(dollar+2,Numeric) then
                     Code2($C000 or BranchDisplacement);
                  end
               else if HasFarCall then
                  begin
                  Code2($F000 or (2 shl 3)); //brcc
                  Code2($940C);
                  InsertWord(Numeric shr 1); //jmp
                  end
               else
                  begin
                  Code2($F000 or (3 shl 3)); //brcc
                  Code2($E000 or RegConstant(30, lo(Numeric shr 1)));
                  Code2($E000 or RegConstant(31, hi(Numeric shr 1)));
                  RegisterTrack(30);
                  RegisterTrack(31);
                  Code2($9409); //IJmp
                  end;
               end
            else
               begin
               Code2($F000 or (1 shl 3)); //brcc
               if RelativeRange2K(Dollar+4, Numeric) then
                  Code2($C000 or BranchDisplacement); //rjmp
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure M_PUSH;

   var
      X,N: cardinal;                                                                 // longint
      S: string;                                                                     //ss short
      I: integer;

   begin
   if ParseItem(spSpaceIsTerminator) and not Comma then
      begin
      if FinalPass then
         begin
         S := 'Interrupt_Entry at line ' + IntToStr(SourceLineNumber+1) + ', File ' + ExtractFileName(SourceFiles[SourceFileNumber-1]);
         InfoList.Add(S);
         S := 'Saved ';
         end;

      with RegsUsed do
         begin
         if CItem[1] <> '#' then
            CItem := {#1} '!!' + char(LinkFileNumber+64) + CItem;
         I := IndexOf(CItem);
         if I >- 1 then
            begin
            X := cardinal(Objects[I]);                                                 // longint
            for N := 0 to 31 do
               begin
               if LongMask[N] and X <> 0 then
                  begin
                  if FinalPass then
                     begin
                     S := S + 'R' + IntToStr(N) + ' ';
                     end;
                  Code2($920F or (n shl 4));
                  end;
               end;

            if FinalPass then
               InfoList.Add(S);
            end;
         end;
      InstructionError:=false;
      end;
   end;

{==============================================================================}

procedure M_POP;

   var
      X,N: longint;                                                                 // longint
      S: string;                                                                     //ss short
      I:  integer;

   begin
   if ParseItem(spSpaceIsTerminator) and not Comma then
      begin
      if FinalPass then
         begin
         S := 'Interrupt_Exit at line '+IntToStr(SourceLineNumber+1)+', File '+
           ExtractFileName(SourceFiles[SourceFileNumber-1]);
         InfoList.Add(s);
         S:='Restored ';
         end;

      with RegsUsed do
         begin
         if CItem[1] <> '#' then
            CItem:= {#1} '!!' +char(LinkFileNumber+64)+CItem;
         I := IndexOf(CItem);
         if I > -1 then
            begin
            X := cardinal(Objects[I]);                                                 // longint
            for N := 31 downto 0 do
               begin
               if LongMask[N] and X <> 0 then
                  begin
                  if FinalPass then
                     begin
                     S := S + 'R' + IntToStr(N) + ' ';
                     end;
                  Code2($900F or (n shl 4));
                  end;
               end;

            if FinalPass then
               InfoList.Add(S);
            end;
         end;
      InstructionError := false;
      end;
   end;

procedure D_LLChar;
begin
  if ParseItem(spSpaceIsTerminator) and not Comma then
  begin
    if length(CItem)=1 then
    begin
      if CharInSet (CItem[1], ['?','!','@','#','%','^','&','[',']','{','}','=','_','\','~']) then
      begin
        LLChar:=CItem[1];
        InstructionError:=false;
      end;
    end else MakeError(0);
  end else MakeError(0);
end;

procedure D_DEFREG;
var
  reg: byte;
begin
  If GetExpressionComma then
  begin
    If IsRegister then
    begin
      reg:=RegisterCode;
      if ParseItem(spSpaceIsTerminator) and not comma then
      begin
        if length(CItem)<3 then MakeError(28) else
        begin
          RegisterSubs[reg] := CItem;
          InstructionError:=false;
        end;
      end;
    end;
  end;
end;

procedure D_MemErr;
var
  num: longint;                                                                 // longint
begin
  if getExpressionComma then
  begin
    if ExpressionType=ExExpression then
    begin
      num:=Numeric;
      If ParseItem(spSpaceIsTerminator) and not comma then
      begin
        InstructionError:=false;
        if FinalPass then
        if num<assemblyPointer then
        begin
          ErrorString:='MemErr: '+CItem;
          MakeError(100);
        end;
      end;
    end;
  end;
end;

procedure D_Report;
var
  num: longint;                                                                 // longint
begin
  if GetExpressionComma then
  begin
    if ExpressionType=ExExpression then
    begin
      num:=Numeric;
      If ParseItem(spSpaceIsTerminator) and not comma then
      begin
        InstructionError:=false;
        if FinalPass then
          Status(CItem+':'+IntToStr(num)+'/'+IntToHex(num,4)+'H');
      end;
    end;
  end;
end;

{==============================================================================}

procedure A_SPM;

   begin
   Code2($95E8);
   end;

{==============================================================================}

procedure A_MULS;

   var  Reg: word;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         Reg := RegisterCode;
         RegisterTrack(0);
         RegisterTrack(1);
         if GetExpressionNoComma then
            begin
            if IsRegister then
               begin
//               if (Reg > 15) and (RegisterCode > 15) then
               if (Reg in [16..31]) and (RegisterCode in [16..31]) then
                  Code2($0200 or RegSplit44(Reg, RegisterCode));
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure A_MULSU;

   var  Reg: word;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         Reg := RegisterCode;
         RegisterTrack(0);
         RegisterTrack(1);
         if GetExpressionNoComma then
            begin
            if IsRegister then
               begin
//               if (Reg > 15) and (Reg < 24) and (RegisterCode>15) and (RegisterCode<24) then
               if (Reg in [16..23]) and (RegisterCode in [16..23]) then
                  Code2($0300 or RegSplit44(Reg, RegisterCode));
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure A_FMUL;

   var  Reg: word;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         Reg := RegisterCode;
         RegisterTrack(0);
         RegisterTrack(1);
         if GetExpressionNoComma then
            begin
            if IsRegister then
               begin
//               if (Reg>15) and (Reg<24) and (RegisterCode>15) and (RegisterCode<24) then
               if (Reg in [16..23]) and (RegisterCode in [16..23]) then
                  Code2($0308 or RegSplit44(Reg, RegisterCode));
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure A_FMULS;

   var  Reg: word;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         Reg := RegisterCode;
         RegisterTrack(0);
         RegisterTrack(1);
         if GetExpressionNoComma then
            begin
            if IsRegister then
               begin
//               if (Reg>15) and (Reg<24) and (RegisterCode>15) and (RegisterCode<24) then
               if (Reg in [16..23]) and (RegisterCode in [16..23]) then
                  Code2($0380 or RegSplit44(Reg, RegisterCode));
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure A_FMULSU;

   var  Reg: word;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         Reg := RegisterCode;
         RegisterTrack(0);
         RegisterTrack(1);
         if GetExpressionNoComma then
            begin
            if IsRegister then
               begin
               if (Reg in [16..23]) and (RegisterCode in [16..23]) then
//               if (Reg>15) and (Reg<24) and (RegisterCode>15) and (RegisterCode<24) then
               Code2($0388 or RegSplit44(Reg, RegisterCode));
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure A_MOVW;

   var  Reg: word;

   begin
   if GetExpressionComma then
      begin
      if IsRegister then
         begin
         Reg := RegisterCode;
         if GetExpressionNoComma then
            begin
            if IsRegister then
               begin
               RegisterTrack(Reg);
               if (Reg and 1 = 0) and (RegisterCode and 1 = 0) then
                  Code2($0100 or RegSplit44(Reg shr 1, RegisterCode shr 1));
               end;
            end;
         end;
      end;
   end;

{==============================================================================}

procedure M_LPM;

   var  Reg: word;

   begin
   if not ExtendedLPM then
      begin
      if GetExpressionComma then
         begin
         if IsRegister then
            begin
            Reg := RegisterCode;
            RegisterTrack(RegisterCode);
            if ParseItem(spSpaceIsTerminator) then
               begin
               if CItem = 'Z' then
                  begin
                  Code2($9004 or (Reg shl 4));
                  end
               else if CItem = 'Z+' then
                  begin
                  Code2($9005 or (Reg shl 4));
                  end;
               end;
            end;
         end
      else
         begin
         Code2($95C8);
         RegisterTrack(0);
         end;
      end
   else
      begin
      if GetExpressionComma then
         begin
         if IsRegister then
            begin
            Reg := RegisterCode;
            RegisterTrack(RegisterCode);
            if ParseItem(spSpaceIsTerminator) then
               begin
               if CItem='Z' then
                  begin
                  Code2($9006 or (Reg shl 4));
                  end
               else if CItem='Z+' then
                  begin
                  Code2($9007 or (Reg shl 4));
                  end;
               end;
            end;
         end
      else
         begin
         Code2($95D8);
         RegisterTrack(0);
         end;
      end;
   end;

procedure A_ELPM;
var
  reg: word;
begin
  if GetExpressionComma then
  begin
    if IsRegister then
    begin
      Reg:=RegisterCode; RegisterTrack(RegisterCode);
      if ParseItem(spSpaceIsTerminator) then
      begin
        if CItem='Z' then
        begin
          Code2($9006 or (Reg shl 4));
        end else
        if CItem='Z+' then
        begin
          Code2($9007 or (Reg shl 4));
        end;
      end;
    end;
  end else
  begin
    Code2($95D8);
    RegisterTrack(0);
  end;
end;

procedure D_COFF_FN;
var
  i: longint;                                                                   // longint
begin
  if GetExpressionComma then
  begin
    ProcedureStartLine:=Numeric;
    if ParseItem(spSpaceIsTerminator) then
    begin
      ProcedureName := CItem;
      InstructionError:=false;
      if FinalPass and WriteSymbolFile and COFFEnabled then
      begin
        I := MakeSymbol(ProcedureName, Dollar, 1, $24, 2, 1);
        CoffLineInfo.Line:=0;
        CoffLineInfo.Address:=i;
        i:=FileSize(CoffLineFile);
        BlockWrite(CoffLineFile,CoffLineInfo,6);
        MakeAux(ProcedureStartLine,i+$FFFFFF,$C,0,0,0);
        MakeSymbol('.bf', Dollar, 1, 0, $65, 1);
        MakeAux(ProcedureStartLine,0,0,0,0,0);
        MakeSymbol('.bb', Dollar, 1, 0, $64, 1);
        MakeAux(ProcedureStartLine,0,0,0,0,0);
      end;
    end;
  end
end;

procedure D_COFF_FE;
begin
  if GetExpressionNoComma then
  begin
    InstructionError:=false;
    if FinalPass and WriteSymbolFile and COFFEnabled then
    begin
      MakeSymbol('.eb', Dollar, 1, 0, $64, 1);
      MakeAux(Numeric,0,0,0,0,0);
      MakeSymbol('.ef', 0, 1, 0, $65, 1);
      MakeAux(Numeric,0,0,0,0,0);
    end;
  end;
end;

procedure D_COFF_VAR;
var
  v: longint;                                                                    // longint
  s: string;
begin
  if GetExpressionComma then
  begin
    InstructionError:=false;
    v:=Numeric;
    s:=CItem;
    if GetExpression then
    begin
      if FinalPass and WriteSymbolFile and COFFEnabled then
      begin
        MakeStaticSymbol(s,v,3,Numeric,3,0);
        //MakeAux(Numeric,0,0,0,0,0);
      end;
    end;
  end;
end;

procedure D_COFF_SM;
var
  i: longint;                                                                    // longint
begin
  if ParseItem(spSpaceIsTerminator) then
  begin
    if FinalPass and WriteSymbolFile and COFFEnabled then
    begin
      i := MakeSymbol(CItem, Dollar, 1, $24, 2, 1);
      CoffLineInfo.Line:=0;
      CoffLineInfo.Address:=i;
      i:=FileSize(CoffLineFile);
      BlockWrite(CoffLineFile,CoffLineInfo,6);
      MakeAux(ProcedureStartLine,i+$FFFFFF,$C,0,0,0);
      MakeSymbol('.bf', Dollar, 1, 0, $65, 1);
      MakeAux(ProcedureStartLine,0,0,0,0,0);
      MakeSymbol('.bb', Dollar, 1, 0, $64, 1);
      MakeAux(ProcedureStartLine,0,0,0,0,0);
      MakeSymbol('.eb', Dollar, 1, 0, $64, 1);
      MakeAux(ProcedureStartLine,0,0,0,0,0);
      MakeSymbol('.ef', 0, 1, 0, $65, 1);
      MakeAux(ProcedureStartLine,0,0,0,0,0);
    end;
    InstructionError:=false;
  end;
end;

procedure D_EXLPM;
begin
  if ParseItem(spSpaceIsTerminator) then
  begin
    InstructionError:=false;
    if CItem='-' then ExtendedLPM:=false else
    if CItem='+' then ExtendedLPM:=true else InstructionError:=true;
  end;
end;

procedure D_TRIM;
begin
  if GetExpressionNoComma then
  begin
    TRIMDown:=Numeric;
    if (TRIMDown>=0) and (TRIMDown<$20000) then InstructionError:=false;
  end;
end;

