unit RPNUtils;

interface

function EvaluateExpression(var V: longint; S: string): boolean;                 //ss  short

implementation

uses AsmDefs, SysUtils;

var
    RPN : RPN_Array;
    X : Var_Vector;
    Error_Express : string;                                                      //ss   short

(*------------------------------------ Make_RPN_Expression -------*)

procedure Make_RPN_Expression(var Express, Error_Express: string; var RPN: RPN_Array);   //ss  short

   var OK : boolean;
       I, Error_Location,
       Stack_Height: integer;

{==================}

      procedure Prepare_String(var S: string);                                   //ss  short
(* Procedure to remove blank characters and convert string into upper case *)
         var I: integer;
             Out_String: string;                                                 //ss  short

         begin
         Out_String := '';
         for I := 1 to length(S) do
            if S[I] <> ' ' then
               begin
               Out_String := Out_String + UpCase(S[I]);
               end; (* IF *)
         S := Out_String;
         end; (* Prepare_String *)


(*---------------------------- Make_RPN_Expression ------------------*)
(*------------------------------------------- CheckParn -------------*)
{===================}

      procedure CheckParn(var S: string; var Error_Location: integer);           //ss  short
(* Procedure to check the correct use of paranthesis *)
         var C : char;                                                           //ss ansichar
             Num_Open,
             Num_Close,
             I, L: integer;

(*---------------------------- Make_RPN_Expression ------------------*)
(*---------------------------------- CheckParn \ Check --------------*)

         function Check: boolean;
(* Logical function returning TRUE if parenthesis use is correct, *)
(* FALSE if otherwise.                                            *)

            begin
            Check := (Num_Close <= Num_Open)
            END; (* Check *)

(*--------------------------------------------------------------*)

      begin
      Num_Open := 0;
      Num_Close := 0;
      OK := true;
      I := 1;
      L := length(S);
      while (I <= L) and OK do
         begin
         C := S[I];
         if C = '(' then
             begin
             inc(Num_Open);
             OK := Check;
             end; (* IF *)

         if C = ')' then
            begin
            inc(Num_Close);
            OK := Check;
            end; (* IF *)

         inc(I);
         end; (* WHILE *)

      if Num_Open <> Num_Close then
         Error_Express := 'Unbalanced number of open/closed paranthesis'
      else
         Error_Express := '';
      end; (* CheckParn *)

(*---------------------------- Make_RPN_Expression ------------------*)
(*------------------------------------------ ScanCHS ---------------*)

      procedure ScanCHS(var S: string);                                          //ss  short
(* Procedure to scan for change-of-sign *)
         var I, L: integer;

         begin
         if S[1] = '-' then
            S := '0.' + S;
         I := 2;
         L := length(S);
         while I <= L do
            begin
            if S[I] = '(' then
               begin
               inc(I);
               if S[I] = '-' then
                  begin
                  insert ('0.', S, I);
                  I := I + 2;
                  L := Length(S);
                  end; (* IF *)
               end; (* IF *)
            inc(I);
            end; (* WHILE *)
         end; (* ScanCHS *)


(*------------------------ Make_RPN_Expression ------------------*)
(*---------------------------------- Convert_to_RPN -------------*)


      procedure Convert_to_RPN(var S: string; var RPN: RPN_Array;                //ss  short

         var Error_Location: INTEGER);
(* Procedure to convert an algebraic expression to one in RPN *)

(*------------------------ Make_RPN_Expression ------------------*)
(*--------------------------- Convert_to_RPN \ Strip_Paren ------*)

         procedure Strip_Paren(var Express: string; var First, Last: integer);   //ss   short

            begin
            while (Express[First] = '(') and (Express[Last] = ')') and (First < Last) do
               begin
               inc(First);
               dec(Last);
               end; (* WHILE *)
            end; (* Strip_Paren *)


(*------------------------ Make_RPN_Expression ----------------------*)
(*--------------------------- Convert_to_RPN \ Convert_Express ------*)

         procedure Convert_Express(var Express: string; First, Last: integer); forward;    //ss  short


(*-------------------------- Make_RPN_Expression ----------------------*)
(*--------------------------- Convert_to_RPN \ Convert_SubFactor ------*)

         procedure Convert_SubFactor(var Express: string; First, Last: integer);          //ss  short
(* Procedure to scan for lowest components of an expressions *)

{TYPE STRING3 = STRING[3];
     Func_List = ARRAY [1..MAX_EXPR_FUNC] OF STRING3;

CONST  FuncName : Func_List = (
                'ABS','SQR','LOG','EXP','SIN','COS','TAN','ATN');
}
            var I,O: integer;
                RealDummy: real;
                Expr_Var: string;                                                 // short
                Found: boolean;

            begin
    (* Check for sub-expressions *)
            I := 0;
            if (Express[First] = '(') and (Express[Last] = ')') then
               begin
               Strip_Paren(Express, First, Last);
               Convert_Express(Express, First, Last);
               end
    (* Check for possible mathematical function *)
            else if Express[Last] = ')' then
               begin
               Expr_Var := '';
               O := 4;
               Found := false;
               if copy(Express, First, 4) = '.LO(' then
                  begin
                  Found := true;
                  I := 10;
                  end
               else if copy(Express, First, 4) = '.HI(' then
                  begin
                  Found := true;
                  I := 11;
                  end
               else if copy(Express, First, 4) = '.B2(' then
                  begin
                  Found := true;
                  I := 12;
                  end
               else if copy(Express, First, 4) = '.B3(' then
                  begin
                  Found := true;
                  I := 13;
                  end
               else if copy(Express, First, 4)='.W0(' then
                  begin
                  Found := true;
                  I := 14;
                  end
               else if copy(Express, First, 4) = '.W1(' then
                  begin
                  Found := true;
                  I := 15;
                  end;
{        end else
        begin
          FOR I := First TO First + 2 DO
              Expr_Var := Expr_Var + EXPRESS[I];
          // Search for corresponding function
          I := 1;
          Found := FALSE;
          WHILE (I <= MAX_EXPR_FUNC) AND (NOT Found) DO
              IF FuncName[I] = Expr_Var THEN Found := TRUE
                                        ELSE INC(I);
          O:=4;
        end; }
               if Found then
                  begin
                  Convert_Express(EXPRESS, (First+O), (Last-1));
                  inc(Stack_Height);
                  RPN[STack_Height].Operatype := funct;
                  RPN[STack_Height].Index := I;
                  RPN[STack_Height].Value := 0;
                  end
               else
                  begin (* Wrong function name *)
                  Error_Location := First;
                  Error_Express := 'Wrong function name : ' + Expr_Var;
                  end; (* IF *)
               end
    (* Check for a numeric constant *)
            else
               begin
               Expr_Var := '';
               for I :=  First to Last do
                  Expr_Var := Expr_Var + Express[I];

               Val(Expr_Var, RealDummy, Error_Location);
               if Error_Location > 0 then
                  Error_Express := Expr_Var
               else
                  begin
                  inc(Stack_Height);
                  RPN[Stack_Height].Operatype := pushnum;
                  RPN[Stack_Height].Index := 0;
                  RPN[Stack_Height].Value := RealDummy;
                  end; (* IF *)
               end; (* IF *)
            end; (* Convert_SubFactor *)

(*----------------------- Make_RPN_Expression --------------------*)
(*----------------------- Convert_to_RPN \ Convert_Term ----------*)

         procedure Convert_Term(var Express: string; First, Last: integer);      //ss  short
(* Procedure to scan for factors of an expression *)

            var Expr_Ptr, Last_Ptr,
                Paren_Level, Copy_Last: integer;
                Pending_Mult_Op,
                Scan_Char : char;                                                //ss ansichar
                R: RPN_Rec;


(*----------------------- Make_RPN_Expression --------------------*)
(*------ Convert_to_RPN \ Convert_Term \ Insert_Mult_Op ----------*)

            procedure Insert_Mult_Op(C: char);                                   //ss ansichar
(* Procedure to insert '*' or '/' ot '%' in RPN stack *)

               begin
               R.Operatype := operation;
               R.Value := 0.0;
                  case C of
                  '*' : R.Index := 3;
                  '/' : R.Index := 4;
                  #$7 : R.Index := 6;
                  #$2 : R.Index := 9;
                  #$3 : R.Index := 8;
                  end; (* CASE *)
               inc(Stack_Height);
               RPN[Stack_Height] := R;
               end; (* Insert_Mult_Op *)

         var GO: boolean;

         begin
         Expr_Ptr := First;
         Last_Ptr := First;
         Paren_Level := 1;
         Pending_Mult_Op := ' ';
         while (Expr_Ptr < Last) and (Error_Location = 0) do
            begin
            GO := true;
            Scan_Char := Express[Expr_Ptr];
            if Scan_Char = '.' then
               begin
               if copy(Express, Expr_Ptr, 5) = '.MOD.' then
                  begin
                  delete(Express, Expr_ptr, 5);
                  insert(#$2, Express, Expr_ptr);
                  Scan_Char := #$7;
                  dec(Last, 4);
                  end
               else if copy(Express, Expr_Ptr, 5) = '.SHL.' then
                  begin
                  delete(Express, Expr_ptr, 5);
                  insert(#$2, Express, Expr_ptr);
                  Scan_Char := #$2;
                  dec(Last, 4);
                  end
               else if copy(Express, Expr_Ptr, 5) = '.SHR.' then
                  begin
                  delete(Express, Expr_ptr, 5);
                  insert(#$3, Express, Expr_ptr);
                  Scan_Char := #$3;
                  dec(Last, 4);
                  end
               else
                  GO := false;
               end;

            if Scan_Char = '(' then
               inc(Paren_Level)
            else if Scan_Char = ')' then
               dec(Paren_Level)
            else if (CharInSet (Scan_Char, ['*','/',#$7,#$2,#$3])) and Go and (Paren_Level = 1) then
               begin
               if Last_Ptr < Expr_Ptr then
                  begin
                  Copy_Last := Expr_Ptr - 1;
               //Convert_Factor(EXPRESS, Last_Ptr, Copy_Last);
                  Convert_SubFactor(Express, Last_Ptr, Copy_Last);
                  if Pending_Mult_Op <> ' ' then
                     Insert_Mult_Op(Pending_Mult_Op);
                  Pending_Mult_Op := Scan_Char;
                  Last_Ptr := Expr_Ptr + 1;
                  end
               else  (* Error *)
                  begin
                  Error_Location := Last_Ptr;
                  Error_Express := EXPRESS[Last_Ptr] + EXPRESS[Expr_Ptr]
                  end; (* IF *)
               end; (* IF *)

            inc(Expr_Ptr);
            end; (* WHILE *)

         if Error_Location = 0 then
            begin
//        Convert_Factor(EXPRESS, Last_Ptr, Expr_Ptr);
            Convert_SubFactor(EXPRESS, Last_Ptr, Expr_Ptr);
            if Pending_Mult_Op <> ' ' then
               Insert_Mult_Op(Pending_Mult_Op);
            end; (* IF *)
         end; (* Convert_Term *)


(*----------------------- Make_RPN_Expression -----------------------*)
(*--------------------------- Convert_to_RPN \ Convert_Express ------*)

         procedure Convert_Express;
(* Procedure to scan for terms in an expression *)

            var Expr_Ptr, Last_Ptr,
                Paren_Level, Copy_Last: integer;
                Pending_Add_Op, Scan_Char: char;                                 //ss  ansichar
                R : RPN_Rec;

(*----------------------- Make_RPN_Expression -----------------------*)
(*----------- Convert_to_RPN \ Convert_Express \ Insert_Add_Op ------*)

               procedure Insert_Add_Op(C : char);                                //ss  ansichar
(* Procedure to insert '+' or '-' in RPN stack *)
                  begin
                  R.Operatype := operation;
                  R.Value := 0.0;
                     case C of
                     '-' : R.Index := 1;
                     '+' : R.Index := 2;
                     #$4 : R.Index := 12;
                     #$5 : R.Index := 13;
                     #$6 : R.Index := 14;
                     end; (* CASE *)

                  inc(Stack_Height);
                  RPN[Stack_Height] := R;
                  end; (* Insert_Add_Op *)

            begin
            Expr_Ptr := First;
            Last_Ptr := First;
            Paren_Level := 1;
            Pending_Add_Op := ' ';
            while (Expr_Ptr < Last) and (Error_Location = 0) do
               begin
               Scan_Char := Express[Expr_Ptr];
               if Scan_Char = '.' then
                  begin
                  if copy(Express, Expr_Ptr, 5) = '.AND.' then
                     begin
                     delete(Express, Expr_ptr, 5);
                     insert(#$4, Express, Expr_ptr);
                     Scan_Char := #$4;
                     dec(Last, 4);
                     end
                  else if copy(Express, Expr_Ptr, 4) = '.OR.' then
                     begin
                     delete(Express, Expr_ptr, 4);
                     insert(#$5, Express, Expr_ptr);
                     Scan_Char:=#$5;
                     dec(Last, 3);
                     end
                  else if copy(Express, Expr_Ptr, 5) = '.XOR.' then
                     begin
                     delete(Express, Expr_ptr, 5);
                     insert(#$6, Express, Expr_ptr);
                     Scan_Char := #$6;
                     dec(Last, 4);
                     end;
                  end;

               if Scan_Char = '(' then
                  inc(Paren_Level)
               else if Scan_Char = ')' then
                  dec(Paren_Level)
               else if (CharInSet (Scan_Char, ['+','-',#$4,#$5,#$6])) and (Paren_Level = 1) then
                  begin
                  if Last_Ptr < Expr_Ptr then
                     begin
                     Copy_Last := Expr_Ptr - 1;
                     Convert_Term(EXPRESS, Last_Ptr, Copy_Last);
                     if Pending_Add_Op <> ' ' then
                        Insert_Add_Op(Pending_Add_Op);
                     Pending_Add_Op := Scan_Char;
                     Last_Ptr := Expr_Ptr+1;
                     end
                  else (* Error *)
                     begin
                     Error_Location := Last_Ptr;
                     Error_Express := EXPRESS[Last_Ptr] + EXPRESS[Expr_Ptr]
                     end; (* IF *)
                  end; (* IF *)

               inc(Expr_Ptr);
               end; (* WHILE *)

            if Error_Location = 0 then
               begin
               Convert_Term(EXPRESS, Last_Ptr, Expr_Ptr);
               if Pending_Add_Op <> ' ' then
                  Insert_Add_Op(Pending_Add_Op);
               end; (* IF *)
            end; (* Convert_Express *)

         begin (* Convert_to_RPN *)
         Error_Location := 0;
         Error_Express := '';
         Convert_Express(Express, 1, Length(Express));
         end; (* Convert_to_RPN *)

   begin (* Make_RPN_Expression *)
    (* Initialize stack by filling it with End-of-Stack *)
   for I := 1 TO MAX_EXPR_STACK do
      RPN[I].Operatype := EOS;
   Stack_Height := 0;
   Prepare_String(Express);
   CheckParn(Express, Error_Location);
   if length(Error_Express) = 0 then
      begin
      ScanCHS(EXPRESS);
      Convert_to_RPN(EXPRESS, RPN, Error_Location);
      end; (* IF *)
   end; (* Make_RPN_Expression *)


(*---------------------------- Evaluate_RPN_Expression -----------*)

function Evaluate_RPN_Expression(var RPN: RPN_Array; var X: Var_Vector): real;
(* Function to evaluate RPN expression *)

   var I, Stack_Ptr: integer;
       Stack: array [1..MAX_EXPR_STACK] of real;

(*---------------------------- Evaluate_RPN_Expression \ Do_Op -----*)

      procedure Do_Op;
(* Procedure to execute the basic mathematical operations using the *)
(* RPN stack.                                                       *)

         begin
            case RPN[I].Index of
            1 : Stack[Stack_Ptr-1] := Stack[Stack_Ptr-1] - Stack[Stack_Ptr];
            2 : Stack[Stack_Ptr-1] := Stack[Stack_Ptr-1] + Stack[Stack_Ptr];
            3 : Stack[Stack_Ptr-1] := Stack[Stack_Ptr-1] * Stack[Stack_Ptr];
            4 : Stack[Stack_Ptr-1] := Stack[Stack_Ptr-1] / Stack[Stack_Ptr];
            5 : Stack[Stack_Ptr]   := -Stack[Stack_Ptr];
            6 : Stack[Stack_Ptr-1] := trunc(Stack[Stack_Ptr-1]) mod trunc(Stack[Stack_Ptr]);
            {7 : Stack[Stack_Ptr-1] := exp(Stack[Stack_Ptr] *
                                  Ln(Stack[Stack_Ptr-1]));}
            8 : Stack[Stack_Ptr-1] := trunc(Stack[Stack_Ptr-1]) shr trunc(Stack[Stack_Ptr]);
            9 : Stack[Stack_Ptr-1] := trunc(Stack[Stack_Ptr-1]) shl trunc(Stack[Stack_Ptr]);
            12 : Stack[Stack_Ptr-1] := trunc(Stack[Stack_Ptr-1]) and trunc(Stack[Stack_Ptr]);
            13 : Stack[Stack_Ptr-1] := trunc(Stack[Stack_Ptr-1]) or trunc(Stack[Stack_Ptr]);
            14 : Stack[Stack_Ptr-1] := trunc(Stack[Stack_Ptr-1]) xor trunc(Stack[Stack_Ptr]);
            end; (* CASE *)

         if not (RPN[I].Index in [5,10,11]) then
            dec(Stack_Ptr);
         end; (* Do_Op *)


(*--------------------- Evaluate_RPN_Expression \ Calc_Funct -----*)

      procedure Calc_Funct;
(* Procedure to evaluate functions *)

         begin
            case RPN[I].Index of
            {1 : Stack[Stack_Ptr] := ABS(Stack[Stack_Ptr]);
             2 : Stack[Stack_Ptr] := SQRT(Stack[Stack_Ptr]);
             3 : Stack[Stack_Ptr] := Ln(Stack[Stack_Ptr]);
             4 : Stack[Stack_Ptr] := EXP(Stack[Stack_Ptr]);
             5 : Stack[Stack_Ptr] := SIN(Stack[Stack_Ptr]);
             6 : Stack[Stack_Ptr] := COS(Stack[Stack_Ptr]);
             7 : Stack[Stack_Ptr] := SIN(Stack[Stack_Ptr])
                                / COS(Stack[Stack_Ptr]);
             8 : Stack[Stack_Ptr] := ArcTan(Stack[Stack_Ptr]);}
            10 : Stack[Stack_Ptr] := lo(trunc(Stack[Stack_Ptr]));
            11 : Stack[Stack_Ptr] := hi(trunc(Stack[Stack_Ptr]));
            12 : Stack[Stack_Ptr] := (trunc(Stack[Stack_Ptr]) shr 16) and $FF;
            13 : Stack[Stack_Ptr] := (trunc(Stack[Stack_Ptr]) shr 24) and $FF;
            14 : Stack[Stack_Ptr] := (trunc(Stack[Stack_Ptr]) and $FFFF);
            15 : Stack[Stack_Ptr] := (trunc(Stack[Stack_Ptr]) shr 16) and $FFFF;
            end;
         end; (* Calc_Funct *)

(**--------------------------------------------------------------**)

   begin
   (* Initialize stack *)
   for I := 1 to MAX_EXPR_STACK do
      Stack[I] := 0.0;
   I := 1;
   Stack_Ptr := 0;
   while (RPN[I].Operatype <> EOS) do
      begin
         case RPN[I].Operatype of
         operation: Do_Op;
         funct: Calc_Funct;
         pushvar:
            begin
            inc(Stack_Ptr);
            Stack[Stack_Ptr] := X[RPN[I].Index];
            end;

         pushnum:
            begin
            inc(Stack_Ptr);
            Stack[Stack_Ptr] := RPN[I].Value;
            end;

         EOS: WRITELN;
         end;
      inc(I);
      end; (* CASE *)

   Evaluate_RPN_Expression := Stack[1];
   end; (* Evaluate_RPN_Expression *)

(*----------------------------------------------------------------*)
{==============================================================================}

function EvaluateExpression(var V: longint; S: string): boolean;

   begin
      if (S = '') and (V = 0) then
         begin
         result := false;
         exit;
         end;

      Result := true;
      try
      V := 0;
      Error_Express := '';
      Make_RPN_Expression(S, Error_Express, RPN);
      if Length(Error_Express) > 0 then
         begin
         ErrorString := 'Error at ' + Error_Express;
         Result := false;
         exit;
         end;

      V := trunc(Evaluate_RPN_Expression(RPN, X));
      except
         V := 0;
      end;
   end;

end.
